/* $Header: complex.h,v 1.5 87/04/07 13:36:53 julian Exp $ */
#ifndef COMPLEX_H
#define COMPLEX_H

typedef struct {
	float	re;
	float	im;
} COMPLEX;

typedef struct {
	double	re;
	double	im;
} DCOMPLEX;

#define REAL(z)	((z).re)
#define IMAG(z)	((z).im)

#define CADD(a, b, sum) { \
	REAL(sum) = REAL(a) + REAL(b); \
	IMAG(sum) = IMAG(a) + IMAG(b); \
}
#define CSUB(a, b, sum) { \
	REAL(sum) = REAL(a) - REAL(b); \
	IMAG(sum) = IMAG(a) - IMAG(b); \
}
#define CMULT(a, b, prod) { \
	double ra=REAL(a), rb=REAL(b); \
	REAL(prod) = ra*rb - IMAG(a)*IMAG(b); \
	IMAG(prod) = ra*IMAG(b) + IMAG(a)*rb; \
}
#define HMULT(a, b, prod) { \
	double ra=REAL(a), rb=REAL(b); \
	REAL(prod) = ra*rb + IMAG(a)*IMAG(b); \
	IMAG(prod) = ra*IMAG(b) - IMAG(a)*rb; \
}

#define ABS_(x)		(x > 0.0 ? (x) : -(x))
/* See Knuth, Seminumerical Algorithms, 1st Ed., Prob. 16, p. 195 */
#define CDIV(a, b, quot) { \
	double r, s; \
	double ra=REAL(a), rb=REAL(b); \
	if (ABS_(rb) >= ABS_(IMAG(b))) { \
		r = IMAG(b)/rb; \
		s = 1./(rb + r*IMAG(b)); \
		REAL(quot) = (ra + r*IMAG(a))*s; \
		IMAG(quot) = (IMAG(a) - r*ra)*s; \
	} else { \
		r = rb/IMAG(b); \
		s = 1./(IMAG(b) + r*rb); \
		REAL(quot) = (r*ra + IMAG(a))*s; \
		IMAG(quot) = (r*IMAG(a) - ra)*s; \
	} \
}

#define CINV(c, result) { \
	double r, s; \
	double rc = REAL(c); \
	if (ABS_(rc) >= ABS_(IMAG(c))) { \
		r = IMAG(c)/rc; \
		s = 1./(rc + r*IMAG(c)); \
		REAL(result) = s; \
		IMAG(result) = -(r*s); \
	} else { \
		r = rc/IMAG(c); \
		s = 1./(IMAG(c) + r*rc); \
		REAL(result) = r*s; \
		IMAG(result) = (-s); \
	} \
}

#define SMULT(a, c, prod) { \
	REAL(prod) = a*REAL(c); \
	IMAG(prod) = a*IMAG(c); \
}

#define CABS(z) hypot(REAL(z), IMAG(z))
#define CARG(z) atan2(IMAG(z), REAL(z))

/* Dwight, Tables of Integrals and Other Mathematical Data, formula 58.2 */
#define CSQRT(c, result) { \
	double	r = CABS(c); \
	REAL(result) = sqrt(0.5*(r + REAL(c))); \
	IMAG(result) = sqrt(0.5*(r - REAL(c))); \
	if (IMAG(c) < 0.) IMAG(result) *= -1.; \
}

#define CEXP(c, result) { \
	double	r = exp(REAL(c)); \
	REAL(result) = r*cos(IMAG(c)); \
	IMAG(result) = r*sin(IMAG(c)); \
}
#endif COMPLEX_H
