/*
 * Dynamic Pointer Arrays
 * Manage allocated arrays of pointers to any types of objects
 */
#ifndef DPA_H
#define DPA_H

char	*emalloc(), *realloc();

#ifndef CHUNKSIZE
#define CHUNKSIZE	32	/* No. of pointers allocated at a time	*/
#endif CHUNKSIZE

/* Declare a dynamic pointer array */
#define DPA_DCL(type, name) type **name

/* Initialize a dynamic pointer array */
#define DPA_INIT(type, name, size, n) { \
		n = 0; \
		size = CHUNKSIZE; \
		name = (type **)emalloc((unsigned)size*sizeof(type*)); \
	}
/* Add one element to a dynamic pointer array */
#define DPA_ADD(type, name, size, n) { \
		if (n >= size) { \
			size += CHUNKSIZE; \
			name = (type**)realloc((char*)name, (unsigned)size*sizeof(type*)); \
			if (name == NULL) error("Can't realloc dpa type"); \
		} \
		(name)[n++] = (type*)emalloc((unsigned)sizeof(type)); \
	}
/* Free a dynamic pointer array */
#define DPA_FREE(name, n) { \
		int i; \
		for (i=0; i<n; i++) free((char*)(name)[i]); \
		free((char*)name); \
	}
#endif DPA_H
