/*                  Q U A K E . H
 * Bruce R. Julian, USGS Menlo Park Calif., Dec 15 1983
 */
#ifndef QUAKE_H
#define QUAKE_H

#include "stdtyp.h"
#include "date_time.h"

#ifndef NULL
#define NULL 0
#endif

#define repeat do
#define until(c) while(!(c))

#define FOREVER for(;;)				/* Infinite loop	*/
#define NOT !
#define YES 1					/* Truth value		*/
#define NO 0					/* Truth value		*/
#define TRUE 1					/* Truth value		*/
#define FALSE 0					/* Truth value		*/
#define DIM(x) (sizeof(x)/sizeof((x)[0]))	/* Dimension of array	*/
#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))	/* Larger value		*/
#endif MAX
#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))	/* Smaller value	*/
#endif MIN
#define ABS(a) ((a) > 0 ? (a) : -(a))		/* Absolute value	*/
#define FAIL -1					/* Function return	*/
#define SUCCEED 0				/* Function return	*/
#define STDIN 0					/* read() write() unit	*/
#define STDOUT 1				/* read() write() unit	*/
#define STDERR 2				/* read() write() unit	*/
#define READ 0					/* Mode for open()	*/
#define WRITE 1					/* Mode for open()	*/
#define UPDATE 2				/* Mode for open()	*/

/*---------------------- E R R O R   C O D E S -------------------------*/
#define ERRBASE		1000
#define EBADDAY		(ERRBASE + 1)	/* Illegal calendar day		*/
#define EBADMONTH	(ERRBASE + 2)	/* Illegal calendar month	*/
#define EBADYEAR	(ERRBASE + 3)	/* Illegal calendar year	*/
#define EBADVALUE	(ERRBASE + 4)	/* Illegal value		*/
#define EFEWDATA	(ERRBASE + 5)	/* Not enough data		*/
#define ENOCONVRG	(ERRBASE + 6)	/* Failed to converge		*/
#define EUNDERFLOW	(ERRBASE + 7)	/* Arithmetic underflow		*/
#define ENOSTATION	(ERRBASE + 8)	/* Unknown seismograph station	*/
#define ESIZE		(ERRBASE + 9)
#define ESINGULAR	(ERRBASE + 10)	/* Matrix is singular		*/
#define ETOODEEP	(ERRBASE + 11)	/* Focal depth too great	*/
#define EBADPHASE	(ERRBASE + 12)	/* Unknown seismic phase code	*/

/*------------- E V E N T   O R I G I N   S T R U C T U R E ------------*/
typedef struct {
	TIME	o_time;		/* Origin time				*/
	float	o_lat;		/* Epicentral latitude (Rad)		*/
	float	o_lon;		/* Epicentral longitude (Rad)		*/
	float	o_depth;	/* Focal depth				*/
} ORIGIN;

/*------------------ S T A T I O N   S T R U C T U R E -----------------*/
#define MXSTCOD	4
typedef struct {
	char	s_code[MXSTCOD];/* Station code				*/
	float	gdc[3];		/* Geocentric direction cosines		*/
	float	s_elev;		/* Station elevation (m)		*/
} STATION;

/*------------------ R E A D I N G   S T R U C T U R E -----------------*/
#define CHANSIZE	6
#define MXPHCOD		8
/* Masks for flags: */
#define ASSOCIATED	01	/* Phase comes from current event	*/
#define T_EXISTS	02	/* Arrival time exists			*/
#define USE_T		04	/* Should use arrival time		*/
#ifdef SLO
#define S_EXISTS	010	/* Slowness-azimuth exists		*/
#define USE_S		020	/* Should use slowness-azimuth		*/
#endif
typedef struct {
	STATION	sta;		/* Station structure			*/
	char	chan[CHANSIZE];	/* Channel code				*/
	char	phcode[MXPHCOD];/* Phase code				*/
	TIME	atime;		/* Arrival time				*/
	float	setime;		/* Std. err. of arrival time (s) 	*/
	char	onset;		/* 'e' or 'i'				*/
	char	pol;		/* Polarity: e.g. 'u' or 'd'		*/
	char	qual;		/* Quality code				*/
	float	amp;
	float	freq;
#ifdef SLO
	float	slowness;	/* Slowness (s/Rad)			*/
	float	azimuth;	/* Propagation azimuth (Rad)		*/
	float	seslow;		/* Std. err. of slowness (s/Rad)	*/
#endif
	char	flags;
} READING;

#ifdef SLO
/* Array subscripts for slowness-vector components */
#define	EAST	0
#define	NORTH	1
#endif

/*-------------- A S S O C I A T I O N  S T R U C T U R E --------------*/
typedef struct {
	float	delta;		/* Epicentral distance (Rad)		*/
	float	aze;		/* Epicenter -> station azimuth (Rad)	*/
	float	azs;		/* Station -> epicenter azimuth (Rad)	*/
	int	brnum;		/* Branch number			*/
	int	ercode;		/* From body-wave subroutine		*/
	float	atresid;	/* Arrival-time residual		*/
	float	atsigma;	/* Arrival-time std. err.		*/
	float	atwt;		/* Arrival-time weight			*/
	float	atimp;		/* Arrival-time importance		*/
#ifdef SLO
	float	szresid[2];	/* slowness-azimuth residual (s/Rad)	*/
	float	szsigma;	/* Std. err. of slowness-azimuth (s/Rad)*/
	float	szwt;		/* Weight of slowness-azimuth obs.	*/
	float	szimp;		/* Importance of slowness-azimuth obs.	*/
#endif
} ASSOC;

/*------------ R E L O C   C O N T R O L   S T R U C T U R E -----------*/
typedef struct {
	tbool	hold[4];	/* Cordinate-restraint flags		*/
	float	epcvthresh;	/* Convergence threshold for epicenter	*/
	float	fdcvthresh;	/* Convergence threshold for focal depth*/
	float	otcvthresh;	/* Convergence threshold for origin time*/
	float	min_depth;	/* Smallest allowed focal depth		*/
	float	max_depth;	/* Largest allowed focal depth		*/
	int	max_iter;	/* Iteration limit			*/
	int	max_damp;	/* Damping-step limit			*/
	float	lstart;		/* Initial damping parameter		*/
	float	lmin;		/* Minimum damping parameter		*/
	float	lfact;		/* Change in lambda per step		*/
	float	zdamp;		/* Relative depth damping		*/
	float	mu;		/* Frequency of erratic data		*/
	int	iprint;		/* Print-control flag			*/
} RLCTRL;

/* Subscripts for origin components */
#define	ILAT	0
#define	ILON	1
#define	IDEPTH	2
#define	ITIME	3

/* Print control */
#define NEVER	0
#define FINAL	1
#define ALL	2

/*-------- O R I G I N   S T A T I S T I C S   S T R U C T U R E -------*/
typedef struct {
	float	chisq;		/* Chi squared				*/
	float	sumwt;		/* Sum of weights of readings		*/
	int	ndf;		/* No. of degrees of freedom		*/
	int	iter;		/* No. of iterations performed		*/
	int	ntasoc;		/* No. of associated arrival-time obs.	*/
	int	ntused;		/* No. of arrival-time obs. used	*/
#ifdef SLO
	int	nsasoc;		/* No. of associated slowness obs.	*/
	int	nsused;		/* No. of slowness obs. used		*/
#endif
	tbool	converged;
	tbool	held[4];
} STATIS;

/*-------- B O D Y - W A V E   R E S U L T   S T R U C T U R E ---------*/
typedef struct {
	int	bw_brnum;	/* Branch number			*/
	float	bw_t;		/* Travel time				*/
	float	bw_set;		/* Std. err. of t			*/
	float	bw_td;		/* p = dt/ddelta			*/
	float	bw_setd;	/* Std. err. of td			*/
	float	bw_th;		/* dt/dh				*/
	float	bw_tdd;		/* dp/ddelta				*/
	float	bw_tdh;		/* dp/dh				*/
} BWRSLT;
#endif
