#ifndef lint
static char rcsid[] = "$Header: bcucof.c,v 1.4 89/02/07 13:37:58 julian Exp $";
#endif lint
#include <local/cuint.h>
/*
 * Generate coefficients for bicubic interpolation
 */
void
bcucof(f, f1, f2, f12, d1, d2, c)
	float	*f,		/* Array[1..4] of function values	*/
		*f1, *f2,	/* Array[1..4] of derivative values	*/
		*f12;		/* Array[1..4] of cross-derivatives	*/
	float	d1, d2;		/* Grid spacing in 1, 2 directions	*/
	float	**c;		/* Matrix[0..3][0..3] of coefficients 	*/
{
static	float	cfbot[4],
		cf2bot[4],
		cftop[4],
		cf2top[4];
static	int	i;
static	float	tmp;		/* Scratch				*/
static	float	*ci;		/* ith row of c[0..3][0..3]		*/

	/* Coefficients of 1-D cubic for f(x, 0) */
	cfbot[0] = f[1];
	cfbot[1] = f1[1]*d1;
	tmp = f[2] - f[1];
	cfbot[3] = -(tmp + tmp) + cfbot[1] + f1[2]*d1;
	cfbot[2] = -cfbot[3] + tmp - cfbot[1];

	cf2bot[0] = f2[1];
	cf2bot[1] = f12[1]*d1;
	tmp = f2[2] - f2[1];
	cf2bot[3] = -(tmp + tmp) + cf2bot[1] + f12[2]*d1;
	cf2bot[2] = -cf2bot[3] + tmp - cf2bot[1];

	/* Coefficients of 1-D cubic for f(x, 1) */
	cftop[0] = f[3];
	cftop[1] = f1[3]*d1;
	tmp = f[4] - f[3];
	cftop[3] = -(tmp + tmp) + cftop[1] + f1[4]*d1;
	cftop[2] = -cftop[3] + tmp - cftop[1];

	cf2top[0] = f2[3];
	cf2top[1] = f12[3]*d1;
	tmp = f2[4] - f2[3];
	cf2top[3] = -(tmp + tmp) + cf2top[1] + f12[4]*d1;
	cf2top[2] = -cf2top[3] + tmp - cf2top[1];

	/* Fit cubic in y to each x coefficient */
	for (i=0; i<4; i++) {
		ci = c[i];
		ci[0] = cfbot[i];
		ci[1] = cf2bot[i]*d2;
		tmp = cftop[i] - cfbot[i];
		ci[3] = -(tmp + tmp) + ci[1] + cf2top[i]*d2;
		ci[2] = -ci[3] + tmp - ci[1];
	}
}

#ifdef TEST
char	*progname = "test_bcucof";

float	**fmatrix();

main()
{
	float	**c;
	float	f[4], f1[4], f2[4], f12[4];
	int	i;

	c = fmatrix(0, 3, 0, 3);
	while (scanf("%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f",
		f  , f1  , f2  , f12  ,
		f+1, f1+1, f2+1, f12+1,
		f+2, f1+2, f2+2, f12+2,
		f+3, f1+3, f2+3, f12+3
	) == 16) {
		bcucof(f-1, f1-1, f2-1, f12-1, 1.0, 1.0, c);
		for (i=0; i<4; i++) {
			float	*r=c[i];
			printf("%g\t%g\t%g\t%g\n", r[0], r[1], r[2], r[3]);
		}
	}
}
#endif TEST
