#ifndef lint
static char rcsid[] = "$Header: born.c,v 1.1 88/10/17 13:57:57 julian Exp $";
#endif lint
/*
 * Scattering of elastic waves in random medium
 * First Born approximation
 */
#include <math.h>
#include <local/mathconst.h>
#include <local/born.h>

#define B(x)		mp->s2[x]*((*mp->bfunct)(mp->a[x], dk))
#define DOT(a,b)	(a[X]*b[X]+a[Y]*b[Y]+a[Z]*b[Z])

void	vset();

/* P->P scattering */
void
born_pp(mp, alpha, beta, amp, omega, cm)
	SCATMDL	*mp;
	float	alpha[3];	/* Incident-wave unit vector		*/
	float	beta[3];	/* Scattered-wave unit vector		*/
	double	amp;		/* Amplitude of incident wave		*/
	double	omega;		/* Frequency				*/
	float	cm[6];		/* Coherency matrix of scattered wave	*/
{
	double	ab;		/* alpha dot beta			*/
	double	ab2;		/* ab^2					*/
	float	dk[3];		/* Change in k vector upon scattering	*/
	int	i, j;		/* Loop index				*/
	int	ij;		/* subscript [i,j] in symm. storage mode*/
	double	kp;		/* P wavenumber				*/
	double	kp2;		/* kp^2					*/
	double	kp4;		/* kp^4					*/
	double	ms;		/* Mean-square ampl. of scattered wave	*/
	double	sum;
	double	w2;		/* omega^2				*/
	double	w4;		/* omega^4				*/

	kp = omega/mp->vp; kp2 = kp*kp; kp4 = kp2*kp2;
	w2 = omega*omega; w4 = w2*w2;
	ab = DOT(alpha, beta);
	ab2 = ab*ab;
	for (i=0; i<3; i++)
		dk[i] = kp*(alpha[i] - beta[i]);
	sum  =   kp4 * B(LAMLAM);
	sum +=   4.*kp4*(ab2*ab2) * B(MUMU);
	sum +=   w4*ab2 * B(RHORHO);
	sum +=   4.*kp4*ab2 * B(LAMMU);
	sum +=  -2.*w2*kp2*ab*(ab2-ab+1.) * B(LAMRHO);
	sum +=  -2.*w2*kp2*ab2*(ab+1.) * B(MURHO);
	ms = amp/(4.*PI*mp->rho*mp->vp*mp->vp);
	ms = sum*ms*ms;
	ij = 0;
	for (i=0; i<3; i++)
		for (j=0; j<=i; j++)
			cm[ij++] += ms*beta[i]*beta[j];
}

/* P->S scattering */
void
born_ps(mp, alpha, beta, amp, omega, cm)
	SCATMDL	*mp;
	float	alpha[3];	/* Incident-wave unit vector		*/
	float	beta[3];	/* Scattered-wave unit vector		*/
	double	amp;		/* Amplitude of incident wave		*/
	double	omega;		/* Frequency				*/
	float	cm[6];		/* Coherency matrix of scattered wave	*/
{
	double	ab;		/* alpha dot beta			*/
	double	ap[3];		/* Proj. of alpha normal to beta	*/
	double	sum;
	float	dk[3];		/* Change in k vector upon scattering	*/
	double	dm1;		/* Scratch variable			*/
	int	i, j;		/* Loop indices				*/
	int	ij;		/* Symmetric-storage-mode subscript	*/
	double	kp, ks;		/* P, S wavenumbers			*/
	double	ms;		/* Mean-square scattered amplitude	*/
	double	w2;		/* omega^2				*/

	kp = omega/mp->vp; ks = omega/mp->vs;
	ab = DOT(alpha, beta);
	for (i=0; i<3; i++)
		dk[i] = kp*alpha[i] - ks*beta[i];
	w2 = omega*omega;
	dm1 = kp*ks*ab;
	sum = w2*w2 * B(RHORHO) +
		4.*dm1*dm1 * B(MUMU) +
		4.*w2*dm1 * B(MURHO);
	ms = amp/(4.*PI*mp->rho*mp->vs*mp->vs);
	ms = sum*ms*ms;
	ij = 0;
	for (i=0; i<3; i++) {
		ap[i] = alpha[i] - ab*beta[i];
		for (j=0; j<=i; j++)
			cm[ij++] += ms*ap[i]*ap[j];
	}
}

/* S->P scattering */
void
born_sp(mp, alpha, beta, u, omega, cm)
	SCATMDL	*mp;
	float	alpha[3];	/* Incident-wave unit vector		*/
	float	beta[3];	/* Scattered-wave unit vector		*/
	float	u[3];		/* Displacement vector of incident wave	*/
	double	omega;		/* Frequency				*/
	float	cm[6];		/* Coherency matrix of scattered wave	*/
{
	int	i, j;		/* Loop indices				*/
	int	ij;		/* Symmetric-storage-mode subscript	*/
	double	ms;		/* Mean-square scattered amplitude	*/
#undef RECIPROCITY
#ifdef RECIPROCITY
	float	cmps[6];	/* Coherency matrix for P->S scattering	*/

	vset(cmps, 1, 6, 0.);
	ps(mp, beta, alpha, (mp->vs)/(mp->vp), omega, cmps);
	cmps[YX] *= 2.; cmps[ZX] *= 2.; cmps[ZY] *= 2.;
	ms = 0.;
	ij = 0;
	for (i=0; i<3; i++)
		for (j=0; j<=i; j++)
			ms += cmps[ij++]*u[i]*u[j];
#else RECIPROCITY
	double	sum;
	float	dk[3];		/* Change in k vector upon scattering	*/
	double	dm1;		/* Scratch variable			*/
	double	kp, ks;		/* P, S wavenumbers			*/
	double	w2;		/* omega^2				*/

	kp = omega/mp->vp; ks = omega/mp->vs;
	for (i=0; i<3; i++)
		dk[i] = ks*beta[i] - kp*alpha[i];
	w2 = omega*omega;
	dm1 = kp*ks*DOT(alpha, beta);
	sum = w2*w2 * B(RHORHO) +
		4.*dm1*dm1 * B(MUMU) +
		4.*w2*dm1 * B(MURHO);
	ms = DOT(beta, u)/(4.*PI*mp->rho*mp->vp*mp->vs);
	ms = sum*ms*ms;
#endif RECIPROCITY
	ij = 0;
	for (i=0; i<3; i++)
		for (j=0; j<=i; j++)
			cm[ij++] += ms*beta[i]*beta[j];
}

/* S->S scattering */
void
born_ss(mp, alpha, beta, u, omega, cm)
	SCATMDL	*mp;
	float	alpha[3];	/* Incident-wave unit vector		*/
	float	beta[3];	/* Scattered-wave unit vector		*/
	float	u[3];		/* Displacement vector of incident wave */
	double	omega;		/* Frequency				*/
	float	cm[6];		/* Coherency matrix of scattered wave	*/
{
	double	ab;		/* alpha dot beta			*/
	float	ap[3];		/* Proj. of alpha perpendicular to beta	*/
	double 	api, apj;	/* Temp for ap[i], ap[j]		*/
	double	bg;		/* beta dot u				*/
	double	bmm, bmr, brr;	/* B(MUMU), B(MURHO), B(RHORHO)		*/
	float	dk[3];		/* Change in k vector on scattering	*/
	double 	dq;		/* Accumulator for cm			*/
	double	fact;
	float	gp[3];		/* Proj. of u perpendicular to beta	*/
	double 	gpi, gpj;	/* Temp for gp[i], gp[j]		*/
	int	i;		/* Loop index				*/
	int	ij;		/* subscript [i,j] in symm. storage mode*/
	int	j;		/* Loop index				*/
	double	ks;		/* S wavenumber				*/
	double	ks2;		/* ks^2					*/
	double	sl;		/* Shear-wave slowness			*/
	double	sl2;		/* sl^2					*/

	sl = 1./mp->vs; sl2 = sl*sl;
	ks = omega*sl; ks2 = ks*ks;
	ab = DOT(alpha, beta);
	bg = DOT(beta, u);
	for (i=0; i<3; i++) {
		dk[i] = ks*(alpha[i] - beta[i]);
		ap[i] = alpha[i] - ab*beta[i];
		gp[i] = u[i] - bg*beta[i];
	}
	bmm = B(MUMU); bmr = B(MURHO); brr = B(RHORHO);
	ij = 0;
	fact = ks2/(4.*PI*mp->rho);
	fact *= fact;
	for (i=0; i<3; i++) {
		api = ap[i]; gpi = gp[i];
		for (j=0; j<=i; j++) {
			apj = ap[j]; gpj = gp[j];
			dq = gpi*gpj * brr;
			dq -= sl2*(2.*ab*gpi*gpj+bg*(api*gpj+gpi*apj)) * bmr;
			dq += sl2*sl2*(ab*gpi+bg*api)*(ab*gpj+bg*apj) * bmm;
			cm[ij++] += dq*fact;
		}
	}
}
