#ifndef lint
static char rcsid[] = "$Header$";
#endif lint
#include <math.h>
#include <local/local.h>
#include <local/qerr.h>

#ifdef DOUBLE
#define float double
#define fvector dvector
#define free_fvector free_dvector
#define fmatrix dmatrix
#define free_fmatrix free_dmatrix
#endif DOUBLE

#define IMAX	11
#define NUSE	7
#define SHRINK	0.95
#define GROW	1.2

float	**d = 0;		/* Used by rzextr() or pzextr()		*/
float	*x = 0;			/* Used by rzextr() or pzextr()		*/

	float	**fmatrix();
	void	free_fmatrix();
	float	*fvector();
	void	free_fvector();
	void	mmid();
static	void	rzextr();	/* Defined below			*/

/*
 * Integrate system of ordinary differential equations
 * Bulirsch-Stoer method
 */
int
bsstep(y, dydx, n, xx, htry, eps, yscal, hdid, hnext, derivs)
	float	y[];		/* [1..n] Dependent variables		*/
	float	dydx[];		/* [1..n] Derivatives			*/
	int	n;		/* No. of dependent variables		*/
	float	*xx;		/* Independent variable			*/
	float	htry;		/* Stepsize to attempt			*/
	float	eps;		/* Required accuracy			*/
	float	yscal[];	/* [1..n] Scaling for errors		*/
	float	*hdid;		/* Stepsize accomplished		*/
	float	*hnext;		/* Next estimated stepsize		*/
	void	(*derivs)();	/* User-supplied RHS function		*/
{
	float	*dysav;
	float	errmax;
extern	int	errno;
	float	h;
	int	i;		/* Loop index				*/
static	int	nseq[IMAX+1] =
		{ 0, 2, 4, 6, 8, 12, 16, 24, 32, 48, 64, 96 };
	int	j;		/* Loop index				*/
	float	temp;
	float	xest;
	float	xsav;
	float	*yerr;
	float	*ysav;
	float	*yseq;

	/* Allocate arrays */
	ysav = fvector(1, n);
	dysav = fvector(1, n);
	yseq = fvector(1, n);
	yerr = fvector(1, n);
	x = fvector(1, IMAX);
	d = fmatrix(1, n, 1, NUSE);

	h = htry;
	xsav = (*xx);

	/* Save the starting values */
	for (i=1; i<=n; i++) {
		ysav[i] = y[i];
		dysav[i] = dydx[i];
	}

	for (;;) {
		/* Evaluate sequence of modified midpoint integrations */
		for (i=1; i<=IMAX; i++) {
			mmid(ysav, dysav, n, xsav, h, nseq[i], yseq, derivs);
			xest = (temp=h/nseq[i], temp*temp);

			/* Rational function extrapolation */
			rzextr(i, xest, yseq, y, yerr, n, NUSE);

			/* Check local truncation error */
			errmax = 0.0;
			for (j=1; j<=n; j++)
				if (errmax < fabs(yerr[j]/yscal[j]))
					errmax = fabs(yerr[j]/yscal[j]);

			errmax /= eps;
			if (errmax < 1.0) {
				/* Step converged */
				*xx += h;
				*hdid = h;
				*hnext = (i == NUSE)	? h*SHRINK :
					 (i == NUSE-1)	? h*GROW :
							 (h*nseq[NUSE-1])/nseq[i];
				free_fmatrix(d, 1, n, 1, NUSE);
				free_fvector(x, 1, IMAX);
				free_fvector(yerr, 1, n);
				free_fvector(yseq, 1, n);
				free_fvector(dysav, 1, n);
				free_fvector(ysav, 1, n);
				return SUCCEED;
			}
		}

		/* Step failed:  reduce stepsize and try again */
		h *= 0.25;
		for (i=1; i<=(IMAX-NUSE)/2; i++)
			h *= 0.5;
		if (*xx+h == *xx) {
			errno = EUNDERFLOW;
			return FAIL;
		}
	}
}

/*
 * Rational function extrapolation.
 * Estimate value at x = 0 of n functions by fitting a
 * diagonal rational function to a sequence of estimates with
 * progressively smaller values x = xest and function values yest[1..n].
 */
static void
rzextr(iest, xest, yest, yz, dy, n, nuse)
	int	iest;		/* Number in sequence of calls		*/
	int	n;		/* No. of functions			*/
	int	nuse;		/* Max. no. of estimates to use		*/
	float	xest;		/* Independent variable			*/
	float	yest[];		/* [1..n] Estimated function values	*/
	float	yz[];		/* [1..n] Extrapolated function values	*/
	float	dy[];		/* [1..n] Estimated errors		*/
{
	float	*fx;
	float	b1;
	float	b;
	float	c;
	float	ddy;
	int	j;
	int	k;
	int	m1;
	float	v;
	float	yy;

	fx = fvector(1, nuse);
	x[iest] = xest;
	if (iest == 1)
		for (j=1; j<=n; j++)
			yz[j] = d[j][1] = dy[j] = yest[j];
	else {
		m1 = MIN(iest, nuse);
		for (k=1; k<=m1; k++)
			fx[k+1] = x[iest-k]/xest;

		/* Evaluate next diagonal in tableau */
		for (j=1; j<=n; j++) {
			yy = yest[j];
			v = d[j][1];
			d[j][1] = c = yy;
			for (k=2; k<=m1; k++) {
				b1 = fx[k]*v;
				b = b1 - c;
				if (b != 0.0) {
					b = (c - v)/b;
					ddy = c*b;
					c = b1*b;
				} else
					ddy = v;
				if (k != m1)
					v = d[j][k];
				yy += (d[j][k] = ddy);
			}
			dy[j] = ddy;
			yz[j] = yy;
		}
	}
	free_fvector(fx, 1, nuse);
}

#ifdef TEST
#include <stdio.h>

#define N	4
#define IMAX	11
#define NUSE	7

char	*progname = "test_bsstep";

void
derivs(x, y, dydx)
	float	x;
	float	y[];
	float	dydx[];
{
	dydx[1] = -y[2];
	dydx[2] = y[1] - (1.0/x)*y[2];
	dydx[3] = y[2] - (2.0/x)*y[3];
	dydx[4] = y[3] - (3.0/x)*y[4];
}

main()
{
	float	*dydx;
	float	eps;
	float	hdid;
	float	hnext;
	float	htry;
	int	i;
	float	x = 1.0;
	float	*y;
	float	*yscal;

	y = fvector(1, N);
	dydx = fvector(1, N);
	yscal = fvector(1, N);
	y[1] = j0(x);
	y[2] = j1(x);
	y[3] = jn(2, x);
	y[4] = jn(3, x);
	dydx[1] = -y[2];
	dydx[2] = y[1] -     y[2];
	dydx[3] = y[2] - 2.0*y[3];
	dydx[4] = y[3] - 3.0*y[4];
	for (i=1; i<=N; i++)
		yscal[i] = 1.0;
	htry = 1.0;
	printf("\n%10s %10s %12s %12s\n", "eps", "htry", "hdid", "hnext");
	for (i=1; i<=15; i++) {
		eps = exp((double) -i);
		bsstep(y, dydx, N, &x, htry, eps, yscal, &hdid, &hnext, derivs);
		printf(" %11e %8.2f %14.6f %12.6f\n", eps, htry, hdid, hnext);
	}
	free_fvector(yscal, 1, N);
	free_fvector(dydx, 1, N);
	free_fvector(y, 1, N);
}
#endif TEST
