#ifndef lint
static char rcsid[]="$Header: calendar.c,v 1.1 90/11/25 12:45:10 julian Exp $";
#endif
/*
 * Calendar computations (old and new-style)
 * Bruce R. Julian, USGS Menlo Park, CA
 */
#include <local/local.h>
#include <local/date_time.h>
#include <local/qerr.h>

/* floor(x/y), where x, y>0 are integers, using integer arithmetic */
#define qfloor(x, y) (x>0 ? (x)/y : -((y-1-(x))/y))

static int	eom[2][15] = {
	{ 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365, 396, 424 },
	{ 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366, 397, 425 },
};

/* Recognize leap years */
bool
isleap(yr, cal)
	int	yr;		/* Year					*/
	int	cal;		/* Calendar type			*/
{
	bool	l;

	if (yr < 0)
		yr++;
	l = (yr%4 == 0);
	if (cal == GREGORIAN)
		l = l && (yr%100 != 0 || yr%400 == 0);
	return l;
}

/* Check calendar date for legality */
int
ckdate(y, m, d, cal)
	int	y, m ,d;	/* Year, month, date			*/
	int	cal;		/* Calendar type			*/
{
	int	*et = eom[isleap(y, cal)];

	if (y == 0)			return EBADYEAR;
	if (m < 1 || m > 12)		return EBADMONTH;
	if (d < 1 || d > et[m]-et[m-1])	return EBADDAY;
	return  0;
}

/* Compute day of year */
int
yrday(mo, day, lp)
	int	mo, day;	/* Month, date				*/
	bool	lp;		/* Leap year?				*/
{
	return eom[lp][mo-1] + day;
}

/* Calculate month, date from day of year */
void
mnday(d, lp, pm, pd)
	int	d;		/* Day of year				*/
	bool	lp;		/* Leap year?				*/
	int	*pm, *pd;	/* Month, date				*/
{
	int	i;
	int	*et = eom[lp];

	for (i=1; d>et[i]; i++)
		;
	*pm = i;
	*pd = d - et[i-1];
}

/* Compute Julian Day Number from calendar date */
long
jdn(yr, mo, day, cal)
	int	yr, mo, day;	/* Year, month, date			*/
	int	cal;		/* Calendar type 			*/
{
	if (yr < 0)
		yr++;

	/* Move Jan. & Feb. to end of previous year */
	if (mo <= 2) {
		--yr;
		mo += 12;
	}

	return qfloor((long)(4*365+1)*(yr+4712), 4) + eom[0][mo-1] + day +
		(cal==GREGORIAN ? -qfloor(yr, 100) + qfloor(yr, 400) + 2 : 0);
}

/* Compute calendar date from Julian Day Number */
void
date(n, py, pm, pd, cal)
	long	n;		/* Julian Day Number			*/
	int	*py, *pm, *pd;	/* Year, month, date			*/
	int	cal;		/* Calendar type			*/
{
	long	d, t;
	int	y;

/* Find position within cycles that are nd days long */
#define CYCLE(n, nd) { t=qfloor(d-1,nd); y+=t*n; d-=t*nd; }

/* The same, with bound on cycle number */
#define LCYCLE(n, nd, l) { t=qfloor(d-1,nd); if (t>l) t=l; y+=t*n; d-=t*nd; }

	y = -4799;
	if (cal == GREGORIAN) {
		d = n + 31739;		/* JD -31739 = 31 Dec 4801 B.C.	*/
		CYCLE(400, 146097)	/* Four-century cycle		*/
		LCYCLE(100, 36524, 3)	/* 100-year cycle		*/
	}
	else
		d = n + 31777;		/* JD -31777 = 31 Dec 4801 B.C.	*/

	CYCLE(4, 1461)			/* Four-year cycle		*/
	LCYCLE(1, 365, 3)		/* Yearly cycle			*/

	if (y <= 0)
		--y;
	*py = y;
	mnday((int)d, isleap(y, cal), pm, pd);
}
