#ifndef lint
static char rcsid[]="$Header: cmd_opt.c,v 1.1 92/03/10 14:58:16 julian Exp $";
#endif lint
/*
 * Routines for processing command-line options
 * Idea from James W. Herriot, USGS, Menlo Park, Calif.
*/

#include <stdio.h>
#include <local/stdtyp.h>
#include <local/cmd_opt.h>

#define  strneq		!strncmp
#define  isnumstr(s)	!verify(s, "+-.0123456789eE")

int	eargc;		/* Copy of argc		*/
char	**eargv;	/* Copy of argv		*/
int	iarg;		/* Loop index over args	*/

double		atof();
void		error();
void		exit();
char		*verify();
static void	prt_hdoc();
static void	prt_1doc();

/* Get next arg as string */
char *aarg()
{
	if (++iarg >= eargc)
		error("Too few args");
	return eargv[iarg];
}

/* Look at next arg, but don't increment counter */
char *peekarg()
{
	return (iarg+1<eargc ? eargv[iarg+1] : (char *)NULL);
}

/* Is the next arg numeric? */
int
isnarg()
{
	return (iarg+1<eargc && isnumstr(eargv[iarg+1]));
}

/* Get next arg as a number */
double
narg()
{
	if (!isnarg())
		error("Numeric arg needed");
	return atof(aarg());
}

void
prt_doc(s)
	char	*s;		/* "[file...]", usually			*/
{
	extern char		*progname;
	extern struct command	cmd[];
	extern int		ncmd;
	register int		i;

	(void)fprintf(stderr, "Usage:  %s [options]", progname);
	if (s != NULL)
		(void)fprintf(stderr, " %s", s);
	(void)fprintf(stderr, "\n");
	prt_hdoc();
	for (i=0; i<ncmd; i++)
		prt_1doc(&cmd[i]);
	exit(1);
}

struct command *findname(name)
	char *name;
{
	extern struct command	cmd[];
	extern char		*progname;
	extern int		ncmd;
	register int		i;
	int			len = strlen(name), n;
	struct command		*c;

	n = 0;
	for (i=0; i<ncmd; i++)
		if (strneq(name, cmd[i].c_name, len)){
			c = &cmd[i];
			if (len == strlen(cmd[i].c_name))
				return (c);
			n++;
		}

	switch (n) {
	case 0:		/* No match */
		error("Option \"%s\" not known", name);
		break;
	case 1:		/* One match */
		return (c);
		break;
	default:	/* Multiple match */
		(void)fprintf(stderr, "%s: option \"%s\" is ambiguous: cf. ", progname, name);
		for (i=0; i<ncmd; i++)
			if (strneq(name, cmd[i].c_name, len))
				(void)fprintf(stderr, " %s", cmd[i].c_name);
		(void)fprintf(stderr, "\n");
		exit(1);
	}
	return ( (struct command *)0 );		/* Make lint be quiet */
}

void
prt_help()
{
	extern int	eargc, iarg;
	extern char	**eargv;

	prt_hdoc();
	for (iarg++; iarg<eargc; iarg++)
		prt_1doc( findname(eargv[iarg]) );
	exit(1);
}

static
void
prt_hdoc()
{
	(void)fprintf(stderr, "Option       Args            Remarks...\n\n"); 
}

static
void
prt_1doc(c)
	register struct command	*c;
{
	(void)fprintf(stderr, "%-12s %-15s %s\n",
		c->c_name, c->c_args, c->c_comment);
}
