#ifndef lint
static char rcsid[]="$Header: dispcf.c,v 1.2 88/10/17 10:42:50 julian Exp $";
#endif lint
/*
 * Static surface displacements from moment tensor in half-space
 * Right-handed coordinate system with +z axis downward
 * After Okada, Yoshimitsu, Surface deformation due to shear and tensile
 * faults in a half space, BSSA, 75, no. 4, pp. 1135-1154, 1985
*/

#include <math.h>
#include <local/mathconst.h>

dispcf(x, y, d, nu, ux, uy, uz)
	double	x, y;		/* Position on surface		(input)	*/
	double	d;		/* Depth of nucleus		(input)	*/
	double	nu;		/* Poisson's ratio		(input)	*/
	double	ux[6],		/* Coefficients for determining		*/
		uy[6],		/* displacements		(output)*/
		uz[6];
{
	double	mlm = 1.0-2.0*nu;/* mu/(lambda+mu)			*/
	double	d1, d2, d3;	/* d+1, d+2, d+3			*/
	double	r;		/* Slant distance			*/
	double	rsq;		/* r^2					*/
	double	i1, i2;		/* Sub-expressions			*/
	double	fact;		/* Scratch variable			*/

	/* Calculate slant distance and convert x, y, d to direction cosines */
	rsq = x*x + y*y + d*d;
	r = sqrt(rsq);
	x /= r; y /= r; d /= r;

	d1 = d + 1.0; d2 = d + 2.0; d3 = d + 3.0;
	i1 = mlm*y*(1.0 - x*x*d3/d1)/d1/d1;
	i2 = mlm*x*(1.0 - y*y*d3/d1)/d1/d1;

	/* Off-diagonal (double couple) terms */
	fact = 1.0/TWOPI/rsq;
	ux[XY] =  fact*(3.0*x*x*y + i1);
	uy[XY] =  fact*(3.0*x*y*y + i2);
	uz[XY] = -fact*x*y*(3.0*d - 2.0*nu*d2/d1/d1);
	ux[XZ] = -fact*3.0*x*x*d;
	uy[XZ] = -fact*3.0*x*y*d;
	uz[XZ] =  fact*3.0*x*d*d;
	ux[YZ] =  uy[XZ];
	uy[YZ] = -fact*3.0*y*y*d;
	uz[YZ] =  fact*3.0*y*d*d;

	/* Diagonal (dipole) terms */
	fact = 1.0/FOURPI/rsq;
	ux[XX] =  fact*(3.0*x*x*x - mlm*x*(1.0 - y*y*d3/d1)/d1/d1 - 2.0*nu*x);
	uy[XX] =  fact*(3.0*x*x*y + mlm*y*(1.0 - x*x*d3/d1)/d1/d1 - y);
	uz[XX] = -fact*(3.0*x*x*d - mlm  *(1.0 - y*y*d2/d1)/d1    - 2.0*nu*d);
	ux[YY] =  fact*(3.0*y*y*x + mlm*x*(1.0 - y*y*d3/d1)/d1/d1 - x);
	uy[YY] =  fact*(3.0*y*y*y - mlm*y*(1.0 - x*x*d3/d1)/d1/d1 - 2.0*nu*y);
	uz[YY] = -fact*(3.0*y*y*d - mlm  *(1.0 - x*x*d2/d1)/d1    - 2.0*nu*d);
	fact = (3.0*d*d - 2.0*nu)/FOURPI/rsq;
	ux[ZZ] =  fact*x;
	uy[ZZ] =  fact*y;
	uz[ZZ] = -fact*d;
}
