#ifndef lint
static char rcsid[]="$Header: fmdcmp.c,v 1.2 90/06/25 20:46:37 julian Exp $";
#endif lint
/*
 * Decompose moment tensor into explosion + CLVD + double couple
 * (Knopoff and Randall, JGR, 75, 4957-4963, 1970, eqn. 12)
 * Bruce R. Julian, USGS Menlo Park, CA
 */
#include <math.h>
#include <stdio.h>
#include <assert.h>
#include <local/mathconst.h>

	void	exit();		/* -lc					*/

fmdcmp(pmom, th, ph, mv, mdc, mclvd, tdc, pdc, tclvd, pclvd)
	float	pmom[3];	/* Principal moments		(input) */
	float	th[3],		/* Orientation of princ. axes	(input) */
		ph[3];
	float	*mv,		/* Moments of components	(output)*/
		*mdc,
		*mclvd;
	float	tdc[3],		/* Orientations of D.C. axes	(output)*/
		pdc[3];
	float	*tclvd,		/* Orientation of CLVD axis	(output)*/
		*pclvd;
{
	int	i;		/* Loop index				*/
	int	ip, it;
	float	t;		/* Scratch				*/

#define TAXIS	0
#define IAXIS	1
#define PAXIS	2
#define SWAP(a,b)	{t=a; a=b; b=t;}

	/* Calculate volumetric component and subtract it from moment tensor */
	*mv = (pmom[0] + pmom[1] + pmom[2])/3.0;
	for (i=0; i<3; i++)
		pmom[i] -=  *mv;

	assert(pmom[0] >= pmom[1]);
	assert(pmom[1] >= pmom[2]);

	/* Arrange so abs(pmom[1]) <= abs(pmom[0]) <= abs(pmom[2]) */
	if (pmom[1] < 0.0) {
		SWAP(pmom[0], pmom[2]);
		SWAP(th[0], th[2]);
		SWAP(ph[0], ph[2]);
	}
	assert(fabs(pmom[1]) <= fabs(pmom[0]));
	assert(fabs(pmom[0]) <= fabs(pmom[2]));

	/* Calculate double-couple and CLVD moments */
	*mclvd = -2.0*pmom[1];
	*mdc = pmom[2] - *mclvd;

	/* Store moments, angles in output variables */
	*tclvd  = th[2]; *pclvd  = ph[2];
	if (*mdc > 0.0)
		{ it = 2; ip = 0; }
	else
		{ it = 0; ip = 2; *mdc *= -1.0; }
	tdc[TAXIS] = th[it]; pdc[TAXIS] = ph[it];
	tdc[IAXIS] = th[1];  pdc[IAXIS] = ph[1];
	tdc[PAXIS] = th[ip]; pdc[PAXIS] = ph[ip];
}
