#ifndef lint
static char rcsid[] = "$Header: model3d.c,v 1.1 90/10/03 13:25:39 julian Exp $";
#endif lint
/*
 * Three-dimensional seismic wave-speed model
 */
#include <stdio.h>
#include <math.h>
#include <local/mathconst.h>
#include <local/earthconst.h>
#include <local/stdtyp.h>
#include <local/local.h>
#include <local/cuint.h>
#include <local/model3d.h>

void	back();			/* libq					*/
void	cvptor();		/* libq					*/
void	cvrtop();		/* libq					*/
void	delaz();		/* libq					*/
char	*emalloc();		/* libq					*/
float	***farray3();		/* libq					*/
void	free_fa3();		/* libq					*/
void	free_fvector();		/* libq					*/
void	free_mdl();		/* Defined below			*/
float	*fvector();		/* libq					*/
void	lltoxy();		/* Defined below			*/
int	read3();		/* Defined below			*/
void	refpt();		/* libq					*/
void	xytoll();		/* Defined below			*/

/*
 * Read in model
 */
MODEL3D *
rdmdl(f)
	FILE	*f;
{
	MODEL3D	*m;

	/* Allocate space for data structure */
	m = (MODEL3D *)emalloc((unsigned)sizeof(MODEL3D));

	/* Read in dimensions, etc. */
	if (fread((char *)&(m->limits), sizeof(m->limits), 1, f) != 1) {
		free((char *)m);
		return NULL;
	}
	refpt(m->limits.lat0, m->limits.lon0);

	/* Allocate space for gridded values */
	m->xgrid = fvector(0, m->limits.nx-1);
	m->ygrid = fvector(0, m->limits.ny-1);
	m->zgrid = fvector(0, m->limits.nz-1);
#define FA3 farray3(0, m->limits.nx-1, 0, m->limits.ny-1, 0, m->limits.nz-1)
	m->s	 = FA3;
	m->sx	 = FA3;
	m->sy	 = FA3;
	m->sz	 = FA3;
	m->sxy	 = FA3;
	m->sxz	 = FA3;
	m->syz	 = FA3;
	m->sxyz	 = FA3;
#undef FA3

	/* Read in grid coordinates */
#define RDGRID(grid, n) fread((char *)(grid), sizeof(grid[0])*(n), 1, f)
	if (RDGRID(m->xgrid, m->limits.nx) != 1 ||
	    RDGRID(m->ygrid, m->limits.ny) != 1 ||
	    RDGRID(m->zgrid, m->limits.nz) != 1 ) {
		free_mdl(m);
		return NULL;
	}
#undef RDGRID

	/* Read in slowness values and derivatives */
	if (read3(m->s,    &m->limits, f) == FAIL ||
	    read3(m->sx,   &m->limits, f) == FAIL ||
	    read3(m->sy,   &m->limits, f) == FAIL ||
	    read3(m->sz,   &m->limits, f) == FAIL ||
	    read3(m->sxy,  &m->limits, f) == FAIL ||
	    read3(m->sxz,  &m->limits, f) == FAIL ||
	    read3(m->syz,  &m->limits, f) == FAIL ||
	    read3(m->sxyz, &m->limits, f) == FAIL) {
		free_mdl(m);
		return NULL;
	}

	return m;
}

/*
 * Read values into 3-D array
 */
PRIVATE int
read3(a, l, f)
	float	***a;		/* [0..l->nx-1][0..l->ny-1][0..l->nz-1]	*/
	struct lim *l;
	FILE	*f;
{
	int	i, j;		/* Dummy loop indices			*/

	for (i=0; i<l->nx; i++)
		for (j=0; j<l->ny; j++)
			if (fread((char *)a[i][j], sizeof(***a), l->nz, f)
				!= l->nz)
				return FAIL;
	return SUCCEED;
}

/*
 * Free space allocated for model
 */
void
free_mdl(m)
	MODEL3D	*m;
{
	free_fvector(m->xgrid, 0, m->limits.nx-1);
	free_fvector(m->ygrid, 0, m->limits.ny-1);
	free_fvector(m->zgrid, 0, m->limits.nz-1);
#define FRFA3(a) free_fa3(a,	0, m->limits.nx-1, \
				0, m->limits.ny-1, \
				0, m->limits.nz-1)
	FRFA3(m->s);
	FRFA3(m->sx);
	FRFA3(m->sy);
	FRFA3(m->sz);
	FRFA3(m->sxy);
	FRFA3(m->sxz);
	FRFA3(m->syz);
	FRFA3(m->sxyz);
#undef FRFA3
	free((char *)m);
}

/*
 * Convert from (lat, lon) to (x, y) coordinates
 */
void
lltoxy(m, lat, lon, x, y)
	MODEL3D	*m;
	float	lat, lon;
	float	*x, *y;
{
	double	r, az, b;

	delaz(lat, lon, &r, &az, &b);
	cvptor(r*REARTH, az+(m->limits.rot), x, y);
}

/*
 * Convert from (x,y) to (lat, lon) coordinates
 */
void
xytoll(m, x, y, lat, lon)
	MODEL3D	*m;
	float	x, y;
	float	*lat, *lon;
{
	float	r, th;
	double	lt, ln;

	if (x == 0.0 && y == 0.0)
		r = th = 0.0;
	else
		cvrtop(x, y, &r, &th);
	back(r/REARTH, th-(m->limits.rot), &lt, &ln);
	*lat = lt; *lon = ln;
}

/*
 * Compute slowness, derivatives at specified point
 */
void
sder(m, rx, ry, rz, p)
	MODEL3D	*m;
	float	rx, ry, rz;	/* Spatial coordinates:  x, y, z	*/
	EPOINT	*p;		/* Seismic wave slowness, derivatives	*/
{
	double	dx, dy, dz;	/* Grid-square dimensions		*/
	double	x, y, z;	/* Scaled Cartesian coordinates		*/
	int	ix, iy, iz;	/* Grid-square indices			*/
	int	i, j, k;	/* Dummy loop indices			*/
	int	ii, jj, kk;	/* Dummy loop indices			*/
	int	ijk;		/* Dummy subscript			*/
static	int	ixlast= -1;	/* Grid square of last computed point	*/
static	int	iylast= -1;
static	int	izlast= -1;
static	float	*c1;		/* [0..3] 1-D polynomial coefficients	*/
static	float	***c;		/* [0..3][0..3][0..3] Polynomial coefs.	*/
	PT3	p3;
static	float	*f,		/* [1..8] Values at local vertices	*/
		*fx, *fy, *fz,
		*fxy, *fxz, *fyz,
		*fxyz;

#define S	(p->s)
#define Sx	(p->sx)
#define Sy	(p->sy)
#define Sz	(p->sz)
#define Sxx	(p->sxx)
#define Syy	(p->syy)
#define Szz	(p->szz)
#define Sxy	(p->sxy)
#define Sxz	(p->sxz)
#define Syz	(p->syz)

	/* On first call, allocate storage */
	if (c == NULL) {
		f    = fvector(1, 8);
		fx   = fvector(1, 8);
		fy   = fvector(1, 8);
		fz   = fvector(1, 8);
		fxy  = fvector(1, 8);
		fxz  = fvector(1, 8);
		fyz  = fvector(1, 8);
		fxyz = fvector(1, 8);
		c   = farray3(0, 3, 0, 3, 0, 3);
		c1  = fvector(0, 3);
	}

	/* Locate grid square */
#define hunt0(xx, n, x, i) (hunt(xx-1, n, x, i+1) - 1)
	ix = hunt0(m->xgrid, m->limits.nx, rx, ixlast);
	iy = hunt0(m->ygrid, m->limits.ny, ry, iylast);
	iz = hunt0(m->zgrid, m->limits.nz, rz, izlast);
#undef hunt0

	if (ix >= 0 && ix < m->limits.nx-1 &&
	    iy >= 0 && iy < m->limits.ny-1 &&
	    iz >= 0 && iz < m->limits.nz-1) {
		dx = m->xgrid[ix+1] - m->xgrid[ix];
		x = (rx - m->xgrid[ix])/dx;
		dy = m->ygrid[iy+1] - m->ygrid[iy];
		y = (ry - m->ygrid[iy])/dy;
		dz = m->zgrid[iz+1] - m->zgrid[iz];
		z = (rz - m->zgrid[iz])/dz;

		/* Are we in a new grid square? */
		if (ix != ixlast || iy != iylast || iz != izlast) {
			ixlast = ix; iylast = iy; izlast = iz;

			/* Compute polynomial coefficients for new square */
			ijk = 1;
			for (kk=0; kk<2; kk++) {
				k = iz + kk;
				for (jj=0; jj<2; jj++) {
					j = iy + jj;
					for (ii=0; ii<2; ii++) {
						i = ix + ii;
						f   [ijk] = m->s   [i][j][k];
						fx  [ijk] = m->sx  [i][j][k];
						fy  [ijk] = m->sy  [i][j][k];
						fz  [ijk] = m->sz  [i][j][k];
						fxy [ijk] = m->sxy [i][j][k];
						fxz [ijk] = m->sxz [i][j][k];
						fyz [ijk] = m->syz [i][j][k];
						fxyz[ijk] = m->sxyz[i][j][k];
						ijk++;
					}
				}
			}
			tcucof(f, fx, fy, fz, fxy, fxz, fyz, fxyz,
				dx, dy, dz, c);
		}
		tcuint(c, dx, dy, dz, x, y, z, &p3);

		S   = p3.f;
		Sx  = p3.fx;
		Sy  = p3.fy;
		Sz  = p3.fz;
		Sxx = p3.fxx;
		Sxy = p3.fxy;
		Syy = p3.fyy;
		Sxz = p3.fxz;
		Syz = p3.fyz;
		Szz = p3.fzz;
	}
	else {
		/* Outside laterally varying region */
		if (iz < 0) {
			S = m->s[0][0][0];
			Sz = Szz = 0.0;
		} else if (iz >= m->limits.nz-1) {
			S = m->s[0][0][m->limits.nz-1];
			Sz = Szz = 0.0;
		} else {
			dz = m->zgrid[iz+1] - m->zgrid[iz];
			z = (rz - m->zgrid[iz])/dz;
			cucof(&m->s[0][0][iz]-1, &m->sz[0][0][iz]-1, dz, c1);
			cuint(c1, dz, z, &S, &Sz, &Szz);
		}
		Sx = Sy = 0.0;
		Sxx = Sxy = Syy = Sxz = Syz = 0.0;
	}
}

#ifdef TEST
char	*progname;

FILE	*efopen();
void	error();

main(argc, argv)
	int	argc;
	char	*argv[];
{
	FILE	*fmdl;
	MODEL3D	*m;
	float	x, y, z;
	EPOINT	p;

	progname = NULL;
	if (argc < 2)
		error("Usage: %s model_file", argv[0]);
	progname = "test_model3d";

	/* Read model from file */
	fmdl = efopen(argv[1], "r");
	if ((m = rdmdl(fmdl)) == NULL)
		error("Can't read model");
	(void)fclose(fmdl);

	/* Loop: read coordinates, compute and print slownesses, derivatives */
	while (scanf("%f%f%f", &x, &y, &z) == 3) {
		sder(m, x, y, z, &p);
		printf("%f\n%f\t%f\t%f\n%f\t%f\t%f\t%f\t%f\t%f\n",
			p.s,
			p.sx, p.sy, p.sz,
			p.sxx, p.sxy, p.syy, p.sxz, p.syz, p.szz);
	}

	free_mdl(m);
}

#endif TEST
