#ifndef lint
static char rcsid[]="$Header: pexcf.c,v 1.2 88/10/17 11:16:39 julian Exp $";
#endif lint
#include <local/mathconst.h>

/*
 *     Calculate coefficients for determining the far-field
 *     radiation pattern of P waves from the moment-rate tensor
 *     components of a point source in an infinite, homogeneous,
 *     elastic medium.
 *     The radiation pattern is normalized; to obtain
 *     particle amplitudes, multiply by
 *
 *          1.0/(4.0*pi*rho*(v^3)*r),
 *
 *     where:
 *          rho is the density in the source region,
 *          v is the P-wave speed in the source region, and
 *          r is the geometric spreading factor
 *            (for a homogeneous medium, this is equal to the distance
 *            to the observation point.)
 *
 *      Reference:
 *         Aki, Keiiti, and Paul G. Richards, Quantitative Seismology,
 *         Freeman, San Francisco, 1980, equation 49.1, page 118.
 */
pexcf(u, a)
	float	u[3];		/* Unit vector in ray direction		*/
	float	a[6];		/* Excitation coefficients		*/
				/*   Order:  xx, xy, yy, xz, yz, zz	*/
{
	a[XX] =     u[X]*u[X];
	a[XY] = 2.0*u[X]*u[Y];
	a[YY] =     u[Y]*u[Y];
	a[XZ] = 2.0*u[Z]*u[X];
	a[YZ] = 2.0*u[Y]*u[Z];
	a[ZZ] =     u[Z]*u[Z];
}
