#ifndef lint
static char rcsid[] = "$Header$";
#endif lint
#include <math.h>
#include <local/local.h>
#include <local/qerr.h>

#define SWAP(a,b)	{ swap=a; a=b; b=swap; }

/*
 * Find root of function
 * by combination of Newton's method and bisection
 */
int
rtsafe(f, x1, x2, e, n, root)
	double	(*f)();	/* Function				*/
	double	x1, x2;		/* Bounds on root			*/
	double	e;		/* Desired accuracy			*/
	int	n;		/* Iteration limit			*/
	double	*root;		/* Computed root			*/
{
	double	dy;
	double	dx;
	double	dxold;
extern	int	errno;
	double	y;
	double	yh;
	double	yl;
	int	j;
	double	rts;
	double	swap;
	double	temp;
	double	xh;
	double	xl;

	(void)(*f)(x1, &yl, &dy);
	(void)(*f)(x2, &yh, &dy);
	if (yl*yh >= 0.0) {
		/* Root not bracketed */
		errno = EBADVALUE;
		return FAIL;
	}

	/* Orient the search so that f(xl) < 0 */
	if (yl < 0.0) {
		xl = x1;
		xh = x2;
	} else {
		xh = x1;
		xl = x2;
		SWAP(yl, yh);
	}

	/* Initialize */
	rts = 0.5*(x1+x2);
	dxold = fabs(x2-x1);
	dx = dxold;
	(*f)(rts, &y, &dy);

	/* Iteration loop */
	for (j=1; j<=n; j++) {
		if ((((rts-xh)*dy-y)*((rts-xl)*dy-y) >= 0.0) ||
		    (fabs(2.0*y) > fabs(dxold*dy))) {
			/* Newton out of range or decreasing slowly: Bisect */
			dxold = dx;
			dx = 0.5*(xh-xl);
			rts = xl + dx;
			if (xl == rts) {
				*root = rts;
				return SUCCEED;
			}
		} else {
			/* Accept Newton step */
			dxold = dx;
			dx = y/dy;
			temp = rts;
			rts -= dx;
			if (temp == rts) {
				*root = rts;
				return SUCCEED;
			}
		}
		if (fabs(dx) < e) {
			*root = rts;
			return SUCCEED;
		}
		(*f)(rts, &y, &dy);

		/* Maintain bracket on root */
		if (y < 0.0)	{ xl = rts; yl = y; }
		else		{ xh = rts; yh = y; }
	}
	errno = ENOCONVRG;
	return FAIL;
}

#ifdef TEST
#include <stdio.h>
#include <math.h>

#define N	100
#define NBMAX	20
#define	X1	1.0
#define X2	50.0

char	*progname = "test_rtsafe";

double	*dvector();
void	free_dvector();
void	zbrak();

static double
fx(x)
	double	x;
{
	return j0(x);
}

static double
bessel(x, y, dy)
	double	x;
	double	*y;
	double	*dy;
{
	*y  = j0(x);
	*dy = -j1(x);
	return *y;
}

main()
{
	int	i, nb = NBMAX;
	double	e, root, *xb1, *xb2;

	xb1 = dvector(1, NBMAX);
	xb2 = dvector(1, NBMAX);
	zbrak(fx, X1, X2, N, xb1, xb2, &nb);
	printf("\Roots of Bessel function J0:\n");
	printf("%21s %15s\n", "x", "f(x)");
	for (i=1; i<=nb; i++) {
		e = (1.0e-6)*(xb1[i] + xb2[i])/2.0;
		if (rtsafe(bessel, xb1[i], xb2[i], e, &root) != SUCCEED)
			error("rtsafe() failed");
		printf("root %3d %14.6f %14.6f\n", i, root, fx(root));
	}
	free_dvector(xb1, 1, NBMAX);
	free_dvector(xb2, 1, NBMAX);
}
#endif TEST
