#include <math.h>
#include <local/mathconst.h>

/*
 * Compute strike, dip, rake for shear fault
 * Right-handed coordinate system with x axis north, z axis down.
 * Normal vector n directed from footwall to hanging wall.
 * Slip vector s gives motion of hanging wall w.r.t. footwall.
 * Fault dips to the right when facing strike direction.
 */
void
sdr(n, s, strike, dip, rake)
	float	n[3];		/* Unit normal to fault plane	(input)	*/
	float	s[3];		/* Unit slip vector		(input)	*/
	float	*strike,	/* 				(output)*/
		*dip,
		*rake;
{
	double	r = hypot(n[X], n[Y]);
	double	hx, hy;		/* Horizontal unit vector in fault plane*/
	double	vx, vy, vz;	/* Vertical unit vector in fault plane	*/
				/* v = n X h				*/
	double	s1, s2;		/* Fault-plane components of slip vector*/

	if (r == 0.) {
		/* Strike undefined:  make it zero */
		hx = 1.;
		hy = 0.;
	}
	else {
		hx =  n[Y]/r;
		hy = -n[X]/r;
	}
	*strike = atan2(hy, hx);
	*dip = atan2(r, -n[Z]);
	s1 = hx*s[X] + hy*s[Y];
	vx = -n[Z]*hy;
	vy =  n[Z]*hx;
	vz =  n[X]*hy - n[Y]*hx;
	s2 = vx*s[X] + vy*s[Y] + vz*s[Z];
	*rake = atan2(s2, s1);
}
