#ifndef lint
static char rcsid[]="$Header: sexcf.c,v 1.2 88/10/17 11:16:44 julian Exp $";
#endif lint
/*
 * S-wave excitation by moment-tensor source
 */
#include <math.h>
#include <local/mathconst.h>

sexcf(l, p, a)
register float	l[3];	/* Unit vector tangent to ray		*/
register float	p[3];	/* Direction of particle motion		*/
register float	a[6];	/* Excitation coefficients		*/
			/*   Order:  xx, xy, yy, xz, yz, zz	*/
/*
 *     Calculates coefficients for determining the far-field
 *     radiation pattern of S waves from the moment-rate tensor
 *     components of a point source in an infinite, homogeneous,
 *     elastic medium.
 *     The radiation pattern is normalized; to obtain
 *     particle amplitudes, multiply by
 *
 *          1.0/(4.0*pi*rho*(v^3)*r),
 *
 *     where:
 *          rho is the density in the source region,
 *          v is the Sh-wave speed in the source region, and
 *          r is the geometric spreading factor
 *            (for a homogeneous medium, this is equal to the distance
 *            to the observation point.)
 *
 *      Reference:
 *         Aki, Keiiti, and Paul G. Richards, Quantitative Seismology,
 *         Freeman, San Francisco, 1980, equation 49.1, page 118.
 */
{
	a[XX] = p[X]*l[X];
	a[YY] = p[Y]*l[Y];
	a[ZZ] = p[Z]*l[Z];
	a[XY] = p[X]*l[Y] + p[Y]*l[X];
	a[XZ] = p[X]*l[Z] + p[Z]*l[X];
	a[YZ] = p[Y]*l[Z] + p[Z]*l[Y];
}
