#ifndef lint
static char rcsid[]="$Header: shexcf.c,v 1.2 88/10/17 11:16:46 julian Exp $";
#endif lint
/*
 * Sh-wave excitation by moment-tensor source
 */
#include <math.h>
#include <local/mathconst.h>

shexcf(u, a)
register float	u[3];	/* Unit vector tangent to ray		*/
register float	a[6];	/* Excitation coefficients		*/
			/*   Order:  xx, xy, yy, xz, yz, zz	*/
			/*   (z axis vertical)			*/
/*
 *     Calculates coefficients for determining the far-field
 *     radiation pattern of Sh waves from the moment-rate tensor
 *     components of a point source in an infinite, homogeneous,
 *     elastic medium.
 *     The radiation pattern is normalized; to obtain
 *     particle amplitudes, multiply by
 *
 *          1.0/(4.0*pi*rho*(v^3)*r),
 *
 *     where:
 *          rho is the density in the source region,
 *          v is the Sh-wave speed in the source region, and
 *          r is the geometric spreading factor
 *            (for a homogeneous medium, this is equal to the distance
 *            to the observation point.)
 *
 *      Reference:
 *         Aki, Keiiti, and Paul G. Richards, Quantitative Seismology,
 *         Freeman, San Francisco, 1980, equation 49.1, page 118.
 */
{
	double	ri;

	ri = 1./hypot(u[X], u[Y]);
	a[XX] = -(a[YY] = u[X]*u[Y]*ri);
	a[XY] = (u[X]*u[X]-u[Y]*u[Y])*ri;
	a[XZ] = -u[Y]*u[Z]*ri;
	a[YZ] = -u[X]*u[Z]*ri;
	a[ZZ] = 0.;
}
