#ifndef lint
static char rcsid[]="$Header: straincf.c,v 1.2 88/10/17 11:16:49 julian Exp $";
#endif lint
/*
 * Static surface strains from moment tensor in half-space
 * Right-handed coordinate system with +z axis downward
 * After Okada, Yoshimitsu, Surface deformation due to shear and tensile
 * faults in a half space, BSSA, 75, no. 4, pp. 1135-1154, 1985
*/

#include <math.h>
#include <local/mathconst.h>

straincf(x, y, d, nu, uxx, uxy, uyx, uyy)
	double	x, y;		/* Position on surface		(input)	*/
	double	d;		/* Depth of nucleus		(input)	*/
	double	nu;		/* Poisson's ratio		(input)	*/
	double	uxx[6],		/* Coefficients for determining		*/
		uxy[6],		/* strains			(output)*/
		uyx[6],
		uyy[6];
{
	double	mlm = 1.-2.*nu;	/* mu/(lambda+mu)			*/
	double	d1, d3;		/* d+1, d+3				*/
	double	r;		/* Slant distance			*/
	double	rsq;		/* r^2					*/
	double	j1, j2, j4;	/* Sub-expressions			*/
	double	fact;		/* Scratch variable			*/
	double	tmp;		/* Scratch variable			*/

	/* Calculate slant distance and convert x, y, d to direction cosines */
	rsq = x*x + y*y + d*d;
	r = sqrt(rsq);
	x /= r; y /= r; d /= r;

	d1 = d + 1; d3 = d + 3.;
	tmp = 5. + d*(4. + d);
	j1 = 3.*mlm*x*y*(-d3 + x*x*tmp/d1)/d1/d1/d1;
	j2 = mlm*(1. - 3.*(1. - x*x*y*y*tmp/d1/d1)/d1/d1);
	j4 = mlm*(-3.*x*y) - j1;

	/* Off-diagonal (double couple) terms */
	fact = 1./TWOPI/r/rsq;
	uxx[YZ] = -3.*fact*y*d*(1. - 5.*x*x);
	uxy[YZ] = -3.*fact*x*d*(1. - 5.*y*y);
	uyx[YZ] = 15.*fact*x*y*y*d;
	uyy[YZ] = -3.*fact*y*d*(2. - 5.*y*y);

	uxx[XZ] = -3.*fact*x*d*(2. - 5.*x*x);
	uxy[XZ] = 15.*fact*x*x*y*d;
	uyx[XZ] = -3.*fact*y*d*(1. - 5.*x*x);
	uyy[XZ] = -3.*fact*x*d*(1. - 5.*y*y);

	uxx[XY] = fact*(3.*x*y*(2. - 5.*x*x) + j1);
	uxy[XY] = fact*(3.*x*x*(1. - 5.*y*y) + j2);
	uyx[XY] = fact*(3.*y*y*(1. - 5.*x*x) + j2);
	uyy[XY] = fact*(3.*x*y*(2. - 5.*y*y) + j4);

	/* Diagonal (dipole) terms */
	fact *= 0.5;	/* 1./FOURPI/r^3 */
	uxx[XX] = fact*(-1. + 3.*x*x*(3.-5.*x*x) +
			mlm*3.*(1. - x*x*y*y*tmp/d1/d1)/d1/d1 + 6.*nu*x*x);
	uxy[XX] = fact*3.*x*y*(-5.*x*x + mlm*(d3 - y*y*tmp/d1)/d1/d1/d1 + 2.*nu);
	uyx[XX] = uxy[XX] + fact*6.*x*y;
	uyy[XX] = fact*(3.*(1.-d*d) - 15.*x*x*y*y +
			mlm*3.*(-1. + x*x*y*y*tmp/d1/d1)/d1/d1 - 2.*nu);

	uxx[YY] = uyy[XX];
	uyx[YY] = fact*3.*x*y*(-5.*y*y + mlm*(d3 - x*x*tmp/d1)/d1/d1/d1 + 2.*nu);
	uxy[YY] = uyx[YY] + fact*6.*x*y;
	uyy[YY] = fact*(-1. + 3.*y*y*(3.-5.*y*y) +
			mlm*3.*(1. - x*x*y*y*tmp/d1/d1)/d1/d1 + 6.*nu*y*y);

	uxx[ZZ] = fact*(d*d*(3.-15.*x*x) + nu*(6.*x*x-2.));
	uxy[ZZ] = uyx[ZZ] = fact*x*y*(-15.*d*d + 6.*nu);
	uyy[ZZ] = fact*(d*d*(3.-15.*y*y) + nu*(6.*y*y-2.));
}
