#ifndef lint
static char rcsid[]="$Header: svexcf.c,v 1.2 88/10/17 11:16:52 julian Exp $";
#endif lint
/*
/* Sv-wave excitation by moment-tensor source
 */
#include <math.h>
#include <local/mathconst.h>

svexcf(u, a)
register float	u[3];	/* Unit vector tangent to ray		*/
register float	a[6];	/* Excitation coefficients		*/
			/*   Order:  xx, xy, yy, xz, yz, zz	*/
			/*   (z axis vertical)			*/
/*
 *     Calculates coefficients for determining the far-field
 *     radiation pattern of Sv waves from the moment-rate tensor
 *     components of a point source in an infinite, homogeneous,
 *     elastic medium.
 *     The radiation pattern is normalized; to obtain
 *     particle amplitudes, multiply by
 *
 *          1.0/(4.0*pi*rho*(v^3)*r),
 *
 *     where:
 *          rho is the density in the source region,
 *          v is the Sv-wave speed in the source region, and
 *          r is the geometric spreading factor
 *            (for a homogeneous medium, this is equal to the distance
 *            to the observation point.)
 *
 *      Reference:
 *         Aki, Keiiti, and Paul G. Richards, Quantitative Seismology,
 *         Freeman, San Francisco, 1980, equation 49.1, page 118.
 */
{
	double	r, px, py;

	r = hypot(u[X], u[Y]);
	px = u[X]*u[Z]/r;
	py = u[Y]*u[Z]/r;
	a[XX] = u[X]*px;
	a[XY] = u[X]*py + u[Y]*px;
	a[YY] = u[Y]*py;
	a[XZ] = -u[X]*r + u[Z]*px;
	a[YZ] = -u[Y]*r + u[Z]*py;
	a[ZZ] = -u[Z]*r;
}
