#ifndef lint
static char rcsid[] = "$Header: tcucof.c,v 1.3 89/02/07 13:56:40 julian Exp $";
#endif lint
/*
 * Generate coefficients for tricubic interpolation
 */
#include <local/cuint.h>

float	**fmatrix();
void	free_fmatrix();

void
tcucof(f, f1, f2, f3, f12, f13, f23, f123, d1, d2, d3, c)
	float	*f,		/* Vectors[1..8] of function and	*/
		*f1, *f2, *f3,	/* derivative values			*/
		*f12, *f13, *f23,
		*f123;
	float	d1, d2, d3;	/* Grid spacings			*/
	float	***c;		/* Coefficients[0..3][0..3][0..3]	*/
{
static	float	cfb [4][4], *cfbot [4] = {cfb [0], cfb [1], cfb [2], cfb [3]};
static	float	cf3b[4][4], *cf3bot[4] = {cf3b[0], cf3b[1], cf3b[2], cf3b[3]};
static	float	cft [4][4], *cftop [4] = {cft [0], cft [1], cft [2], cft [3]};
static	float	cf3t[4][4], *cf3top[4] = {cf3t[0], cf3t[1], cf3t[2], cf3t[3]};
				/* Matrices[0..3][0..3] of coefficients	*/
				/* of (x,y) polynomials for f and fz on	*/
				/* planes z=0 ("top") and z=d3 ("top")	*/
static	int	i, j;		/* Dummy loop index			*/
	float	*cij;
	float	tmp;

	/* Coefficients of 2-D bicubics for f(x, y, 0) and fz(x, y, 0) */
	bcucof(f , f1 , f2 , f12 , d1, d2, cfbot);
	bcucof(f3, f13, f23, f123, d1, d2, cf3bot);

	/* Coefficients of 2-D bicubics for f(x, y, d3) and fz(x, y, d3) */
	bcucof(f +4, f1 +4, f2 +4, f12 +4, d1, d2, cftop);
	bcucof(f3+4, f13+4, f23+4, f123+4, d1, d2, cf3top);

	/* Fit cubic in z to each (x, y) coefficient */
	for (i=0; i<4; i++) {
		for (j=0; j<4; j++) {
			cij = c[i][j];
			cij[0] = cfbot[i][j];
			cij[1] = cf3bot[i][j]*d3;
			tmp = cftop[i][j] - cfbot[i][j];
			cij[3] = -(tmp + tmp) + cij[1] + cf3top[i][j]*d3;
			cij[2] = -cij[3] + tmp - cij[1];
		}
	}
}

#ifdef TEST
char	*progname = "test_tcucof";

float	*fvector();
float	***farray3();

main()
{
	float	***c;		/* Coefficients[0..3][0..3][0..3]	*/
	float	*f,		/* Vectors[1..8] of function values	*/
		*f1, *f2, *f3,	/* and derivatives			*/
		*f12, *f13, *f23,
		*f123;
	int	i, j;

	/* Allocate space */
	c = farray3(0, 3, 0, 3, 0, 3);
	f    = fvector(1, 8);
	f1   = fvector(1, 8); f2   = fvector(1, 8); f3   = fvector(1, 8);
	f12  = fvector(1, 8); f13  = fvector(1, 8); f23  = fvector(1, 8);
	f123 = fvector(1, 8);

	/* Loop, reading values, computing and printing coefficients */
	while (getvals(f, f1, f2, f3, f12, f13, f23, f123) == 64) {
		tcucof(f, f1, f2, f3, f12, f13, f23, f123, 1.0, 1.0, 1.0, c);
		for (i=0; i<4; i++) {
			for (j=0; j<4; j++) {
				float	*r=c[i][j];
				printf("%g\t%g\t%g\t%g\n", r[0], r[1], r[2], r[3]);
			}
		}
	}
}

int
getvals(f, f1, f2, f3, f12, f13, f23, f123)
	float	*f,		/* Vectors[1..8] of function values	*/
		*f1, *f2, *f3,	/* and derivatives			*/
		*f12, *f13, *f23,
		*f123;
{
	int	i;		/* Dummy loop index			*/

	for (i=1; i<=8; i++)
		if (scanf("%f%f%f%f%f%f%f%f",
			f+i, f1+i, f2+i, f3+i, f12+i, f13+i, f23+i, f123+i
		) != 8) return 0;
	return 64;
}
#endif TEST
