#ifndef lint
static char rcsid[] = "$Header: tcuint.c,v 1.6 89/02/20 14:39:55 julian Exp $";
#endif lint
/*
 * Tri-cubic interpolation
 */
#include <local/cuint.h>

void
tcuint(c, d1, d2, d3, x, y, z, pt)
	float	***c;		/* Coefficients[0..3][0..3][0..3]	*/
	float	d1, d2, d3;	/* Interval lengths			*/
	float	x, y, z;	/* Independent variables		*/
	PT3	*pt;		/* Computed value and derivatives	*/
{
static	int	i, j;		/* Dummy loop indices			*/
static	float	ai,
		aiy, aiz,
		aiyy, aiyz, aizz;
static	float	bi[4],
		biz[4],
		bizz[4];
static	float	p,
		px, py, pz,
		pxx, pxy, pyy,
		pxz, pyz, pzz;
static	float	*cij;
static	float	b, bz, bzz;

	p = 0.0;
	px = py = pz = 0.0;
	pxx = pxy = pyy = pxz = pyz = pzz = 0.0;
	for (i=3; i>=0; --i) {
		for (j=3; j>=0; --j) {
			cij = c[i][j];
			b = bz = bzz = cij[3];
			b   = b  *z + cij[2];
			bz  = bz *z + b;
			b   = b  *z + cij[1];
			bizz[j] = 2.0*(bzz*z + bz);
			biz [j] = bz*z + b;
			bi  [j] = b*z + cij[0];
		}
		ai  = ((bi[3]*y + bi[2])*y + bi[1])*y + bi[0];
		aiy = (3.0*bi[3]*y + 2.0*bi[2])*y + bi[1];
		aiyz = (3.0*biz[3]*y + 2.0*biz[2])*y + biz[1];
		aiyy = 6.0*bi[3]*y + 2.0*bi[2];
		aiz = ((biz[3]*y + biz[2])*y + biz[1])*y + biz[0];
		aizz = ((bizz[3]*y + bizz[2])*y + bizz[1])*y + bizz[0];
		pxx	= pxx*x + px;
		px	= px *x + p;
		p	= p  *x + ai;
		pxy	= pxy*x + py;
		pxz	= pxz*x + pz;
		pyy	= pyy*x + aiyy;
		pyz	= pyz*x + aiyz;
		py	= py *x + aiy;
		pzz	= pzz*x + aizz;
		pz	= pz *x + aiz;
	}

	/* Scale derivatives, and copy into structure */
	pt->f = p;
	pt->fx = px/d1;
	pt->fy = py/d2;
	pt->fz = pz/d3;
	pt->fxx = 2.0*pxx/(d1*d1);
	pt->fxy = pxy/(d1*d2);
	pt->fyy = pyy/(d2*d2);
	pt->fxz = pxz/(d1*d3);
	pt->fyz = pyz/(d2*d3);
	pt->fzz = pzz/(d3*d3);
}

#ifdef TEST
char	*progname = "test_tcuint";

float	*fvector();
float	***farray3();

main()
{
	float	***c;		/* Coefficients[0..3][0..3][0..3]	*/
	float	*f,		/* Vectors[1..8] of function values	*/
		*f1, *f2, *f3,	/* and derivatives			*/
		*f12, *f13, *f23,
		*f123;
	int	i, j;
	PT3	p;
	float	x, y, z;	/* Coordinates of interpolated point	*/

	/* Allocate space */
	c = farray3(0, 3, 0, 3, 0, 3);
	f    = fvector(1, 8);
	f1   = fvector(1, 8); f2   = fvector(1, 8); f3   = fvector(1, 8);
	f12  = fvector(1, 8); f13  = fvector(1, 8); f23  = fvector(1, 8);
	f123 = fvector(1, 8);

	/* Read values, compute and print coefficients */
	if (getvals(f, f1, f2, f3, f12, f13, f23, f123) != 64)
		error("Can't read function and derivative values");
	tcucof(f, f1, f2, f3, f12, f13, f23, f123, 1.0, 1.0, 1.0, c);
	for (i=0; i<4; i++) {
		for (j=0; j<4; j++) {
			float	*r=c[i][j];
			printf("%g\t%g\t%g\t%g\n", r[0], r[1], r[2], r[3]);
		}
		printf("\n");
	}

	/* Loop, reading coordinates, computing and printing interpolated values */
	while (scanf("%f%f%f", &x, &y, &z) == 3) {
		tcuint(c, 1.0, 1.0, 1.0, x, y, z, &p);
		printf("%f\n%f\t%f\t%f\n%f\t%f\t%f\t%f\t%f\t%f\n",
			p.f, p.fx, p.fy, p.fz, p.fxx, p.fxy, p.fyy, p.fxz, p.fyz, p.fzz);
	}
}

int
getvals(f, f1, f2, f3, f12, f13, f23, f123)
	float	*f,		/* Vectors[1..8] of function values	*/
		*f1, *f2, *f3,	/* and derivatives			*/
		*f12, *f13, *f23,
		*f123;
{
	int	i;		/* Dummy loop index			*/

	for (i=1; i<=8; i++)
		if (scanf("%f%f%f%f%f%f%f%f",
			f+i, f1+i, f2+i, f3+i, f12+i, f13+i, f23+i, f123+i
		) != 8) return 0;
	return 64;
}
#endif TEST
