#ifndef lint
static char rscid[]="$Header: tiltcf.c,v 1.2 88/10/17 11:16:54 julian Exp $";
#endif lint
/*
 * Static surface tilts from moment tensor in half-space
 * Right-handed coordinate system with +z axis downward
 * After Okada, Yoshimitsu, Surface deformation due to shear and tensile
 * faults in a half space, BSSA, 75, no. 4, pp. 1135-1154, 1985
*/

#include <math.h>
#include <local/mathconst.h>

tiltcf(x, y, d, nu, uzx, uzy)
	double	x, y;		/* Position on surface		(input)	*/
	double	d;		/* Depth of nucleus		(input)	*/
	double	nu;		/* Poisson's ratio		(input)	*/
	double	uzx[6],		/* Coefficients for determining		*/
		uzy[6];		/* tilts			(output)*/
{
	double	mlm = 1.-2.*nu;	/* mu/(lambda+mu)			*/
	double	d1, d2;		/* d+1, d+2				*/
	double	r;		/* Slant distance			*/
	double	rsq;		/* r^2					*/
	double	fact;		/* Scratch variable			*/
	double	tmp;		/* Scratch variable			*/

	/* Calculate slant distance and convert x, y, d to direction cosines */
	rsq = x*x + y*y + d*d;
	r = sqrt(rsq);
	x /= r; y /= r; d /= r;

	d1 = d + 1; d2 = d + 2.;
	tmp = 8. + d*(9. + 3.*d);

	/* Off-diagonal (double couple) terms */
	fact = 1./TWOPI/r/rsq;
	uzx[YZ] = -fact*15.*x*y*d*d;
	uzy[YZ] =  fact*3.*d*d*(1.-5.*y*y);

	uzx[XZ] =  fact*3.*d*d*(1.-5.*x*x);
	uzy[XZ] = -fact*15.*x*y*d*d;

	uzx[XY] = -fact*(3.*d*y*(1.-5.*x*x) - mlm*y*(d2 - x*x*tmp/d1)/d1/d1);
	uzy[XY] = -fact*(3.*d*x*(1.-5.*y*y) - mlm*x*(d2 - y*y*tmp/d1)/d1/d1);

	/* Diagonal (dipole) terms */
	fact *= 0.5;	/* 1./FOURPI/r^3 */
	uzx[XX] = -fact*x*(3.*d*(2.-5.*x*x) + mlm*(d2 - y*y*tmp/d1)/d1/d1 + 6.*nu*d);
	uzy[XX] = -fact*y*(3.*d*(1.-5.*x*x) - mlm*(d2 - x*x*tmp/d1)/d1/d1);

	uzx[YY] = -fact*x*(3.*d*(1.-5.*y*y) - mlm*(d2 - y*y*tmp/d1)/d1/d1);
	uzy[YY] = -fact*y*(3.*d*(2.-5.*y*y) + mlm*(d2 - x*x*tmp/d1)/d1/d1 + 6.*nu*d);

	uzx[ZZ] = -fact*3.*x*d*(-5.*d*d + 2.*nu);
	uzy[ZZ] = -fact*3.*y*d*(-5.*d*d + 2.*nu);
}
