#ifndef lint
static char rcsid[] = "$Header: pcql2ah.c,v 1.3 95/03/03 17:10:27 seis Exp $";
#endif lint
/*
 * Filter to convert a PCQL seismic data file to AH format (XDR version)
 * Usage: pcql2ah [-stafile station_file] [pcql_file...] >ah_file
 */
#include <stdio.h>
#include <ctype.h>
#include <rpc/rpc.h>
#include <local/local.h>
#include <local/date_time.h>
#include <local/cmd_opt.h>
#include <local/station.h>
#include <local/mathconst.h>
#include <local/ahhead.h>
#include "pcql.h"

/* No default station file since we have 3 station files (1 per net).	*/
#define STAFILE	""
#define BLKSIZE	64		/* Samples per block			*/

	int	ah_errno;	/* Ah error flag			*/
	int	errno;		/* Error flag				*/
	char	*stafile;	/* Station-list file name		*/

PRIVATE	void	do_file();	/* Defined below			*/
	FILE	*efopen();	/* -lq (efopen.c)			*/
	char	*emalloc();	/* -lq (emalloc.c)			*/
	void	error();	/* -lq (error.c)			*/
	void	exit();		/* UNIX C library			*/
	float	maxmag();	/* -lah (tssubs.c)			*/
	char	*mkdatspace();	/* -lah (ioroutin.c)			*/
PRIVATE	void	mkhdr();	/* Defined below			*/
	char	*sprintf();	/* UNIX C library			*/
	char	*strcpy();	/* UNIX C library			*/
	char	*strncpy();	/* UNIX C library			*/

PRIVATE	int ah_sample_type =    FLOAT;

/*----  C O M M A N D - L I N E   A R G U M E N T   P R O C E S S I N G  ----*/

qhelp	(){ if(eargc>2) prt_help(); else prt_doc("[file...]"); }
qstafile(){ stafile = aarg(); }
qahtype (){
    char *typestr = aarg();
    if (strcasecmp(typestr,"float")==0) {
	ah_sample_type = FLOAT;
    }
    else if (strcasecmp(typestr,"short")==0) {
	ah_sample_type = SHORT;
    }
    else error("Invalid ahtype: %s", typestr);
}
	
struct command cmd[] = {
	qhelp	, "-help"	, "[opt...]"	, "Documentation help",
	qstafile, "-stafile"	, "file"	, "Station file - required option",
	qahtype, "-ahtype"	, "float|short"	, "Data type of ah output (default float)",
};

main(argc, argv)
	int	argc;
	char	*argv[];
{
	FILE	*fout;		/* Output stream			*/
	FILE	*fsta;		/* Station-table input stream		*/

	/* Initialize */
	fout = stdout;
	errno = ah_errno = 0;
	stafile = STAFILE;

	/* Interpret command-line arguments */
	PROCESS_OPTS

	/* Open station file */
	fsta = efopen(stafile, "r");

	 /* Process PCQL trace files */
	if (iarg >= argc)
		do_file(stdin, fout, fsta);
	else
		for (; iarg<argc; iarg++) {
			FILE *fin = efopen(argv[iarg], "r");
			do_file(fin, fout, fsta);
			(void)fclose(fin);
		}
	(void)fclose(fsta);
	exit(0);
}

/*
 * Convert one PCQL file
 */
PRIVATE void
do_file(fin, fout, fsta)
	FILE	*fin, *fout;
	FILE	*fsta;		/* Station table			*/
{
	float	*ah_data;	/* [0..nsamp-1] AH seismogram		*/
	ahhed	ah_head;	/* AH header				*/
	char	comp[2];
	HDR1	h1;		/* First PCQL header			*/
	HDR2	h2;		/* Second PCQL header			*/
	long	i;		/* Dummy loop index			*/
	long	nsamp;		/* Number of samples to get		*/
	int	nrec;		/* No. of BLKSIZE records		*/
	char	stcode[4];
	char	stime[10];
	void	*v;		/* [0..BLKSIZE*nrec-1]			*/
				/* PCQL seismogram			*/
	XDR	xdrs;
	int	ah_sample_size;	/* output sample size (in bytes).	*/

	if (isatty(fileno(fin)))
		error("Can't read binary input from a terminal.");
	if (isatty(fileno(fout)))
		error("Can't send binary output to a terminal.");

	xdrstdio_create(&xdrs, fout, XDR_ENCODE);

	/* Loop: read and process PCQL headers and data */
	while (get_pcql_hdr(fin, &h1) == SUCCEED) {

		nsamp = h1.npts;
		nrec = (nsamp+BLKSIZE-1)/BLKSIZE;

		/* Allocate space for one seismogram */
		ah_head.record.type = ah_sample_type;
		ah_sample_size = size(&ah_head);
		v = (void *)emalloc((unsigned)BLKSIZE*nrec*ah_sample_size);

		/* Read all seismograms in file */
		while (get_pcql_trace(fin, &h2, nrec, v) == SUCCEED) {

			/* Look in station file to find station and component */
			/* for this channel and time */
			rewind(fsta);
			sprintf(stime, "%2d%3d%2d%2d",
				h1.yr, h1.day, h1.hr, h1.mn);
			getsta(fsta, h2.chnum, stime, stcode, comp);

			/* Construct AH header and data records */
			mkhdr(&ah_head, &h1, stcode, comp);
			ah_data = (float *)mkdatspace(&ah_head);
			memcpy (ah_data, v, nsamp * ah_sample_size);
			ah_head.record.maxamp = maxmag(ah_data, (int)nsamp, 1);

			/* Write AH header and seismogram to fout */
			if (xdr_putrecord(&ah_head, (char *)ah_data, &xdrs) != SUCCEED)
				error("Can't write data");

			free((char *)ah_data);
		}
		free((char *)v);
	}
}

/*
 * Read header from PCQL trace file
 */
int
get_pcql_hdr(f, p)
	FILE	*f;
	HDR1	*p;
{
	HDR1	q;

	if (fread ((char *)&q, sizeof(q), 1, f) == NULL)
		return FAIL;
	bcopy((char *)&q, (char *)p, 4);
	swab(((char *)&q)+4, ((char *)p)+4, sizeof(q)-4);
	return SUCCEED;
}

/*
 * Read next seismogram from PCQL trace file
 */
int
get_pcql_trace(f, p, nrec, v)
	FILE	*f;
	HDR2	*p;
	int	nrec;
	void	*v;	/* Seismogram				*/
{
	HDR2	q;
	short	buf[BLKSIZE];
	short	swabbuf[BLKSIZE];
	int	i, j;

	if (fread ((char *)&q, sizeof(q), 1, f) == NULL)
		return FAIL;
	swab((char *)&q, (char *)p, 6);
	for (i=0; i<nrec; i++) {
		if (fread ((char *)buf, BLKSIZE*sizeof(short), 1, f) == NULL)
			return FAIL;
		swab((char *)buf, (char *)swabbuf, BLKSIZE*sizeof(short));
		switch (ah_sample_type) {
		  case SHORT:
		    swab((void *)buf, (void *)(((short *)v)+(i*BLKSIZE)), BLKSIZE*sizeof(AHTYPE));
		    memcpy((void *)(((short *)v)+(i*BLKSIZE)), (void *)swabbuf, BLKSIZE*sizeof(short));
		    break;
		  case FLOAT:
		    for (j=0; j<BLKSIZE; j++) {
			*(((float *)v)+(i*BLKSIZE)+j) = (float)swabbuf[j];
		    }
		    break;
		  default:
		    error ("Unknown output sample type\n");
		    break;
		}
	}
	return SUCCEED;
}

/*
 * Look up channel and time in PCQL .stn file
 */
int
getsta(f, chnum, stime, stcode, comp)
	FILE	*f;
	short	chnum;
	char	stime[10];
	char	stcode[4];
	char	comp[2];
{
	char	line[256];

	/* Read lines from station file, looking for match */
	while (fgets(line, sizeof(line), f) != NULL) {
		if (line[0] == '*')
			/* Comment line */
			continue;
		if (atoi(&line[2]) != chnum)
			/* Wrong channel */
			continue;
		if (strlen(line) >= 71 && strncmp(stime, &line[61], 9) > 0)
			/* Too early */
			continue;
		if (strncmp(stime, &line[51], 9) < 0)
			/* Too late */
			continue;

		/* Found it! */
		strncpy(stcode, &line[25], 4);

		/* Extract component identifier from station code */
		switch (line[28]) {
		case 'Z':
		case 'N':
		case 'E':
			/* 3-character station code + component code */
			strncpy(comp, &line[28], 1);
			stcode[3] = '\0';
			break;
		case ' ':
			/* 3-character station code, implied component code */
			strcpy(comp, "z");
			stcode[3] = '\0';
			break;
		default:
			/* 4-character station code, implied component code */
			strcpy(comp, "z");
			break;
		}
		return SUCCEED;
	}
	return FAIL;
}

/*
 * Construct AH header from info in PCQL headers
 */
PRIVATE void
mkhdr(ah_head, h1, stcode, comp)
	ahhed	*ah_head;
	HDR1	*h1;		/* PCQL header				*/
	char	stcode[4];	/* Station code				*/
	char	comp[2];	/* Component code			*/
{
	int	d, m;
	long	j;

	get_null_head(ah_head);

	/* Record info */
	ah_head->record.type = ah_sample_type;
	ah_head->record.ndata = h1->npts;
	ah_head->record.delta = 1.0/(h1->srate1 + 0.001*h1->srate2);

	/* Date */
	ah_head->record.abstime.yr  = 1900 + h1->yr;
	mnday(h1->day, isleap(1900+h1->yr, GREGORIAN), &m, &d);
	ah_head->record.abstime.mo  = m;
	ah_head->record.abstime.day = d;
	ah_head->record.abstime.hr  = h1->hr;
	ah_head->record.abstime.mn  = h1->mn;
	ah_head->record.abstime.sec = h1->sec + 0.001*h1->ms -
		h1->nprior*ah_head->record.delta;

	/* Station info */
	(void)strncpy(ah_head->station.code, stcode, 4);
	(void)strcpy(ah_head->station.chan, comp);
	(void)strncpy(ah_head->station.stype, "PCQL", 4);

	/* Extra stuff (unused) */
	for (j=0; j<DIM(ah_head->extra); j++)
		ah_head->extra[j] = 0.0;
}
