#include <stdio.h>

#include <math.h>
#include <ctype.h>
#include <sys/param.h>
#include <local/local.h>
#include <local/date_time.h>
#include <local/cmd_opt.h>
#include <local/mathconst.h>
#include <local/earthconst.h>
#include <local/decode.h>

#include "cnss_catalog.h"
#include "global.h"
#include "procs.h"

/************************************************************************/
/*  NOTE: Floating point fields in this format have explicit decimal	*/
/*  points, and therefore need NO explicit scaling.			*/
/************************************************************************/

/* Limits */
#define	MAXLINELEN	512
#define DDECODE(line,len,func,default) ( \
    xx[len] = '\0', \
    strncpy(xx,line,len), \
    (blen(xx)>0) ? func(xx) : default)
char xx[80];

double no_depth = -10000.0;
double no_mag = -10.0;
double no_rms = -1.0;
double no_close = -1.0;
double no_otime_err = -1.0;
double no_lat_err = -1.0;
double no_lon_err = -1.0;
double no_depth_err = -1.0;
int no_num_sta = -1;
int no_max_az_gap = -1;
int no_num_mag_obs = -1;

search_cnss_catalog(fp) 
     FILE *fp;
{
    int		i,j,k; 	
    char	line[MAXLINELEN];
    int 	num_mag_obs,num_m0_obs,num_sta,max_az_gap;
    double	depth,mag,m0,min_delta,close;
    double	otime_err,lat_err,lon_err,depth_err,rms,ot;
    double	lat,lon,cdelt,cdlon,ct1,st1;
    char	qua,felt;
    double	se;
    int		yr, mo, da, hr, mn;
    int		lines = 0;
    int		selected = 0;
    int		linelen;

    while (fgets(line,MAXLINELEN,fp) != NULL) {
	linelen = strlen(line);
	++lines;
	/*
	 * now filter the event
	 */

	if (do_otime) {		/* ORIGIN TIME */
	    yr = DECODE(line+YEAR_OFFSET, YEAR_LEN, atoi);
	    mo = DECODE(line+MONTH_OFFSET, MONTH_LEN, atoi);
	    da = DECODE(line+DAY_OFFSET, DAY_LEN, atoi);

	    hr = DECODE(line+HOUR_OFFSET, HOUR_LEN, atoi);
	    mn = DECODE(line+MINUTE_OFFSET, MINUTE_LEN, atoi);
	    se = DECODE(line+SECOND_OFFSET, SECOND_LEN, atof);
	    ot = timvar(yr, mo, da, hr, mn, se, GREGORIAN);
	    if (ot < otmin || ot > otmax)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_lat) {		/* LATITUDE */
	    lat = DECODE(line+LAT_OFFSET, LAT_LEN, atof);
	    if (lat < latmin || latmax < lat)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_lon) {		/* LONGITUDE */
	    lon = DECODE(line+LON_OFFSET, LON_LEN, atof);
	    if (lon < lonmin || lonmax < lon)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_depth) {		/* FOCAL DEPTH */
	    depth = (linelen < DEPTH_OFFSET+DEPTH_LEN) ? no_depth :
		DDECODE (line+DEPTH_OFFSET, DEPTH_LEN, atof, no_depth);
	    if (depth < zmin || zmax < depth)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_mag) {		/* MAG */
	    mag =  (linelen < MAG_OFFSET+MAG_LEN) ? no_mag :
		DDECODE (line+MAG_OFFSET, MAG_LEN, atof, no_mag);
	    if (mag < magmin || magmax < mag)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_nsta) {		/* NUMBER OF STATIONS (PHASES for CNSS)*/
	    num_sta = (linelen < NPH_LOC_OFFSET+NPH_LOC_LEN) ? no_num_sta :
		DDECODE (line+NPH_LOC_OFFSET, NPH_LOC_LEN, atoi, no_num_sta);
	    if (num_sta < nstmin || nstmax < num_sta)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_rms) {		/* RMS RESIDUAL */
	    rms = (linelen < RMS_OFFSET+RMS_LEN) ? no_rms :
		DDECODE(line+RMS_OFFSET, RMS_LEN, atof, no_rms);
	    if (rms < rmsmin || rmsmax < rms)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_gap) {		/* AZIMUTHAL GAP */
	    max_az_gap = (linelen < AZ_GAP_OFFSET+AZ_GAP_LEN) ? no_max_az_gap : 
		DDECODE (line+AZ_GAP_OFFSET, AZ_GAP_LEN, atoi, no_max_az_gap);
	    if (max_az_gap < gapmin || gapmax < max_az_gap)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_close) {		/* DISTANCE TO CLOSEST STATION */
	    close = (linelen < DIST_OFFSET+DIST_LEN) ? no_close :
		DDECODE(line+DIST_OFFSET, DIST_LEN, atof, no_close);
	    if (close < clomin || clomax < close)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
/*;;	if (do_qual) {		/* "QUALITY" CODE */
/*::	    qua = *(line+QUAL_OFFSET);
/*::	    if (!index(quals, (isupper(qua) ? tolower(qua) : qua)))
/*::	      if (negate) goto OUTPUT;
/*::	      else        continue;
/*::	}
::*/
	if (do_mag_obs) {	/* NUMBER OF OBSERVATIONS USED TO GET MAG */
	    num_mag_obs = (linelen < DIST_OFFSET+DIST_LEN) ? no_num_mag_obs :
		DDECODE(line+N_MAG_OFFSET, N_MAG_LEN, atoi, no_num_mag_obs);
	    if (num_mag_obs < nmagmin || nmagmax < num_mag_obs)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
/*::	if (do_m0) {		/* SCALAR MOMENT */
/*::	    m0 = DECODE (line+M0_OFFSET, M0_LEN, atof);
/*::	    if (m0 < m0min || m0 > m0max) 
/*::	      if (negate) goto OUTPUT;
/*::	      else        continue;
/*::	}
/*::	if (do_m0_obs) {	/* NUMBER OF OBSERVATIONS USED TO GET M0 */
/*::	    num_m0_obs = DECODE (line+N_M0_OFFSET, N_M0_LEN, atoi);
/*::	    if (num_m0_obs < nm0min || num_m0_obs > nm0max)
/*::	      if (negate) goto OUTPUT;
/*::	      else        continue;
/*::	}
/*::	if (do_felt) {		/* FELT EVENT */
/*::	    felt = *(line+FELT_OFFSET);
/*::	    if (felt != 'F') 
/*::	      if (negate) goto OUTPUT;
/*::	      else        continue;
/*::	}
::*/
	if (do_oterr) {		/* ERROR IN ORIGIN TIME */
	    otime_err = (linelen < DIST_OFFSET+DIST_LEN) ? no_otime_err :
		DDECODE(line+OT_ERR_OFFSET, OT_ERR_LEN, atof, no_otime_err);
	    if (otime_err < otemin || otime_err > otemax)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_laterr) {	/* ERROR IN LATITUDE */
	    lat_err = (linelen < DIST_OFFSET+DIST_LEN) ? no_lat_err :
		DDECODE(line+LAT_ERR_OFFSET, LAT_ERR_LEN, atof, no_lat_err);
	    if (lat_err < laemin || lat_err > laemax)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_lonerr) {	/* ERROR IN LONGITUDE */
	    lon_err = (linelen < DIST_OFFSET+DIST_LEN) ? no_lon_err :
		DDECODE(line+LON_ERR_OFFSET, LON_ERR_LEN, atof, no_lon_err);
	    if (lon_err < loemin || lon_err > loemax)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_deperr) {	/* ERROR IN DEPTH */
	    depth_err = (linelen < DIST_OFFSET+DIST_LEN) ? no_depth_err :
		DDECODE(line+DEPTH_ERR_OFFSET, DEPTH_ERR_LEN, atof, no_depth_err);
	    if (depth_err < zerrmin || depth_err > zerrmax) 
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_poly) {		/* POLYGONAL REGION */
	    /* Note: do_poly implies do_lat and do_lon */
	    if (inpoly(lat, lon, plat, plon, nvert) == 0)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_delta) {		/* EPICENTRAL DISTANCE */
	    /* Note: do_delta implies do_lat and do_lon */
	    lat = ggtogc(RAD*lat);
	    ct1 = sin(lat);
	    st1 = cos(lat);
	    cdlon = cos(RAD*lon - phi0);
	    cdelt = st0*st1*cdlon + ct0*ct1;
	    if (cdelt > cdmin || cdelt < cdmax)
	      if (negate) goto OUTPUT;
	      else        continue;
	    /* sdlon = sin(lon - phi0);					*/
	    /* cvrpd(st0*ct1-st1*ct0*cdlon, st1*sdlon, &sdelt, az0);	*/
	    /* *delta = atan2(sdelt, cdelt);				*/
	    /* cvrpd(st1*ct0-st0*ct1*cdlon, -sdlon*st0, &sdelt, az1);	*/
	}
	if (negate)
	  continue;


	/*
	 * now do output
	 */
		
      OUTPUT:
	fputs(line,stdout); 
	++selected;
    }
    if (verbose) fprintf (stderr,"%d lines, %d selected\n", lines, selected);
}
