'\" t
.TH EQSELECT 1Q "26 April 1993"
.SH NAME
.B eqselect
\- filter to select earthquakes satisfying user-defined criteria
.SH SYNTAX
.B eqselect
.RI [ option ...]
.RI [ file ...]
.SH DESCRIPTION
.B eqselect
reads earthquake catalog cards of various formats from an on-line
catalog, specified 
.IR files
or standard input,
and outputs those whose locations, magnitudes, or other characteristics
agree with criteria specified in the command-line \fIoption\fRs.
If neither the 
.B -catalog
option or specific files are specified, input data
will read from \fIstdin\fR.

.B eqselect
writes the selected information to stdout.  If you want the selected
information to be written to an output file, use the shell I/O redirection
.br
	eqselect ...options... > outputfilename
.br
to send the output to a file.
.SH OPTIONS
.PP
The following command-line \fIoption\fRs can be used to control the program.
Options are scanned from left to right;
if conflicting \fIoption\fRs are given, the rightmost one takes precedence.
Only enough characters need be given to uniquely identify an \fIoption\fR.
.TP
\fB-help\fR [\fIoption\fR ...]
Print brief explanations of the meanings of the named \fIoption\fRs.
If no \fIoption\fRs are named, descriptions are printed for all \fIoptions\fR.
The explanations are written to the standard error output.
.TP
\fB-format\fI ncsn | mech | ucb | cnss\fR
Selects the format of the files to be searched.  
This option only specifies what format to expect when reading input data.
It does NOT imply that the data will come from the online catalog for this
format.  If you wish to read data from the online catalog,
you must use the 
.B -catalog
option.
\fBeqselect\fR can search events that are in one of several format.
The table below outlines the currently supported formats:

.TS
l l
l l .
Format	Description
_
ncsn	NCSN earthquake catalog format (HYPOINVERSE).
mech	NCSN earthquake first motion mechanism format.
cnss	CNSS earthquake catalog format.
ucb	UCB earthquake catalog format.
.TE

The default format is \fIncsn\fR.
.TP
\fB-catalog\fR 
Specifies that the on-line data files should be searched in
addition to any specified input files.  Note that input will NOT be read
from stdin when this option is used.  The location of the on-line data is
dependent on the 
.I format
option for the data.  For each format, a variable specifies the directory
that contains the on-line data for that format.  The variable may be either
an environment variable or a variable in a system-wide LOCATIONS file.  The
following table shows the variable used for each format.  These values of
these variables should be already defined by the systems administrator.

.TS
l | l | l
l | l | l .
Format	Variable	Description
_
ncsn	CUSPCATDIR	Dir containing NCSN event catalog.
mech	CUSPMECHDIR	Dir containing NCSN focal mechanism data.
cnss	CNSSCATDIR	Dir containing CNSS event catalog.
ucb	UCBCATDIR	Dir containing UCB event catalog.
.TE
.TP
\fB-latitude\fI min max\fR
Selects earthquakes with latitudes between
.I min
and
.IR max .
These limits are measured in degrees,
and may optionally be followed immediately by one or two fields
specifying minutes and seconds.
If multiple fields are used,
they must be separated from each other by colons \fB:\fR\ .
For example,
.B 37.5
and
.B 37:30
both specify the same latitude -- the first specifies it in
decimal degrees, and the second specifies it in degrees and minutes.
.TP
\fB-longitude\fI min max\fR
Selects earthquakes with longitudes between
.I min
and
.IR max .
These are in the same format as the latitude limits,
described above.
(Note:  West longitudes are negative!)
.TP
\fB-depth\fI min max\fR
Selects earthquakes with focal depths between
.I min
and
.I max
(km).
.TP
\fB-time\fI start end\fR
Selects earthquakes with origin times between the epochs
.I start
and
.I end,
which are given in the form
[cc]yymmddhhmmss .
If the century is not given, the twentieth century is assumed.
The seconds field may contain a decimal point and a fractional part,
but in any case the integral part must be two digits,
even if they are zeros.
An epoch may optionally be followed by an offset consisting of a
.B +
or
.B \-
sign and one or more occurrences of a number followed by a
.BR d ,
.BR h ,
.BR m ,
or
.B s
character
(for days, hours, minutes, or seconds).
For example,
.B +1.5d
and
.B +1d12h
mean the same thing.
This option is also used with the \fB-catalog\fR entry to limit the number of
catalog files to search.
.TP
\fB-magnitude\fI min max\fR
Selects earthquakes with magnitudes between
.I min
and
.I max.
The specific field(s) in the input data used for the magnitude are
specified by the
.B -mtype
option.
.TP
\fB-mtype\fI type_1,type_2,...,type_n
Specifies an ordered list of magnitude types that should be used for the 
magnitude selection specified by the 
.B -magnitude 
option.  If the magnitude value is empty (or zero) for a
specified magnitude type, the value for the next magnitude type will be
examined.  Since not all events have values for all magnitude types,
this allow you to specify a preferred list of magnitudes to used when searching
on magnitude values.
The current magnitude types are:

.TS
l l l
l l l .
magtype	formats	description
_
ml	ncsn,ucb	Richter ML magnitude from BKS,ARC,MIN,MHC.
mml	ncsn	ML computed by Menlo from UCB amplitudes.
coda	ncsn	Coda magnitude.
amp	ncsn	Amplitude magnitude.
avg	ncsn,mech	Average of Coda and Amp magnitudes.
mln	ucb	"Network" ML (from all UCB stations).
bm	ucb	Simulated ML from 14-kg Benioff seismometer.
mw	ucb	Momemt magnitude Mw.
.TE

The default mtype search orders are:

.TS
l l
l l .
Input data format	Default mtype list
_
ncsn	mml,ml,coda,amp
mech	avg
ucb	mln,ml,bm
cnss	preferred magnitude in catalog
.TE
.TP
\fB-etype\fI letters\fR 
Selects catalog entries for any of the specified event types.  Valid event types are:  A for ALL events, E
for Earthquakes (including Felt earthquakes), Q for Quarry blasts, N for
Nuclear tests, R for Reflection/Refraction blasts, and F for Felt
Earthquakes (not currently used).  The difference between upper and lower
case letters is ignored.  The default event type is E.  
[\fIncsn\fR format only]
.TP
\fB-closest\fI min max\fR
Selects earthquakes with distances to the closest station between
.I min
and
.I max
(km).
.TP
\fB-nstations\fI min max\fR
Selects earthquakes with the number of stations between
.I min
and
.I max.
For the \fIncsn\fR format, this uses columns 37-39.
.TP
\fB-rms\fI min max\fR
Selects earthquakes with root-mean-squared residuals between
.I min
and
.I max
(s).
.TP
\fB-gap\fI min max\fR
Selects earthquakes with azimuthal gaps between
.I min
and
.I max
(deg).
.TP
\fB-quality\fI letters\fR
Selects earthquakes with specified quality codes (e.g.
.BR acd ).
The difference between upper and lower case letters is ignored.
[\fIucb\fR format only]
.TP
\fB-m0\fI min max\fR
Selects earthquakes with scalar momemt M0 between
.I min
and
.I max.
[\fIucb\fR format only]
.TP
\fB-nm0\fI min max\fR
Selects earthquakes that where the number of stations used to determine the 
scalar moment M0 is between
.I min
and
.I max
stations.
[\fIucb\fR format only]
.TP
\fB-nmag\fI min max\fR
Selects earthquakes that where the number of stations used to determine the 
magnitude is between
.I min
and
.I max
stations.
The specific field(s) in the catalog card used to determine magnitude is
specified with the
.B -mtype
option.
[\fIucb\fR format only]
.TP
\fB-errtime\fI min max\fR
Selects earthquakes where the error in the origin time of the event is between
\fImin\fR and \fImax\fR.
[\fIucb\fR format only]
.TP
\fB-errlatitude\fI min max\fR
Selects earthquakes where the error in the latitude of the event is between
\fImin\fR and \fImax\fR.
[\fIucb\fR format only]
.TP
\fB-errlongitide\fI min max\fR
Selects earthquakes where the error in the longitude of the event is between
\fImin\fR and \fImax\fR.
[\fIucb\fR format only]
.TP
\fB-errhorizontal\fI min max\fR
Selects earthquakes where the error in the horizontal location is between
\fImin\fR and \fImax\fR km.
[\fIncsn, mech, cnss\fR format only]
.TP
\fB-errdepth\fI min max\fR
Selects earthquakes where the error in the deptch is between
\fImin\fR and \fImax\fR km.
.TP
\fB-felt\fR
Selects earthquakes that were reported as being felt by one or more people.
[\fIucb\fR format only]
.TP
\fB-polygon\fI lat lon...\fR
Selects earthquakes within a polygon.
The polygon is specified as three or more pairs of values,
each pair giving the latitude and longitude of a vertex of the polygon.
The format of these latitudes and longitudes is the same as that
for latitude limits, described above.
(Note:  West longitudes are negative!)
.TP
\fB-delta\fI lat lon min max\fR
Selects earthquakes whose distance from the point
.RI ( lat ,
.IR lon )
is between
.I min
and
.IR max .
The format for
.I lat
and
.I lon
is the same as that for latitude limits, described above.
(Note:  West longitudes are negative!)
.I Min
and
.I max
are measured in kilometers.
.TP
\fB-negate\fR
print events that FAIL the specified selection criteria instead of printing
events that meet the selection criteria.
.SH EXAMPLES
.PP
To select earthquakes larger than magnitude 3
from the entire on-line catalog:
.IP
.B
eqselect -catalog -mag 3 10
.PP
To select earthquakes within a two-day interval from the NCSN catalog
.IP
.B
set t=840301123456.7
.br
.B
eqselect -catalog -time $t-2d $t
.PP
To select earthquakes within a two-day interval from the UCB catalog
.IP
.B
set t=840301123456.7
.br
.B
eqselect -catalog -format ucb -time $t-2d $t
.PP
To select earthquakes from the catalog using a number of options,
which are stored in the file
.B option_file
(Note the use of grave accents, not to be confused with apostrophes):
.IP
eqselect -format ncsn -catalog \`cat option_file\`
.B
.PP
To re-select events that were previously extracted into multiple files ending in
.BR .subset
use one of the following:
.IP
cat \fR*\fB.subset | eqselect \fR ...
.br
eqselect \fR ... \fB*.subset
.B
.PP
To select earthquakes larger than magnitude 5
from the on-line NCSN mechanism files:
.IP
.B
eqselect -catalog -format mech -mag 5 10
.SH NOTES
It is STRONLY suggested that you limit your searches whenever possible with
the \fB-time\fR option whenever you use the \fB-catalog\fR option.
Otherwise, all catalog files on the system will be searched, and this can
take a considerable amount of time.
.SH ENVIRONMENT
The following variables must be defined in the shell in which 
\fBextract\fR is invoked:
.TP
.SB CUSPCATDIR
Pathname of the directory hierarchy containing all of the NCSN catalog data.
.TP
.SB UCBCATDIR
Pathname of the directory hierarchy containing all of the UCB catalog data.
.SH SEE ALSO
extract(1), catalog(5), ncsn(5) ncsn.phase(5), ncsn.catalog(5), ucb(5), ucb.catalog(5), ucb.phase(5)
.SH AUTHOR
Adapted from qselect program Bruce R. Julian, USGS Menlo Park, Calif.
.br
Modified by Doug Neuhauser, UCB Seismographic Station, Calif.
