#include    <stdio.h>
#include    <sys/types.h>
#include    <sys/stat.h>
#include    <dirent.h>
#include    <string.h>

#include <local/local.h>
#include <local/date_time.h>

static char **p_catalogfiles = NULL;
static int  n_catalogfiles = 0;
static TIME catmin;
static TIME catmax;
static int  catmin_year = 0;
static int  catmin_month = 0;
static int  catmax_year = 0;
static int  catmax_month = 0;

/* Limits */
#define	MAXCATALOGFILES	1023		/* Max # of catalog files to search */
#define	    MAXPATHLEN	1024
#define	DATE_GE(y1,m1,y2,m2)	(y1>y2 | (y1==y2 && m1>=m2))
#define	DATE_LE(y1,m1,y2,m2)	(y1<y2 | (y1==y2 && m1<=m2))
#define	YEAR_GE(y1,y2)		(y1>=y2)
#define	YEAR_LE(y1,y2)		(y1<=y2)

/*  Simple sort of filenames.  Since year and month are first,		*/
/*  we can use standard string sort.					*/
cmp_catalog_filename(pp1, pp2) char **pp1, **pp2; { return (strcmp(*pp1,*pp2)); }

/*  Determine whether we want to search this catalog file.		*/
/*  NOTE:   Decision is based on syntax of the filename.		*/
/*	    This routine will have to change if the filename structure	*/
/*	    is ever changed.						*/

/*  The following filename formats will be recognized:			*/
/*	1.  year.month.anything		- monthly file.			*/
/*	2.  year.anything		- yearly file.			*/
/*	3.  year-year.anything		- range of years file.		*/
/*	4.  anything			- file for all dates.		*/

catalog_file_in_range (filename)
    char *filename;
{
    int year, month, year1, year2;
    char *p;
    char *file = ((p = strrchr(filename,'/'))!=NULL) ? ++p : filename;

    /*	1.  year.month.anything		- monthly file.			*/
    if ((sscanf (file, "%d.%d.", &year, &month)) == 2)
    return ((catmin_year == 0 || DATE_GE(year,month,catmin_year,catmin_month)) &&
	    (catmax_year == 0 || DATE_LE(year,month,catmax_year,catmax_month)));
    
    /*	2.  year.anything		- yearly file.			*/
    if ((sscanf (file, "%d.", &year)) == 1)
    return ((catmin_year == 0 || YEAR_GE(year,catmin_year)) &&
	    (catmax_year == 0 || YEAR_LE(year,catmax_year)));

    /*	3.  year-year.anything		- range of years file.		*/
    if ((sscanf (file, "%d-%d.", &year1, &year2)) == 2)
    return ((catmin_year == 0 || YEAR_GE(year2,catmin_year)) &&
	    (catmax_year == 0 || YEAR_LE(year1,catmax_year)));

    /*	4.  anything			- file for all dates.		*/
    return (1);

}

/*  Recursively search the directory for catalog files.			*/
find_catalog_files (dir)
    char *dir;
{
    char filename[MAXPATHLEN];
    DIR *dirp;
    struct dirent *direntp;
    struct stat st;
    int dir_len = strlen(dir)+1;
    int dir_name_len;
    strcpy (filename, dir);
    strcat (filename, "/");
    
    if ((dirp=opendir (dir)) == NULL) return (-1);
    while ((direntp=readdir(dirp))!=NULL) {
	/*  Ignore all entries beginning with ".".			*/
	if (direntp->d_name[0]=='.') continue;
	if (direntp->d_name[0]==0) {
		fprintf (stderr,"Entry in %s starts with null\n", dir);
		fflush (stderr);
		exit(1);
	}
#ifdef SUNOS
	dir_name_len=direntp->d_namlen;
#else
	dir_name_len=strlen(direntp->d_name);
#endif
	if (dir_len + dir_name_len + 1 >= MAXPATHLEN) continue; 
	filename[dir_len] = 0;
	strcat (filename,direntp->d_name);
	if (stat(filename, &st)!= 0) continue;
	if (S_ISDIR(st.st_mode)) {
	    find_catalog_files(filename);
	}
	else if (S_ISREG(st.st_mode) && catalog_file_in_range (filename)) {
	    if ((p_catalogfiles[n_catalogfiles]=(char *)malloc(strlen(filename))) == NULL) {
		fprintf (stderr, "Unable to malloc space for catalog filename\n");
		exit(1);
	    }
	    strcpy (p_catalogfiles[n_catalogfiles++],filename);
	}
    }
    closedir (dirp);
}

/*  Find the root for the catalog directory, and search for catalog	*/
/*  files that are within our search time interval.			*/
char **find_catalog (catdir_key, minyear, maxyear,key)
    char *catdir_key;
    TIME minyear, maxyear;
{
    char catdir[256];
    char filename[1024];
    int	day, hour, min;
    double sec;
    
    catmin = minyear;
    catmax = maxyear;
    getlocation (catdir_key,catdir, 1);
    n_catalogfiles = 0;
    if (p_catalogfiles == NULL) 
	p_catalogfiles = (char **)malloc(sizeof(char *)*(MAXCATALOGFILES+1));
    if (p_catalogfiles == NULL) {
	fprintf (stderr, "unable to malloc table for catalog files\n");
	return(NULL);
    }

    /*	If we have a start and end date, only process through them.	*/
    /*	Otherwise we have to scan the entire catalog.  Either way	*/
    /*	we need to return the list in date-sorted order.  We can assume	*/
    /*	that the file sort will do that.				*/
    
    if (catmin != 0.0) datime (catmin, &catmin_year, &catmin_month, &day, 
			      &hour, &min, &sec, GREGORIAN);
    if (catmax != 0.0) datime (catmax, &catmax_year, &catmax_month, &day,
			      &hour, &min, &sec, GREGORIAN);
    /*  Perform recursive search of directories looking for files.	*/
    find_catalog_files (catdir);
    p_catalogfiles[n_catalogfiles]=NULL;
    qsort (p_catalogfiles, n_catalogfiles, sizeof(char *), cmp_catalog_filename);
    return (p_catalogfiles);
}

FILE *ecfopen(file, mode)
    char *file, *mode;
{
    char cmd[MAXPATHLEN+100];
    FILE *fp;
    int n;
    int compressed = ((n=strlen(file))>2) ? (strcmp(file+n-2,".Z")==0) : 0;
    if (compressed) {
	sprintf (cmd, "uncompress -c %s", file);
	fp = popen (cmd, mode);
    }
    else fp = fopen (file, mode);
    if (fp ==NULL) {
	fprintf (stderr, "Unable to open file %s\n", file);
	exit(1);
    }
    return (fp);
}

int ecfclose (fp)
    FILE *fp;
{
    int status;
    status = pclose (fp);
    if (status < 0) status = fclose (fp);
    return (status);
}
