#ifndef lint
static char rcsid[]="$Header: inpoly.c,v 1.2 86/10/24 16:41:13 julian Exp $";
#endif lint
/*
 * Determine if a point is inside a polygon
 * Winding number algorithim
 * Bruce R. Julian, USGS Menlo Park, CA
 * After Godkin and Pulli, BSSA 74 no. 5, p. 1845-1848, 1984
 * Returns 0 if point outside polygon
 *	   1 if point at a vertex
 *	 +-2 if point on an edge
 *	 +-4 if point inside
 * + : clockwise circuit;	 - : anti-clockwise
$Log:	inpoly.c,v $
 * Revision 1.2  86/10/24  16:41:13  julian
 * rcsid variable hidden from lint.
 * 
 * Revision 1.1  86/05/16  15:45:44  julian
 * Initial revision
 * 
 */

#define AXIS 4
#define ORIGIN (AXIS/2)

static double	x1, y1, x2, y2;
static double	y1y2, t;
static int 	vertex, inside;

int inpoly(x0, y0, px, py, n)
	double	x0, y0;		/* Point in question		*/
	double	*px, *py;	/* Vertices of polygon		*/
				/* Last point == first point	*/
	int	n;		/* No. of vertices		*/
{
	inside = 0; vertex = 0;
	x1 = *px++ - x0;
	y1 = *py++ - y0;
	n--;
	while (n--) {
		if (vertex)
			return 1;
		x2 = *px++ - x0;
		y2 = *py++ - y0;
		if ((y1y2 = y1*y2) <= 0)	/* Put here for efficiency */
			inside += sicr();
		x1 = x2; y1 = y2;
	}
	return(inside);
}

/* Signed crossing number			*/
/*   0:  Doesn't cross -x axis 			*/
/*   0 (with vertex set): one end at origin	*/
/* +-ORIGIN:  Goes through origin		*/
/* +-AXIS:    Crosses -x axis			*/
/* +-AXIS/2:  "Half crosses" -x axis		*/
int sicr()
{
	/* Test for y1*y2 <= 0 is performed in calling routine	*/
	/* Otherwise it would be placed here and return zero	*/

	t = x1*y2 - x2*y1;
	if (t == 0 && (x1*x2 <= 0 || y1y2 <= 0)) { /* Passes through origin */
		if (y1y2 != 0)		return (y2 > 0 ? ORIGIN : -ORIGIN);
		vertex = (y1 != 0 || y2 != 0);
	}
	else if (y1y2 < 0) {	/* Complete crossing */
		if (t*y2 <= 0)		return (y2 > 0 ? AXIS : -AXIS);
	}
	else {			/* Half crossing */
		if (y1 == 0) {
			if (x1 < 0)	return (y2 > 0 ? AXIS/2 : -AXIS/2);
		}
		else {	/* y2 == 0 */
			if (x2 < 0)	return (y1 < 0 ? AXIS/2 : -AXIS/2);
		}
	}
	return 0;
}
