#ifndef lint
static char rcsid[]="$Header: qselect.c,v 1.13 92/08/13 19:04:44 julian Exp $";
#endif lint
/*
 * Select event "summary cards" matching specified criteria
 * Bruce R. Julian, USGS Menlo Park, CA
 * Significantly altered by Doug Neuhauser, Seismological Laboratory, UCB
 */

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <sys/param.h>
#include <local/local.h>
#include <local/date_time.h>
#include <local/cmd_opt.h>
#include <local/mathconst.h>
#include <local/earthconst.h>
#include <local/decode.h>

/* Types of input formats. */
#define	NCSN_CATALOG_AUTO	  0
#define	NCSN_CATALOG2K		  1
#define	NCSN_CATALOG		  2
#define	NCSN_MECH_AUTO		  3
#define	NCSN_MECH2K		  4
#define	NCSN_MECH		  5
#define UCB_CATALOG		  6
#define CNSS_CATALOG		  7
#define	REDI_CATALOG		  8
#define FULL_CNSS_CATALOG         9

#define		ALLOC_EXTERN			/* Allocate externals.	*/
#include "global.h"
#include "procs.h"

extern int search_ucb_catalog();
extern int search_ncsn_catalog();
extern int search_ncsn_mech();
extern int search_cnss_catalog();
extern int search_redi_catalog();
extern int search_full_cnss_catalog();
extern int search_ncsn_catalog2k();
extern int search_ncsn_mech2k();
double calc_delta();

extern FILE *ecfopen();

/*  Magnitude searches will be done on based on a preference list, 	*/
/*  since there are numerous magnitudes and not all events have values	*/
/*  for all of the magnitudes.						*/

/* Table that defines magtype strings and corresponding values.		*/
struct _magtypes { 
    char *name; 
    int	val;
} magtypes[MAXMAGTYPES] = { 
    "pref",	PREF_MAG,
    "ml",	ML_MAG,
    "mlt",	ML_MAG,
    "mln",	MLN_MAG,
    "mlm",	MML_MAG,
    "mlb",	BM_MAG,
    "md",	CODA_MAG,
    "mx",	AMP_MAG,
    "avg",	AVG_MAG,
    "mw",	MW_MAG,
    "bm",	BM_MAG,	    /* depricated string */
    "amp",	AMP_MAG,    /* depricated string */
    "coda",	CODA_MAG,   /* depricated string */
    "mml",	MML_MAG,    /* depricated string */
    NULL,	0 };

/*  Default preference order for each catalog type.			*/
static MAGS	ucb_catalog_mags[MAXMAGTYPES] =
    { 4, MW_MAG, MLN_MAG, ML_MAG, BM_MAG };
static MAGS	ncsn_mech_mags[MAXMAGTYPES] =
    { 1, AVG_MAG };
static MAGS	ncsn_catalog_mags[MAXMAGTYPES] =
    { 6, PREF_MAG, MW_MAG, MML_MAG, ML_MAG, CODA_MAG, AMP_MAG };

static char ncsn_dir_chars[] = " NSns+-";

/**/
/*----  C O M M A N D - L I N E   A R G U M E N T   P R O C E S S I N G  ----*/
dcheck(mn, mx, s)	/* Be sure mn <= mx */
	double mn, mx;
	char *s;
{
	if (mn > mx) error("minimum %s value greater than maximum", s);
}

icheck(mn,mx,s) 	/* same as dcheck -- but for integers */
int mn,mx;
char *s;
{
	if (mn > mx) error("minimum %s value greater than maximum", s);
}

/* format_work -- decode the format given on the command line, if any */
format_work(format)
char *format;
{
    if (input_format) {
	fprintf (stderr, "error - only 1 format allowed\n");
	exit(1);
    }
    else if (!strcasecmp(format, "ncsn")) input_format = NCSN_CATALOG_AUTO;
    else if (!strcasecmp(format, "calnet")) input_format = NCSN_CATALOG_AUTO;
    else if (!strcasecmp(format, "mech")) input_format = NCSN_MECH_AUTO;
    else if (!strcasecmp(format, "ucb")) input_format = UCB_CATALOG;
    else if (!strcasecmp(format, "cnss")) input_format = CNSS_CATALOG;
    else if (!strcasecmp(format, "f_cnss")) input_format = FULL_CNSS_CATALOG;
    else if (!strcasecmp(format, "redi")) input_format = REDI_CATALOG;
    else {
	fprintf (stderr, "error - invalid format\n");
	exit(1);
    }
}

/* Convert string to angle */
/* "degrees[:minutes[:seconds]]" */
double angle(p)
char	*p;
{
	double	sign;
	double	d, m, s;

	sign = 1.0;
	if (*p == '+')
		p++;
	if (*p == '-') {
		sign = -1.0;
		p++;
	}
	d = atof(p);
	m = s = 0.;
	if ((p = index(p, ':')) != NULL) {
		m = atof(++p);
		if ((p = index(p, ':')) != NULL)
			s = atof(++p);
	}
	return sign*(d + (m + s/60.)/60.);
}

/* Check whether string looks like an angle */
bool
isangle(p)
	char	*p;
{
	if (p == (char *)NULL)
		return FALSE;
	while (*p)
		if (index("+-0123456789.:", *p++) == (char *)NULL)
			return FALSE;
	return TRUE;
}

/* Convert string to TIME */
/* "[cc]yymmddhhmmss.ss...[+-offset]" */
TIME epoch(s)
	char	*s;
{
	double	se;
	int	yr, mo, da, hr, mn;
	char	*p;
	double	offset();

	/* Get year (twentieth century default) */
	for (p=s; isdigit(*p); p++)
		;
	switch (p - s) {
	case 14:
		yr = DECODE(s, 4, atoi);
		s += 2;
		break;
/*::	Depricated - must specify full year. 
	case 12:
		yr = 1900 + DECODE(s, 2, atoi);
		break;
::*/
	default:
		error ("Bad date format: \"%s\"", s);
	}

	/* Get month, day, hour, minute, second */
	mo = DECODE(s+ 2, 2, atoi);
	da = DECODE(s+ 4, 2, atoi);
	hr = DECODE(s+ 6, 2, atoi);
	mn = DECODE(s+ 8, 2, atoi);
	se = atof(s+10);

	/* Skip to end of seconds field */
	s += 12;
	if (*s == '.')
		s++;
	while(isdigit(*s))
		s++;

	return timvar(yr, mo, da, hr, mn, se, GREGORIAN) + offset(s);
}

static double offset(s)
	char	*s;
{
	double	sign, x, offset;

	switch (*s) {
	case '\0':	return 0.;
	case '+':	sign = 1.;	break;
	case '-':	sign = -1.;	break;
	default:
		error("Illegal sign character \"%c\"", *s);
	}
	s++;
	offset = 0.;
	while (isdigit(*s) || *s == '.') {
		x = atof(s);
		while (isdigit(*s))	s++;
		if (*s == '.')		s++;
		while (isdigit(*s))	s++;
		switch (*s) {
		case 'd': case 'D': x *= DAY	; break;
		case 'h': case 'H': x *= HOUR	; break;
		case 'm': case 'M': x *= MINUTE	; break;
		case 's': case 'S': x *= SECOND	; break;
		default:
			error("Bad time offset unit \"%c\"", *s);
		}
		s++;
		offset += x;
	}
	return sign*offset;
}

/* Smaller of two double values */
double fmin2(a, b)
	double	a, b;
{
	return (a<b ? a : b);
}

/* Larger of two double values */
double fmax2(a, b)
	double	a, b;
{
	return (a>b ? a : b);
}

qhelp	(){ if(eargc>2) prt_help(); else prt_doc("[file...]"); }
qneg    (){ negate = TRUE; }
qlat    (){ latmin = angle(aarg()); latmax = angle(aarg()); dcheck(latmin, latmax, "latitude"); do_lat = TRUE; }
qlon    (){ lonmin = angle(aarg()); lonmax = angle(aarg()); dcheck(lonmin, lonmax, "longitude"); do_lon = TRUE; }
qdepth  (){ zmin   = narg(); zmax = narg(); dcheck(zmin, zmax, "depth"); do_depth = TRUE; }
qtime   (){ otmin  = epoch(aarg()); otmax = epoch(aarg()); dcheck(otmin, otmax, "time"); do_otime = TRUE; }
qmtype  (){ parsemag(aarg()); }
qmag    (){ magmin = narg(); magmax = narg(); dcheck(magmin, magmax, "magnitude"); do_mag = TRUE; }
qno_mag (){ do_nomag=TRUE;}
qclose  (){ clomin = narg(); clomax = narg(); dcheck(clomin, clomax, "close"); do_close = TRUE; }
qnsta   (){ nstmin = narg(); nstmax = narg(); icheck(nstmin, nstmax, "nstations"); do_nsta = TRUE; }
qrms    (){ rmsmin = narg(); rmsmax = narg(); dcheck(rmsmin, rmsmax, "rms"); do_rms = TRUE; }
qgap    (){ gapmin = narg(); gapmax = narg(); dcheck(gapmin, gapmax, "gap"); do_gap = TRUE; }
qcat    (){ search_catalog = TRUE; }
qverbose(){ verbose = TRUE; }
qformat (){ format_work(aarg()); }
qnmagobs(){ nmagmin = narg(); nmagmax = narg(); icheck(nmagmin, nmagmax, "nobs for mag"); do_mag_obs = TRUE; }
qm0	(){ m0min = narg(); m0max = narg(); dcheck(m0min, m0max, "M0"); do_m0 = TRUE; }
qnm0obs (){ nm0min = narg(); nm0max = narg(); icheck(nm0min, nm0max, "nobs for M0"); do_m0_obs = TRUE; }
qoterr  (){ otemin = narg(); otemax = narg(); dcheck(otemin, otemax, "o-time error"); do_oterr = TRUE; }
qlaterr (){ laemin = narg(); laemax = narg(); dcheck(laemin, laemax, "lat error"); do_laterr = TRUE; }
qlonerr (){ loemin = narg(); loemax = narg(); dcheck(loemin, loemax, "lon error"); do_lonerr = TRUE; }
qherr	(){ herrmin = narg(); herrmax = narg();	dcheck(herrmin, herrmax, "horizontal error"); do_horerr = TRUE; }
qzerr	(){ zerrmin = narg(); zerrmax = narg();	dcheck(zerrmin, zerrmax, "depth error"); do_deperr = TRUE; }
qfelt	(){ felt = 'F'; do_felt = TRUE; }
qqual   (){ char *p; quals=aarg(); for(p=quals;*p;p++) *p = (isupper(*p) ? tolower(*p) : *p); do_qual = TRUE; }
qetype	(){ char *p; etypes=aarg(); for(p=etypes;*p;p++) *p = (islower(*p) ? toupper(*p) : *p); do_etype = (strchr(etypes,'A')) ? FALSE : TRUE; }
qdelta  ()
{
	double	lat0, lon0;		/* lat and lon of initial point (degrees) */
	double	kmdelmin, kmdelmax;	/* dist of min and max annulus (km) */
	double	rdelmin, rdelmax;	/* dist of min and max annulus (radians) */
	double	delmin, delmax;		/* dist of min and max annulus (degrees) */
	double	rglat0;			/* initial point geocentric lat (radians) */
	double	glat0;			/* initial point geocentric lat (degrees) */
	double	pglat0;			/* polar geocentric lat (degrees) */
	double	dlonmin, dlonmax;	/* delta's lon min and max (degrees) */
	double	pglatmin;		/* delta's polar geocentric lat min (degrees) */
	double	pglatmax;		/* delta's polar geocentric lat max (degrees) */
	double  dellon;			/* delta longitude */
	double	dlatmin, dlatmax;	/* delta's lat min and max (degrees) */

	double	rplat;
	double	dlon, sdlon;
        double  gcl_0, elam_0, plat;
	
/* External variables */
/* See formula in Abromowitz & Stegun, p79 */
/*	double	phi0;			/* lon of initial point (radians) */
/*	double	st0;			/* sin of polar geocentric lat */
/*	double	ct0;			/* cos of polar geocentric lat */
/*	double	cdmin;			/* cos of delmin */
/*	double	cdmax;			/* cos of delmax */

	lat0 = angle(aarg());		/* latitude (degrees) */
	lon0 = angle(aarg());		/* longitude (degrees) */
	kmdelmin = narg();		/* deltamin (km) */
	kmdelmax = narg();		/* deltamax (km) */

	rdelmin = kmdelmin/REARTH;	/* delatmin (radians) */
	rdelmax = kmdelmax/REARTH;	/* deltamax (radians) */
	delmin = DEG*rdelmin;		/* deltamin (degrees) */
	delmax = DEG*rdelmax;		/* deltamax (degrees) */

	/* Check delmin and delmax for proper values. */
	dcheck(kmdelmin, kmdelmax, "delta");
	if (rdelmax > PI) error("max delta %lf value greater 1/2 earth circumference", kmdelmax);
	if (kmdelmin < 0) error("min delta %lf value < 0 ", kmdelmin);
	if (kmdelmax <= 0) error("max delta %lf value <= 0 ", kmdelmax);

	/* Do all possible preliminary arithmetic */
	phi0 = RAD*lon0;	/* longitude (radians) */
	rglat0 = ggtogc (RAD*lat0);	/* geocentric latitude (radians) */
	glat0 = DEG*rglat0;		/* geocentric latitude (degrees) */
	pglat0 = 90 - glat0;		/* polar geocentric latitude (degrees).    */
	
	st0 = sin(90*RAD - rglat0);	/* sin of polar geocentric lat */
	ct0 = cos(90*RAD - rglat0);	/* cos of polar geocentric lat */
	cdmin = cos(rdelmin);		/* cos of delmin */
	cdmax = cos(rdelmax);		/* cos of delmax */

	/* Shrink lat-lon limits to barely enclose delta annulus if possible.*/
	/* Determine whether any of the poles are within the annulus. */

	pglatmin = pglat0 - delmax;	/* min geocentric polar lat (degrees) */
	pglatmax = pglat0 + delmax;	/* max geocentric polar lat (degrees) */
	if (pglatmin < 0 && pglatmax > 180) {
	    /* Overshoot both poles - no bounds. */
	    pglatmin = 0;
	    pglatmax = 180;
	    dlonmin = -180;
	    dlonmax = 180;
	}
	else if (pglatmin < 0) {
	    /* Include and overshoot the north pole. */
	    pglatmin = 0;
	    dlonmin = -180;
	    dlonmax = 180;
	}
	else if (pglatmax > 180) {
	    /* Include and overshoot the south pole. */
	    pglatmax = 180;
	    dlonmin = -180;
	    dlonmax = 180;
	}
	else {
	    /* Compute latitude and longitude box. */
	    pglatmin = pglat0 - delmax;
	    pglatmax = pglat0 + delmax;
	    dellon = DEG*asin(sin(rdelmax)/sin(RAD*pglat0));
	    dlonmin = lon0 - dellon;
/*::	    if (dlonmin < -180) dlonmin = dlonmin + 360;*/
	    if (dlonmin < -180) wraplon = 1;
	    dlonmax = lon0 + dellon;
/*::	    if (dlonmax > 180) dlonmax = dlonmax - 360;*/
	    if (dlonmax > 180) wraplon = 1;
	}

	/* Compute final bounds. */
	dlatmin = DEG*gctogg(RAD*(90-pglatmax));
	dlatmax = DEG*gctogg(RAD*(90-pglatmin));
	latmin = (do_lat) ? fmax2 (latmin, dlatmin) : dlatmin;
	latmax = (do_lat) ? fmin2 (latmax, dlatmax) : dlatmax;
	lonmin = (do_lon) ? fmax2 (lonmin, dlonmin) : dlonmin;
	lonmax = (do_lon) ? fmin2 (lonmax, dlonmax) : dlonmax;
	do_delta = do_lat = do_lon = TRUE;

#ifdef	DEBUG_DELTA
	fprintf (stderr, "Initial delta values:\n");
	fprintf (stderr, "lat0=%lf lon0=%lf kmdelmin=%lf kmdelmax=%lf\n",
		 lat0, lon0, kmdelmin, kmdelmax);
	fprintf (stderr, "latmin=%lf latmax=%lf lonmin=%lf lonmax=%lf\n", 
	    latmin, latmax, lonmin, lonmax);
	fprintf (stderr, "wraplon=%d\n", wraplon);
	fprintf (stderr, "st0=%lf ct0=%lf phi0=%lf\n", st0, ct0, phi0);
	fprintf (stderr, "cdmin=%lf cdmax=%lf\n", cdmin, cdmax);
	fprintf (stderr, "\n");
#endif

}

qpoly   ()
{
	/* Read in vertices of polygon */
	for (nvert=0; isangle(peekarg()) && nvert<MAXVERT; nvert++) {
		plat[nvert] = angle(aarg());
		plon[nvert] = angle(aarg());
	}
	if (nvert < 3)
		error("Polygon needs at least three vertices");
	plat[nvert] = plat[0];
	plon[nvert] = plon[0];
	nvert++;

	/* Shrink latitude-longitude limits to just include polygon */
	if (! do_lat) { latmin =  -90.; latmax =  90.; }
	if (! do_lon) { lonmin = -180.; lonmax = 180.; }
	latmin = fmax2(latmin, dmin(plat, nvert));
	latmax = fmin2(latmax, dmax(plat, nvert));
	lonmin = fmax2(lonmin, dmin(plon, nvert));
	lonmax = fmin2(lonmax, dmax(plon, nvert));
	do_poly = do_lat = do_lon = TRUE;
}

double calc_delta(lat,lon)
     double lat,lon;
{
  double ct1,st1,cdlon,cdelt;

  lat = ggtogc(RAD*lat);
  ct1 = cos(90*RAD - lat);
  st1 = sin(90*RAD - lat);
  cdlon = cos(RAD*lon - phi0);
  cdelt = st0*st1*cdlon + ct0*ct1;
  return cdelt;
}

struct command cmd[] = {
   qhelp    , "-help"     , "[opt...]" , "Print documentation",
   qneg     , "-negate"   , ""         , "Print events that FAIL test",
   qtime    , "-time"     , "min max"  , "Origin-time interval (yyyymmddhhmmss[+-offset])",
   qlat     , "-latitude" , "min max"  , "Latitude range (d[:m[:s]])",
   qlon     , "-longitude", "min max"  , "Longitude range (d[:m[:s]]) N.B. West is negative!",
   qdepth   , "-depth"    , "min max"  , "Depth range (km)",
   qmtype   , "-mtype"    , "type,..." , "Magnitude type[s], in preferred order: (pref, md, mx, avg, ml, mln, mlm, mlb, mw)",
   qmag     , "-magnitude", "min max"  , "Magnitude range",
   qclose   , "-closest"  , "min max"  , "Range of closest station (km)",
   qnsta    , "-nstations", "min max"  , "Range of no. of stations",
   qrms     , "-rms"      , "min max"  , "Range of r.m.s. residual (s)",
   qgap     , "-gap"      , "min max"  , "Range of azimuthal gap (deg)",
   qdelta   , "-delta"    , "lat lon min max"  , "Epicentral distance (km)",
   qpoly    , "-polygon"  , "lat lon...","Polygon (give 3 or more vertices)",
   qcat     , "-catalog"  , ""         , "Search catalog - uses CUSPCATDIR, CUSPMECHDIR, UCBCATDIR, CNSSCATDIR, FULLCNSSCATDIR, or REDICATDIR variable",
   qverbose , "-verbose"  , ""         , "Verbose output during search",
   qformat  , "-format"   , "type"     , "Specify input data format: (ncsn, ucb, mech, cnss, or redi)",
   qqual    , "-quality"  , "letters"  , "Quality codes (e.g. abc)   [ucb only]",
   qetype   , "-etype"	  , "letters"  , "Event types (eg eqnrfa) [ncsn, cnss only]",
   qnmagobs , "-nmag"     , "min max"  , "Range of number of observations for magnitude   [ucb only]",
   qm0      , "-m0"	  , "min max"  , "Range of scalar moment   [ucb only]",
   qnm0obs  , "-nm0    "  , "min max"  , "Range of number of observations for scalar moment   [ucb only]",
   qoterr   , "-errtime"  , "min max"  , "Range of error in origin time (sec)   [ucb only]",
   qlaterr  , "-errlat"   , "min max"  , "Range of error in latitude (km)   [ucb only]",
   qlonerr  , "-errlon"   , "min max"  , "Range of error in longitude (km)   [ucb only]",
   qherr    , "-errhoriz" , "min max"  , "Range of error in horizontal location (km)   [ncsn, mech, cnss only]",
   qzerr    , "-errdepth" , "min max"  , "Range of error in focal depth (km)",
   qfelt    , "-felt"     , ""         , "Select felt events   [ucb cnss ncsn only]",
   qno_mag  , "-no_mag"   , ""         , "Include events which have no reported magnitude [cnss]",
};

#define MAXLINELEN 	200		/* Length of input line		*/

search_ncsn_catalog_auto (f)
    FILE 	*f;
{
    char line[MAXLINELEN];
    char *l = NULL;
    int i;
    if (input_format == NCSN_CATALOG_AUTO) {
	l = fgets(line, sizeof(line), f);
	if (l == NULL) return;
	if (strchr(ncsn_dir_chars,line[16]) && strchr(ncsn_dir_chars,line[24]))
	    input_format = NCSN_CATALOG;
	else if (strchr(ncsn_dir_chars,line[18]) && strchr(ncsn_dir_chars,line[26]))
	    input_format = NCSN_CATALOG2K;
   }
    switch (input_format) {
      case NCSN_CATALOG2K:
	search_ncsn_catalog2k (f,l);
	break;
      case NCSN_CATALOG:
	search_ncsn_catalog (f,l);
	break;
      default:
	fprintf (stderr, "Error: Unable to determine format for ncsn_catalog\n");
	exit(1);
    }
    return (0);
}

search_ncsn_mech_auto (f)
    FILE 	*f;
{
    char line[MAXLINELEN];
    char *l = NULL;
    int i;
    if (input_format == NCSN_MECH_AUTO) {
	l = fgets(line, sizeof(line), f);
	if (l == NULL) return;
	if (line[6] == ' ' && strchr(ncsn_dir_chars,line[20]))
	    input_format = NCSN_MECH;
	else if (line[8] == ' ' && strchr(ncsn_dir_chars,line[22]))
	    input_format = NCSN_MECH2K;
   }
    switch (input_format) {
      case NCSN_MECH2K:
	search_ncsn_mech2k (f,l);
	break;
      case NCSN_MECH:
	search_ncsn_mech (f,l);
	break;
      default:
	fprintf (stderr, "Error: Unable to determine format for ncsn_mech\n");
	exit(1);
    }
    return (0);
}

/*--------------------------  M A I N  ------------------------------*/

main(argc,argv)
	int	argc;
	char	*argv[];
{
	FILE *fpi_cat;
	char filename[MAXPATHLEN],*catdir;
	char *catalog_name;
	int (*search_f)();		/* ptr to func returning int.	*/
	char *location;			/* location of files.		*/

	/* Set defaults */
	negate = FALSE;
	search_catalog = verbose = FALSE;
	do_lat = do_lon = do_depth = do_otime = do_mag = 
	do_close = do_nsta = do_rms = do_gap = do_qual =
	do_delta = do_poly = do_nomag =FALSE;		/* Do nothing		*/

	do_etype = TRUE; etypes = "E";

	/* Interpret command-line options */
	PROCESS_OPTS;

	/* Establish defaults if not explicitly set. */
	if (!input_format) input_format = NCSN_CATALOG_AUTO;
	if ((do_mag || do_nomag) && mags.nprefs == 0) {
	    set_def_mags(input_format);
	}

	/* Process files (standard input default) */
	switch (input_format) {
	  case NCSN_CATALOG_AUTO:
	  case NCSN_CATALOG2K:
          case NCSN_CATALOG:
	    location = "CUSPCATDIR";
	    search_f = search_ncsn_catalog_auto;
	    break;
	  case NCSN_MECH_AUTO:
	  case NCSN_MECH2K:
	  case NCSN_MECH:
	    location = "CUSPMECHDIR";
	    search_f = search_ncsn_mech_auto;
	    break;
	  case UCB_CATALOG:
	    location = "UCBCATDIR";
	    search_f = search_ucb_catalog;
	    break;
	  case CNSS_CATALOG:
	    location = "CNSSCATDIR";
	    search_f = search_cnss_catalog;
	    break;
	  case FULL_CNSS_CATALOG:
	    location = "FULLCNSSCATDIR";
	    search_f = search_full_cnss_catalog;
	    break;
	  case REDI_CATALOG:
	    location = "REDICATDIR";
	    search_f = search_redi_catalog;
	    break;
	  default:
	    fprintf (stderr, "unrecognized input format - should never get here\n");
	    exit(1);
	}

	if ((iarg >= argc) && ! search_catalog) {
	    if (verbose) fprintf (stderr, "processing stdin ...\n");
	    (*search_f)(stdin);
	}

	else {
	    if (search_catalog) {
		TIME catmin = otmin;
		TIME catmax = otmax;
		char **cp, **find_catalog();

		if ((cp = find_catalog(location,catmin,catmax))==NULL) {
		    fprintf(stderr,"unable to find installed files for specified input format.\n");
		    exit(1);
		}
		while (*cp != NULL) {
		    FILE *f = ecfopen(*cp, "r");
		    if (verbose) fprintf(stderr,"processing %s ... ", *cp);
		    (*search_f)(f);
		    (void)ecfclose(f);
		    cp++;
		}
	    }
	    for (; iarg<argc; iarg++) {
		FILE *f = ecfopen(argv[iarg], "r");
		if (verbose) fprintf(stderr,"processing %s ... ", argv[iarg]);
		(*search_f)(f);
		(void)ecfclose(f);
	    }
	}
	exit(0);
}

set_def_mags(format)
     int format;
{
    switch (format) {
      case NCSN_CATALOG_AUTO:
      case NCSN_CATALOG2K:
      case NCSN_CATALOG:
	    bcopy (&ncsn_catalog_mags, &mags, sizeof (mags));
	break;
      case NCSN_MECH_AUTO:
      case NCSN_MECH2K:
      case NCSN_MECH:
	bcopy (&ncsn_mech_mags, &mags, sizeof (mags));
	break;
      case UCB_CATALOG:
	bcopy (&ucb_catalog_mags, &mags, sizeof (mags));
	break;
      case CNSS_CATALOG:
	bcopy ("", &mags, 1);
	break;
      case FULL_CNSS_CATALOG:
	bcopy ("", &mags, 1);
	break;
      case REDI_CATALOG:
	bcopy ("", &mags, 1);
	break;
      default:
	fprintf (stderr, "unknown magnitude pref list for this format\n");
	exit(1);
	break;
    }
}

/* Parse magnitude string into structure. */
int
parsemag(p)
     char *p;
{
    char *q;
    int i;

    while (p != NULL && *p !=  '\0' && mags.nprefs < MAXMAGTYPES) {
	q = strchr (p, ',');
	if (q != NULL) *q = '\0';
	for (i=0; i < MAXMAGTYPES; i++) {
	    if (magtypes[i].name != NULL && strcasecmp(p, magtypes[i].name)==0) {
		mags.magtype[mags.nprefs] = magtypes[i].val;
		++mags.nprefs;
		break;
	    }
	}
	if (i >= MAXMAGTYPES) {
	    fprintf (stderr, "invalid magtype: %s\n", p);
	    exit(1);
	}
	p = (q) ? q+1 : q;
    }
    return(1);
}

