#include <stdio.h>

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <sys/param.h>
#include <local/local.h>
#include <local/date_time.h>
#include <local/cmd_opt.h>
#include <local/mathconst.h>
#include <local/earthconst.h>
#include <local/decode.h>

#include "ncsn_mech2k.h"
#include "global.h"
#include "procs.h"

/************************************************************************/
/*  NOTE: Floating point fields in this format have explicit decimal	*/
/*  points, and therefore need NO explicit scaling.			*/
/************************************************************************/

/* Limits */
#define MAXLINELEN 	150		/* Length of input line		*/

/* match_ncsn_mech_mag -- match magnitude pref list for ncsn_mech	*/
double
match_ncsn_mech2k_mag (line) 
     char *line;
{
    double mag = 0.;
    int coda_mag = 0;
    int amp_mag = 0;
    int i;

    for (i=0; i<mags.nprefs; i++) {
	switch (mags.magtype[i]) {
	  case AVG_MAG:
	    mag =  DECODE(line+AVG_MAG_OFFSET, AVG_MAG_LEN, atof);
	    break;
	  case ML_MAG:
	  case MML_MAG:
	  case CODA_MAG:
	  case AMP_MAG:
	  case MLN_MAG:
	  case BM_MAG:
	  case MW_MAG:
	  default:
	    fprintf (stderr, "Specified magnitude not available for ncsn mech\n");
	    exit(1);
	    break;
	}
	if (mag != 0.) return (mag);
    }
    /* No magnitude was found.  Return 0. */
    return (0.);
}

search_ncsn_mech2k(f,first_line)
    FILE	*f;
    char *first_line;
{
    char		c;
    double		close;	/* Distance to closest station	*/
    double		ct1, st1;
    double		cdelt;
    double		cdlon;
    double		d, m;	/* Degrees, minutes		*/
    double		gap;	/* Largest azimuthal gap	*/
    double		horiz_err; /* Horizontal error		*/
    double		depth_err; /* Depth (Vertical) error	*/
    double		lat, lon, z; /* Hypocentral coordinates	*/
    double		mag;	/* Magnitude			*/
    int			nsta;		/* No. of stations		*/
    TIME		ot;	/* Origin time			*/
    double		rms;	/* Root-mean-square residual	*/
    char		line[MAXLINELEN],buf_line[MAXLINELEN];
    double		se;
    int			yr, mo, da, hr, mn;
    int			lines = 0;
    int			selected = 0;
    char		qua;

    /* Read lines, test, and write those that pass */
    while(first_line || fgets(line, sizeof(line), f) != NULL) {
      if (first_line) {
	strcpy (line, first_line);
	first_line = NULL;
      }
        strcpy(buf_line,line);
	++lines;
	if (do_otime) {		/* ORIGIN TIME */
	    yr = DECODE(line+YEAR_OFFSET, YEAR_LEN, atoi);
	    mo = DECODE(line+MONTH_OFFSET, MONTH_LEN, atoi);
	    da = DECODE(line+DAY_OFFSET, DAY_LEN, atoi);
	    hr = DECODE(line+HOUR_OFFSET, HOUR_LEN, atoi);
	    mn = DECODE(line+MINUTE_OFFSET, MINUTE_LEN, atoi);
	    se = DECODE(line+SECOND_OFFSET, SECOND_LEN, atof);
	    ot = timvar(yr, mo, da, hr, mn, se, GREGORIAN);
	    if (ot < otmin || ot > otmax)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_lat) {		/* LATITUDE */
	    d = DECODE(line+LAT_DEG_OFFSET, LAT_DEG_LEN, atof);
	    m = DECODE(line+LAT_MIN_OFFSET, LAT_MIN_LEN, atof);
	    lat = fabs(d) + m/60.;
	    if (line[LAT_DIR_OFFSET] == 'S')
	      lat *= -1.;
	    if (lat < latmin || latmax < lat)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_lon) {		/* LONGITUDE */
	    d = DECODE(line+LON_DEG_OFFSET, LON_DEG_LEN, atof);
	    m = DECODE(line+LON_MIN_OFFSET, LON_MIN_LEN, atof);
	    lon = fabs(d) + m/60.;
	    if (line[LON_DIR_OFFSET] != 'E')
	      lon *= -1.;
	    if (lon < lonmin || lonmax < lon)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_depth) {		/* FOCAL DEPTH */
	    z = DECODE(line+DEPTH_OFFSET, DEPTH_LEN, atof);
	    if (z < zmin || zmax < z)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_mag) {		/* MAGNITUDE */
	    mag = match_ncsn_mech2k_mag (line);
	    if ((mag < magmin || magmax < mag) && !(do_nomag && mag==0))
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_nsta) {		/* NUMBER OF STATIONS */
	    nsta = DECODE(line+NSTA_OFFSET, NSTA_LEN, atoi);
	    if (nsta < nstmin || nstmax < nsta)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_rms) {		/* RMS RESIDUAL */
	    rms = DECODE(line+RMS_OFFSET, RMS_LEN, atof);
	    if (rms < rmsmin || rmsmax < rms)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_gap) {		/* AZIMUTHAL GAP */
	    gap = DECODE(line+AZ_GAP_OFFSET, AZ_GAP_LEN, atof);
	    if (gap < gapmin || gapmax < gap)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_close) {		/* DISTANCE TO CLOSEST STATION */
	    close = DECODE(line+DIST_OFFSET, DIST_LEN, atof);
	    if (close < clomin || clomax < close)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_horerr) {	/* HORIZONTAL ERROR */
	    horiz_err = DECODE(line+HERR_OFFSET, HERR_LEN, atof) / 100.;
	    if (horiz_err < herrmin || herrmax < horiz_err)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_deperr) {	/* DEPTH (VERTICAL) ERROR */
	    depth_err = DECODE(line+VERR_OFFSET, VERR_LEN, atof) / 100.;
	    if (depth_err < zerrmin || zerrmax < depth_err)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_poly) {		/* POLYGONAL REGION */
	    /* Note: do_poly implies do_lat and do_lon */
	    if (inpoly(lat, lon, plat, plon, nvert) == 0)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_delta) {		/* EPICENTRAL DISTANCE */
	    /* Note: do_delta implies do_lat and do_lon */
	    lat = ggtogc(RAD*lat);
	    ct1 = sin(lat);
	    st1 = cos(lat);
	    cdlon = cos(RAD*lon - phi0);
	    cdelt = st0*st1*cdlon + ct0*ct1;
	    if (cdelt > 1.0) cdelt = 1.0;   /* Should never be > 1.0, but some is... */
	    if (cdelt > cdmin || cdelt <= cdmax)
	      if (negate) goto OUTPUT;
	      else        continue;
#ifdef	DEBUG_DELTA
	    fprintf (stderr, "Values for event:\n");
	    fprintf (stderr, "lat=%lf lon=%lf ct1=%lf st1=%lf\n", lat,lon,ct1,st1);
	    fprintf (stderr, "cdmin=%.16lf cdmax=%.16lf\n", cdmin, cdmax);
	    fprintf (stderr, "cdlon=%.20lf cdelt=%.20lf\n\n", cdlon, cdelt);
#endif
	    /* sdlon = sin(lon - phi0);					*/
	    /* cvrpd(st0*ct1-st1*ct0*cdlon, st1*sdlon, &sdelt, az0);	*/
	    /* *delta = atan2(sdelt, cdelt);				*/
	    /* cvrpd(st1*ct0-st0*ct1*cdlon, -sdlon*st0, &sdelt, az1);	*/
	}
	if (negate)
	  continue;
      OUTPUT:
	(void)fputs(line, stdout);
	++selected;
    }
    yr = DECODE(buf_line+YEAR_OFFSET, YEAR_LEN, atoi);
    mo = DECODE(buf_line+MONTH_OFFSET, MONTH_LEN, atoi);
    da = DECODE(buf_line+DAY_OFFSET, DAY_LEN, atoi);
    hr = DECODE(buf_line+HOUR_OFFSET, HOUR_LEN, atoi);
    mn = DECODE(buf_line+MINUTE_OFFSET, MINUTE_LEN, atoi);
    se = DECODE(buf_line+SECOND_OFFSET, SECOND_LEN, atof);
    otmin = timvar(yr, mo, da, hr, mn, se, GREGORIAN);

    if (verbose) fprintf (stderr,"%d buf_lines, %d selected\n", lines, selected);
}

