#include <stdio.h>

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <sys/param.h>
#include <local/local.h>
#include <local/date_time.h>
#include <local/cmd_opt.h>
#include <local/mathconst.h>
#include <local/earthconst.h>
#include <local/decode.h>

#include "redi_catalog.h"
#include "global.h"
#include "procs.h"

void dy_to_mdy();

/************************************************************************/
/*  NOTE: Floating point fields in this format have explicit decimal	*/
/*  points, and therefore need NO explicit scaling.			*/
/************************************************************************/

/* Limits */
#define	MAXLINELEN	200

search_redi_catalog(fp) 
     FILE *fp;
{
    int		i,j,k; 	
    char	line[MAXLINELEN],buf_line[MAXLINELEN];
    int 	num_mag_obs,num_m0_obs,num_sta,max_az_gap;
    double	depth,mag,m0,min_delta,close;
    double	otime_err,lat_err,lon_err,depth_err,rms,ot;
    double	lat,lon,cdelt,cdlon,ct1,st1;
    char	qua,felt;
    double	se;
    int		yr, mo, dy, da, hr, mn;
    int		lines = 0;
    int		selected = 0;

    while (fgets(line,MAXLINELEN,fp) != NULL) {
      strcpy(buf_line,line);
	++lines;
	/*
	 * now filter the event
	 */

	if (do_otime) {		/* ORIGIN TIME */
	    yr = DECODE(line+YEAR_OFFSET, YEAR_LEN, atoi);
            dy = DECODE(line+DOY_OFFSET, DOY_LEN, atoi);
            dy_to_mdy (dy, yr, &mo, &da);

	    hr = DECODE(line+HOUR_OFFSET, HOUR_LEN, atoi);
	    mn = DECODE(line+MINUTE_OFFSET, MINUTE_LEN, atoi);
	    se = DECODE(line+SECOND_OFFSET, SECOND_LEN, atof);
	    ot = timvar(yr, mo, da, hr, mn, se, GREGORIAN);
	    if (ot < otmin || ot > otmax)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_lat) {		/* LATITUDE */
	    lat = DECODE(line+LAT_OFFSET, LAT_LEN, atof);
	    if (lat < latmin || latmax < lat)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_lon) {		/* LONGITUDE */
	    lon = DECODE(line+LON_OFFSET, LON_LEN, atof);
#ifdef	DEBUG_DELTA
	    fprintf (stderr, "Lon check: ");
#endif
	    if ( (lon < lonmin || lonmax < lon) && (! (wraplon && ((lon+360 <= lonmax) || (lon-360 >= lonmin)) ) ) )
#ifdef	DEBUG_DELTA
	    {
	      fprintf (stderr, "Outside: lon=%lf\n", lon);
	      if (negate) goto OUTPUT;
	      else        continue;
	    }
	    fprintf (stderr, "Inside: lon=%lf\n", lon);
#else
	      if (negate) goto OUTPUT;
	      else        continue;
#endif
	}
	if (do_depth) {		/* FOCAL DEPTH */
	    depth = DECODE (line+DEPTH_OFFSET, DEPTH_LEN, atof);
	    if (depth < zmin || zmax < depth)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_mag) {		/* MAG */
	    mag = DECODE (line+MAG_OFFSET, MAG_LEN, atof);
	    if (mag < magmin || magmax < mag)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_nsta) {		/* NUMBER OF STATIONS */
	    num_sta = DECODE (line+NSTA_OFFSET, NSTA_LEN, atoi);
	    if (num_sta < nstmin || nstmax < num_sta)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_rms) {		/* RMS RESIDUAL */
	    rms = DECODE(line+RMS_OFFSET, RMS_LEN, atof);
	    if (rms < rmsmin || rmsmax < rms)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_qual) {		/* "QUALITY" CODE */
	    qua = *(line+QUAL_OFFSET);
	    if (!index(quals, (isupper(qua) ? tolower(qua) : qua)))
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_mag_obs) {	/* NUMBER OF OBSERVATIONS USED TO GET MAG */
	    num_mag_obs = DECODE(line+N_MAG_OFFSET, N_MAG_LEN, atoi);
	    if (num_mag_obs < nmagmin || nmagmax < num_mag_obs)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_poly) {		/* POLYGONAL REGION */
	    /* Note: do_poly implies do_lat and do_lon */
	    if (inpoly(lat, lon, plat, plon, nvert) == 0)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_delta) {		/* EPICENTRAL DISTANCE */
	    /* Note: do_delta implies do_lat and do_lon */
	    lat = ggtogc(RAD*lat);
	    ct1 = cos(90*RAD - lat);
	    st1 = sin(90*RAD - lat);
	    cdlon = cos(RAD*lon - phi0);
	    cdelt = st0*st1*cdlon + ct0*ct1;
	    if (cdelt > 1.0) cdelt = 1.0;   /* Should never be > 1.0, but some is... */
	    if (cdelt > cdmin || cdelt <= cdmax)
	      if (negate) goto OUTPUT;
	      else        continue;
#ifdef	DEBUG_DELTA
	    fprintf (stderr, "Values for event:\n");
	    fprintf (stderr, "lat=%lf lon=%lf ct1=%lf st1=%lf\n", lat,lon,ct1,st1);
	    fprintf (stderr, "cdmin=%.16lf cdmax=%.16lf\n", cdmin, cdmax);
	    fprintf (stderr, "cdlon=%.20lf cdelt=%.20lf\n\n", cdlon, cdelt);
#endif
	    /* sdlon = sin(lon - phi0);					*/
	    /* cvrpd(st0*ct1-st1*ct0*cdlon, st1*sdlon, &sdelt, az0);	*/
	    /* *delta = atan2(sdelt, cdelt);				*/
	    /* cvrpd(st1*ct0-st0*ct1*cdlon, -sdlon*st0, &sdelt, az1);	*/
	}
	if (negate)
	  continue;


	/*
	 * now do output
	 */
		
      OUTPUT:
	fputs(line,stdout); 
	++selected;
    }
    yr = DECODE(buf_line+YEAR_OFFSET, YEAR_LEN, atoi);
    dy = DECODE(line+DOY_OFFSET, DOY_LEN, atoi);
    dy_to_mdy (dy, yr, &mo, &da);

    hr = DECODE(buf_line+HOUR_OFFSET, HOUR_LEN, atoi);
    mn = DECODE(buf_line+MINUTE_OFFSET, MINUTE_LEN, atoi);
    se = DECODE(buf_line+SECOND_OFFSET, SECOND_LEN, atof)/100.;
    otmin = timvar(yr, mo, da, hr, mn, se, GREGORIAN);

    if (verbose) fprintf (stderr,"%d lines, %d selected\n", lines, selected);
}

/************************************************************************/
/*	Information on dates (for non-leap years)			*/
/*		Names of months, days per months, and day of year.	*/

char	    *MON[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", 
			"Jul", "Aug", "Sep", "Oct", "Nov", "Dec", NULL};
/*		         Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec */
int	    DPM[] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
int	    DOY [] = { 0, 31, 59, 90,120,151,181,212,243,273,304,334,365 };

#define	IS_LEAP(yr)	( yr%400==0 || (yr%4==0 && yr%100!=0) )
#define	LDOY(y,m)	(DOY[m] + (IS_LEAP(y) && m >= 2))

/************************************************************************/
/*  dy_to_mdy:								*/
/*	Return month and day from day,year info.  Handle leap years.	*/
/************************************************************************/
void
dy_to_mdy (day, year, month, mday)
    int		day;
    int		year;
    int		*month;
    int		*mday;
{
    int leap_day;
    *month=1;
    *mday = day;
    while (day > LDOY(year,*month)) ++*month;
    *mday = day - LDOY(year,*month-1);
}

