#include <stdio.h>

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <sys/param.h>
#include <local/local.h>
#include <local/date_time.h>
#include <local/cmd_opt.h>
#include <local/mathconst.h>
#include <local/earthconst.h>
#include <local/decode.h>

#include "ucb_catalog.h"
#include "global.h"
#include "procs.h"

/************************************************************************/
/*  NOTE: Floating point fields in this format have explicit decimal	*/
/*  points, and therefore need NO explicit scaling.			*/
/************************************************************************/

/* Limits */
#define	MAXLINELEN	200

/* match_ucb_catalog_mag -- match magnitude pref list for ucb_catalog */
static nmag = 0;
double
match_ucb_catalog_mag (line) 
     char *line;
{
    double mag = 0.;
    int coda_mag = 0;
    int amp_mag = 0;
    int i;

    for (i=0; i<mags.nprefs; i++) {
	switch (mags.magtype[i]) {
	  case ML_MAG:
	    mag = DECODE (line+ML_OFFSET, ML_LEN, atof);
	    nmag = DECODE (line+N_ML_OFFSET, N_ML_LEN, atoi);
	    break;
	  case MLN_MAG:
	    mag = DECODE(line+MLN_OFFSET, MLN_LEN, atof);
	    nmag = DECODE (line+N_MLN_OFFSET, N_MLN_LEN, atoi);
	    break;
	  case BM_MAG:
	    mag = DECODE(line+BM_MAG_OFFSET, BM_MAG_LEN, atof);
	    nmag = DECODE (line+N_BM_MAG_OFFSET, N_BM_MAG_LEN, atoi);
	    break;
	  case MW_MAG:
	    mag = DECODE(line+MW_OFFSET, MW_LEN, atof);
	    nmag = DECODE (line+N_MW_OFFSET, N_MW_LEN, atoi);
	    break;
	  case MML_MAG:
	  case CODA_MAG:
	  case AMP_MAG:
	  case AVG_MAG:
	  default:
	    fprintf (stderr, "Specified magnitude not available for ucb catalog\n");
	    exit(1);
	    break;
	}
	if (mag > 0) return (mag);
    }
    /* No magnitude was found.  Return 0. */
    nmag = 0;
    return (0.);
}


search_ucb_catalog(fp) 
     FILE *fp;
{
    int		i,j,k; 	
    char	line[MAXLINELEN],buf_line[MAXLINELEN];
    int 	num_mag_obs,num_m0_obs,num_sta,max_az_gap;
    double	depth,mag,m0,min_delta,close;
    double	otime_err,lat_err,lon_err,depth_err,rms,ot;
    double	lat,lon,cdelt,cdlon,ct1,st1;
    char	qua,felt;
    double	se;
    int		yr, mo, da, hr, mn;
    int		lines = 0;
    int		selected = 0;

    while (fgets(line,MAXLINELEN,fp) != NULL) {
        strcpy(buf_line,line);
	++lines;
	/*
	 * now filter the event
	 */

	if (do_otime) {		/* ORIGIN TIME */
	    yr = DECODE(line+YEAR_OFFSET, YEAR_LEN, atoi);
	    mo = DECODE(line+MONTH_OFFSET, MONTH_LEN, atoi);
	    da = DECODE(line+DAY_OFFSET, DAY_LEN, atoi);

	    hr = DECODE(line+HOUR_OFFSET, HOUR_LEN, atoi);
	    mn = DECODE(line+MINUTE_OFFSET, MINUTE_LEN, atoi);
	    se = DECODE(line+SECOND_OFFSET, SECOND_LEN, atof);
	    ot = timvar(yr, mo, da, hr, mn, se, GREGORIAN);
	    if (ot < otmin || ot > otmax)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_lat) {		/* LATITUDE */
	    lat = DECODE(line+LAT_OFFSET, LAT_LEN, atof);
	    if (lat < latmin || latmax < lat)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_lon) {		/* LONGITUDE */
	    lon = DECODE(line+LON_OFFSET, LON_LEN, atof);
#ifdef	DEBUG_DELTA
	    fprintf (stderr, "Lon check: ");
#endif
	    if ( (lon < lonmin || lonmax < lon) && (! (wraplon && ((lon+360 <= lonmax) || (lon-360 >= lonmin)) ) ) )
#ifdef	DEBUG_DELTA
	    {
	      fprintf (stderr, "Outside: lon=%lf\n", lon);
	      if (negate) goto OUTPUT;
	      else        continue;
	    }
	    fprintf (stderr, "Inside: lon=%lf\n", lon);
#else
	      if (negate) goto OUTPUT;
	      else        continue;
#endif
	}
	if (do_depth) {		/* FOCAL DEPTH */
	    depth = DECODE (line+DEPTH_OFFSET, DEPTH_LEN, atof);
	    if (depth < zmin || zmax < depth)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_mag) {		/* MAG */
	    mag = match_ucb_catalog_mag (line);
	    if ((mag < magmin || magmax < mag) && !(do_nomag && mag==0))
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	  
	if (do_nsta) {		/* NUMBER OF STATIONS */
	    num_sta = DECODE (line+NSTA_OFFSET, NSTA_LEN, atoi);
	    if (num_sta < nstmin || nstmax < num_sta)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_rms) {		/* RMS RESIDUAL */
	    rms = DECODE(line+RMS_OFFSET, RMS_LEN, atof);
	    if (rms < rmsmin || rmsmax < rms)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_gap) {		/* AZIMUTHAL GAP */
	    max_az_gap = DECODE (line+AZ_GAP_OFFSET, AZ_GAP_LEN, atoi);
	    if (max_az_gap < gapmin || gapmax < max_az_gap)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_close) {		/* DISTANCE TO CLOSEST STATION */
	    close = DECODE(line+DIST_OFFSET, DIST_LEN, atof);
	    if (close < clomin || clomax < close)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_qual) {		/* "QUALITY" CODE */
	    qua = *(line+QUAL_OFFSET);
	    if (!index(quals, (isupper(qua) ? tolower(qua) : qua)))
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_mag_obs) {	/* NUMBER OF OBSERVATIONS USED TO GET MAG */
	    num_mag_obs = nmag;
	    if (num_mag_obs < nmagmin || nmagmax < num_mag_obs)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_m0) {		/* SCALAR MOMENT */
	    m0 = DECODE (line+M0_OFFSET, M0_LEN, atof);
	    if (m0 < m0min || m0 > m0max) 
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_m0_obs) {	/* NUMBER OF OBSERVATIONS USED TO GET M0 */
	    num_m0_obs = DECODE (line+N_M0_OFFSET, N_M0_LEN, atoi);
	    if (num_m0_obs < nm0min || num_m0_obs > nm0max)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_felt) {		/* FELT EVENT */
	    felt = *(line+FELT_OFFSET);
	    if (felt != 'F') 
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_oterr) {		/* ERROR IN ORIGIN TIME */
	    otime_err = DECODE (line+OT_ERR_OFFSET, OT_ERR_LEN, atof);
	    if (otime_err < otemin || otime_err > otemax)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_laterr) {	/* ERROR IN LATITUDE */
	    lat_err = DECODE (line+LAT_ERR_OFFSET, LAT_ERR_LEN, atof);
	    if (lat_err < laemin || lat_err > laemax)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_lonerr) {	/* ERROR IN LONGITUDE */
	    lon_err = DECODE (line+LON_ERR_OFFSET, LON_ERR_LEN, atof);
	    if (lon_err < loemin || lon_err > loemax)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_deperr) {	/* ERROR IN DEPTH */
	    depth_err = DECODE (line+DEPTH_ERR_OFFSET, DEPTH_ERR_LEN, atof);
	    if (depth_err < zerrmin || depth_err > zerrmax) 
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_poly) {		/* POLYGONAL REGION */
	    /* Note: do_poly implies do_lat and do_lon */
	    if (inpoly(lat, lon, plat, plon, nvert) == 0)
	      if (negate) goto OUTPUT;
	      else        continue;
	}
	if (do_delta) {		/* EPICENTRAL DISTANCE */
	    /* Note: do_delta implies do_lat and do_lon */
	    lat = ggtogc(RAD*lat);
	    ct1 = cos(90*RAD - lat);
	    st1 = sin(90*RAD - lat);
	    cdlon = cos(RAD*lon - phi0);
	    cdelt = st0*st1*cdlon + ct0*ct1;
	    if (cdelt > 1.0) cdelt = 1.0;   /* Should never be > 1.0, but some is... */
	    if (cdelt > cdmin || cdelt <= cdmax)
	      if (negate) goto OUTPUT;
	      else        continue;
#ifdef	DEBUG_DELTA
	    fprintf (stderr, "Values for event:\n");
	    fprintf (stderr, "lat=%lf lon=%lf ct1=%lf st1=%lf\n", lat,lon,ct1,st1);
	    fprintf (stderr, "cdmin=%.16lf cdmax=%.16lf\n", cdmin, cdmax);
	    fprintf (stderr, "cdlon=%.20lf cdelt=%.20lf\n\n", cdlon, cdelt);
#endif
	    /* sdlon = sin(lon - phi0);					*/
	    /* cvrpd(st0*ct1-st1*ct0*cdlon, st1*sdlon, &sdelt, az0);	*/
	    /* *delta = atan2(sdelt, cdelt);				*/
	    /* cvrpd(st1*ct0-st0*ct1*cdlon, -sdlon*st0, &sdelt, az1);	*/
	}
	if (negate)
	  continue;


	/*
	 * now do output
	 */
		
      OUTPUT:
	fputs(line,stdout); 
	++selected;
    }
    yr = DECODE(buf_line+YEAR_OFFSET, YEAR_LEN, atoi);
    mo = DECODE(buf_line+MONTH_OFFSET, MONTH_LEN, atoi);
    da = DECODE(buf_line+DAY_OFFSET, DAY_LEN, atoi);
    hr = DECODE(buf_line+HOUR_OFFSET, HOUR_LEN, atoi);
    mn = DECODE(buf_line+MINUTE_OFFSET, MINUTE_LEN, atoi);
    se = DECODE(buf_line+SECOND_OFFSET, SECOND_LEN, atof)/100.;
    otmin = timvar(yr, mo, da, hr, mn, se, GREGORIAN);

    if (verbose) fprintf (stderr,"%d lines, %d selected\n", lines, selected);
}

