#ifndef lint
static char	rcsid[]="$Header: extract.c,v 1.5 88/03/24 16:22:48 julian Exp $";
#endif lint
/*
 * Read summary "cards" and extract corresponding arrivals ("phase cards")
 * Bruce R. Julian, USGS Menlo Park, Calif, 23 March 1988
 */
#include <stdio.h>
#include <ctype.h>
#include <local/local.h>
#include <local/decode.h>
#include <local/date_time.h>

#define ISDELIM(s) !verify(s, " \t\c\n\f")
#define MAXLINE	256			/* Longest input line		*/
#define TOL	5.			/* Default origin-time tolerance*/
#define USAGE	"Usage: %s [-e error_file] [-t seconds] phase_file... <summary_file"

	int	errno;			/* Error flag			*/
	char	*progname;		/* For error messages		*/

	double	atof();			/* Standard C library		*/
	int	atoi();			/* Standard C library		*/
	FILE	*efopen();		/* libq				*/
	void	error();		/* libq				*/
PRIVATE	TIME	evtime();		/* Defined below		*/
	char	*getl();		/* getl.c			*/
	int	strncmp();		/* Standard C library		*/
	char	*strncpy();		/* Standard C library		*/
	char	*ungetl();		/* getl.c			*/
	char	*verify();		/* libq				*/

main(argc,argv)
	int	argc;
	char	*argv[];
{
	int	c;			/* Option letter from argv	*/
	double	dt;			/* Origin-time difference	*/
	bool	found;
	FILE	*fe;			/* Error output unit		*/
	FILE	*fp;			/* Phase input unit		*/
	FILE	*fs;			/* Summary input unit		*/
	FILE	*fo;			/* Output unit			*/
	long	lp;			/* Line number in phase file	*/
	bool	match;			/* Arrivals satisfy request?	*/
extern	int	optind;
extern	char	*optarg;
	char	ss[MAXLINE];		/* Summary (event) line		*/
	char	sp[MAXLINE];		/* Phase line			*/
	TIME	t;			/* Origin time			*/
	TIME	tlast;			/* Last origin time		*/
	double	tol;			/* Origin-time tolerance	*/
	TIME	tsum;

	/* Set defaults */
	fe = stderr;
	fs = stdin;
	fo = stdout;
	tol = TOL;

	/* Interpret command-line arguments */
	progname = NULL;
	while ((c = getopt(argc, argv, "e:ht:")) != EOF)
		switch (c) {
		case 'e':
			fe = efopen(optarg, "a");
			break;
		case 't':
			tol = atof(optarg);
			break;
		case 'h':
		default:
			error(USAGE, argv[0]);
			break;
		}
	if (optind >= argc)
		error(USAGE, argv[0]);
	progname = argv[0];

	/* Loop over desired events */
	fp = NULL;
	tlast = 0.;
	while (getl(ss, sizeof(ss), fs) != NULL) {
		tsum = evtime(ss);
		found = FALSE;

		/* Scan arrival data until event is found or its time is passed */
		FOREVER {

			/* Get next line from phase files */
			while (fp == NULL || getl(sp, sizeof(sp), fp) == NULL) {
				/* Close old file, if any; open next file, if any */
				if (fp != NULL && fclose(fp) == EOF)
					error("Can't close phase file");
				if (optind >= argc)
					goto BREAK_2;
				fp = efopen(argv[optind++], "r");
				lp = 0;
			}

			/* Determine origin time, check for disorder */
			t = evtime(sp);
			if (tlast != 0. && t < tlast)
				fprintf(fe, "Event out of order at line %ld file %s\n",
					lp, argv[optind-1]);
			tlast = t;

			/* Compare times */
			if ((dt=t-tsum) > tol) {
				/* Push event back into phase stream for future use */
				if (ungetl(sp, fp) == NULL)
					error("Can't un-get summary line");
				--lp;
				break;
			}

			/* Read event lines from phase file, if match write */
			if (match = (dt >= -tol)) {
				found = TRUE;
				fputs(sp, fo);
			}
			while (getl(sp, sizeof(sp), fp) != NULL) {
				lp++;
				if (match)
					fputs(sp, fo);
				if (ISDELIM(sp))
					break;
			}
		} BREAK_2:
		if (! found)
			fprintf(fe, "No arrivals found for event:\n%s\n", ss);
	}

	if (fp != NULL)
		(void)fclose(fp);
	exit(0);
}

/*
 * Read time from summary card
 */
PRIVATE TIME
evtime(s)
	char	*s;
{
	int	yr, mo, da, hr, mn;
	double	sec;

	if (strneq(s, "SUMCARD1", 8))
		s += 9;
	yr	= DECODE(s,    2, atoi) + 1900;
	mo	= DECODE(s+ 2, 2, atoi);
	da	= DECODE(s+ 4, 2, atoi);
	hr	= DECODE(s+ 7, 2, atoi);
	mn	= DECODE(s+ 9, 2, atoi);
	sec	= DECODE(s+12, 5, atof);
	return timvar(yr, mo, da, hr, mn, sec, GREGORIAN);
}
