#ifndef lint
static char rcsid[] = "$Header: qfetch.c,v 1.25 92/01/23 16:09:17 julian Exp $";
#endif lint
/*
 * Get event "summary cards" for specified time window
 */
#include <sys/types.h>
#include <stdio.h>
#include <ctype.h>
#include <signal.h>
#ifdef pdp11
#include <time.h>
struct tm *gmtime();
#else pdp11
#include <sys/time.h>
#endif pdp11
#include <math.h>
#include <local/local.h>
#include <local/quake.h>
#include <local/cmd_opt.h>
#include <local/decode.h>

#define MAXFILES	1000		/* Files to search		*/
#define MAXLINE 	150		/* Length of input line		*/
#define SAMEDAY(a, b)	(floor(a/DAY) == floor(b/DAY))

PRIVATE TIME	tmin, tmax;		/* Origin time window		*/
PRIVATE bool	verbose;		/* Print data-file names?	*/

int	errno;				/* Error flag			*/

	double	atof();		/* -lc					*/
	int	atoi();		/* -lc					*/
	TIME	atotime();	/* -lq					*/
PRIVATE	void	check();	/* Defined below			*/
PRIVATE	TIME	do_file();	/* Defined below			*/
	FILE	*efopen();	/* -lq					*/
	char	*emalloc();	/* -lq					*/
	void	error();	/* -lq					*/
	void	exit();		/* UNIX C library			*/
PRIVATE	void	mark();		/* Defined below			*/
PRIVATE	TIME	now();		/* Defined below			*/
PRIVATE	bool	seen();		/* Defined below			*/
	char	*sprintf();	/* UNIX C library			*/
	char	*strcpy();	/* -lc					*/
	int	strncmp();	/* -lc					*/
	char	*strncpy();	/* -lc					*/
	long	time();		/* -lc					*/
	char	*index();	/* -lc					*/

/*----  C O M M A N D - L I N E   A R G U M E N T   P R O C E S S I N G  ----*/

/*
 * Be sure mn <= mx, to protect users from typos, etc.
 */
PRIVATE void
check(mn, mx, s)
	double mn, mx;
	char *s;
{
	if (mn > mx) error("minimum %s value greater than maximum", s);
}

qhelp	(){ if(eargc>2) prt_help(); else prt_doc(); }
qtime	(){ tmin = atotime(aarg()); tmax = atotime(aarg()); check(tmin, tmax, "time"); }
qverbose(){ verbose = TRUE; }

struct command cmd[] = {
	qhelp    , "-help"    , "[opt...]" , "Print documentation",
	qtime    , "-time"    , "min max"  , "Time window ([cc]yymmddhhmmss[+-offset])",
	qverbose , "-verbose" , ""         , "Print names of files examined",
};

/*
 * Macros to generate data file names,
 * listed in order of preference (highest first)
 */
#define YFILE  sprintf(filename, "/we/calnet/caldata/%02d_sum", yr-1900)
#define MFILE1 sprintf(filename, "/we/calnet/caldata/cusp/%02d%02dKz.smp", yr-1900, mo)
#define MFILE2 sprintf(filename, "/we/calnet/caldata/cusp/%02d%02dKa.smp", yr-1900, mo)
#define MFILE3 sprintf(filename, "/usr/picker/clean/%.3s%02d.sum", mname[mo], yr-1900)
#define MFILE4 sprintf(filename, "/usr/picker/online/%.3s%02d.smp", mname[mo], yr-1900)
#define DFILE  sprintf(filename, "/usr/picker/online/%.3s%02d%02d.smp", \
		mname[mo], da, yr-1900)

PRIVATE char	*mname[] = { "???",
	"jan", "feb", "mar", "apr", "may", "jun",
	"jul", "aug", "sep", "oct", "nov", "dec",
};

main(argc,argv)
	int	argc;
	char	*argv[];
{
static	char	filename[256];
	FILE	*f;
	int	yr, mo, da, hr, mn;
	double	se;
	TIME	temp, t_last;

	/* Set defaults */
	tmax = now();
	tmin = tmax - 3*DAY;
	tmax += DAY;
	verbose = FALSE;

	/* Interpret command-line options */
	PROCESS_OPTS;
	if (iarg < argc)
		error("Unknown option \"%s\".", argv[iarg]);

	/*
	 * Main loop:
	 * Search files in order of priority until events in window found,
	 * output them, increment beginning of window, and repeat.
	 */
	while (tmin < tmax) {

		/* Hunt for previously unexamined file, in order of priority */
		datime(tmin, &yr, &mo, &da, &hr, &mn, &se, GREGORIAN);
		if ((!seen(YFILE)  && (f = fopen(filename, "r")) != NULL ) ||
		    (!seen(MFILE1) && (f = fopen(filename, "r")) != NULL ) ||
		    (!seen(MFILE2) && (f = fopen(filename, "r")) != NULL ) ||
		    (!seen(MFILE3) && (f = fopen(filename, "r")) != NULL ) ||
		    (!seen(MFILE4) && (f = fopen(filename, "r")) != NULL ) ||
		    (!seen(DFILE)  && (f = fopen(filename, "r")) != NULL )) {
			mark(filename);
			if (verbose)
				(void)printf("FILE: %s\n", filename);

			/* Get events within window from file */
			if ((temp = do_file(f)) != 0.0) {
				/* Resume search just after last event found */
				t_last = temp;
				tmin = t_last + 1.0e-6;
			}
			(void)fclose(f);
		}
		else {
			if (! SAMEDAY(t_last, tmin))
				/* No file found for this day */
				(void)fprintf(stderr, "Can't find data for %d %s %d\n",
					yr, mname[mo], da);
			tmin = DAY*(1.0 + floor(tmin/DAY));
		}
	}

	exit(0);
}

/*
 * Return current time
 */
PRIVATE TIME
now()
{
	time_t		clock;
	struct tm	*gmt;

	clock = time((time_t *)0);
	gmt = gmtime(&clock);

	return timvar(	(int)(gmt->tm_year) + 1900,
			(int)(gmt->tm_mon)+1,
			(int)(gmt->tm_mday),
			(int)(gmt->tm_hour),
			(int)(gmt->tm_min),
			(double)(gmt->tm_sec),
			GREGORIAN);
}

/*
 * Routines to keep track of which files have already been examined
 */

PRIVATE int	nseen = 0;		/* Number examined		*/
PRIVATE char	*table[MAXFILES];	/* [0..nseen-1] File names	*/

/*
 * Has file already been examined?
 */
PRIVATE bool
seen(s)
	char	*s;			/* File name			*/
{
	int	i;

	for (i=0; i<nseen; i++)
		if (streq(s, table[i]))
			return TRUE;
	return FALSE;
}

/*
 * Notify seen() that file has already been examined
 */
PRIVATE void
mark(s)
	char	*s;			/* File name			*/
{
	table[nseen] = emalloc((unsigned)strlen(s)+1);
	(void)strcpy(table[nseen], s);
	if (nseen++ >= MAXFILES)
		error("Too many files.");
}

/*
 * Search one file, outputting events within window
 * Return time of last event found
 */
PRIVATE TIME
do_file(f)
	FILE	*f;
{
	int	c;
	long	top, bot, mid;
static	char	line[MAXLINE];
	TIME	ot, t_last;

	t_last = 0.0;

	/* Binary search */
	bot = 0;
	(void)fseek(f, 0L, 2);
	top = ftell(f);
	for (;;) {
		mid = (top + bot)/2;
		(void)fseek(f, mid, 0);
		do {
			c = getc(f);
			mid++;
		} while (c != EOF && c != '\n');
		if (fgets(line, sizeof(line), f) == NULL)
			break;
		switch (compare(line, &ot)) {
		case 1:
		case 0:
			/* Too late */
			if (top <= mid)
				break;
			top = mid;
			continue;
		case -1:
			/* Too early */
			bot = mid;
			continue;
		}
		break;
	}

	/* Copy first desired line to standard output */
	(void)fseek(f, bot, 0);
	while (ftell(f) < top) {
		if (fgets(line, sizeof(line), f) == NULL)
			return t_last;
		switch (compare(line, &ot)) {
		case 1:
			/* Too late */
			return t_last;
		case 0:
			(void)fputs(line, stdout);
			t_last = ot;
			break;
		case -1:
			/* Too early */
			continue;
		}
		break;
	}

	/* Copy remaining desired lines to standard output */
	while ((fgets(line, sizeof(line), f) != NULL) && (compare(line, &ot) == 0)) {
		(void)fputs(line, stdout);
		t_last = ot;
	}
	return t_last;
}

/*
 * Read origin time from summary line, compare with limits
 */
PRIVATE int
compare(line, ot)
	char	*line;			/* Summary line			*/
	TIME	*ot;			/* Origin time			*/
{
	int	yr, mo, da, hr, mn;
	double	se;

	yr = DECODE(line  , 2, atoi) + 1900;
	mo = DECODE(line+2, 2, atoi);
	da = DECODE(line+4, 2, atoi);
	hr = DECODE(line+7, 2, atoi);
	mn = DECODE(line+9, 2, atoi);
	se = atof(line+12);
	*ot = timvar(yr, mo, da, hr, mn, se, GREGORIAN);
	return *ot < tmin ? -1 :
	       *ot > tmax ?  1 :
	                     0 ;
}
