#ifndef lint
static char rcsid[] = "$Header: reloc.c,v 1.5 92/05/25 07:33:00 julian Exp $";
#endif lint
/*
 * General seismic event relocation function
 * Bruce R. Julian, USGS Menlo Park, Calif.
 */

#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <setjmp.h>
#include <local/earthconst.h>
#include <local/quake.h>
#include <local/bwflat.h>
#include <local/mathconst.h>

/* Conditional definitions for double or single precision versions of program */
#ifdef DOUBLE
#define FLOAT	double
#define FITINIT	dfitinit
#define FITCLOSE dfitclose
#define FITEQN	dfiteqn
#define FITCON	dfitcon
#define FITUNCON dfituncon
#define FITSLV	dfitslv
#else DOUBLE
#define FLOAT	float
#define FITINIT fitinit
#define FITCLOSE fitclose
#define FITEQN	fiteqn
#define FITCON	fitcon
#define FITUNCON fituncon
#define FITSLV	fitslv
#endif DOUBLE

#include <local/fit.h>

static jmp_buf	r_env;		/* Environment for error returns */

	void	cvcsd();	/* -lq (cvcsd.c)			*/
	void	cvrpd();	/* -lq (cvrpd.c)			*/
	void	cvscd();	/* -lq (cvscd.c)			*/
	void	dcbaz();	/* -lq (gdcdaz.c)			*/
	char	*emalloc();	/* -lq (emalloc.c)			*/
	void	error();	/* -lq (error.c)			*/
	void	error();	/* UNIX C library			*/
	void	fitcon();	/* -lfit				*/
	void	fiteqn();	/* -lfit				*/
	void	fituncon();	/* -lfit				*/
static	int	mkeqns();	/* Defined below			*/
	float	pctil();
	void	dcrefpt();	/* -lq (gdcdaz.c)			*/
	void	dcdaz();	/* -lq (gdcdaz.c)			*/
	void	dcback();	/* -lq (gdcdaz.c)			*/
	void	fscale();	/* -lq (fscale.c)			*/
	void	fset();		/* -lq (fset.c)				*/

/**/
int
reloc(f, mp, op, rdgs, bwfunc, assoc, bwrslt, nrdg, cp, sp)
	FILE		*f;		/* Printed output	(input)	*/
	MODEL		*mp;		/* Earth model		(input) */
	ORIGIN		*op;		/* Origin of event	(in/out)*/
	READING		*rdgs[];	/* Reading-ptr array	(input)	*/
	tbool		(*bwfunc)();	/* Travel-time function	(input)	*/
	ASSOC		*assoc[];	/* Assoc-ptr array	(output)*/
	BWRSLT		*bwrslt[];	/* Bwrslt-ptr array	(output)*/
	int		nrdg;		/* Number of readings	(input)	*/
	RLCTRL		*cp;		/* Control parameters	(input)	*/
	STATIS		*sp;		/* Statistics of result	(output)*/
{
	FITPROB		*p, *q;
	int		idamp;		/* Damping-step counter		*/
	tbool		print;		/* Print-control flag		*/
	float		fit, oldfit;	/* Goodness-of-fit values	*/
	float		lambda;		/* Damping parameter		*/
	ORIGIN		old_or;

	/* Error handling via non-local goto */
	if (setjmp(r_env) != 0) {
		if (p) FITCLOSE(p);
		if (q) FITCLOSE(q);
		return FAIL;
	}

	/* Iteration loop */
	p = q = (FITPROB *)NULL;
	oldfit = HUGE;
	sp->converged = FALSE;
	lambda = cp->lstart*cp->lfact;
	for (sp->iter=0; !(sp->converged) && sp->iter<(cp->max_iter); (sp->iter)++) {

		/* Generate equations */
		p = FITINIT(4);
		print = (cp->iprint == ALL);
		mkeqns(p, f, mp, op, rdgs, bwfunc,
			assoc, bwrslt, nrdg, cp, sp, print, &fit);
		hold(p, cp->hold);

		/* Step-size damping loop */
		for (idamp=0; fit>oldfit &&idamp<(cp->max_damp); idamp++) {
			*op = old_or;		/* Copy structure */
			lambda *= cp->lfact;
			assert (q != NULL);
			damp(q, lambda, cp->zdamp);
			solve(q, f, op, cp, sp, print);
			mkeqns((FITPROB *)NULL, f, mp, op, rdgs, bwfunc,
			     assoc, bwrslt, nrdg, cp, sp, print, &fit);
		}
		if (idamp > 0) {
			if (print)
				prmsg(f, "Solution damped:  %d steps", idamp);
			if (p)
				FITCLOSE(p);
			p = FITINIT(4);
			mkeqns(p, f, mp, op, rdgs, bwfunc,
			     assoc, bwrslt, nrdg, cp, sp, print, &fit);
			hold(p, cp->hold);
		}

		/* Save current state */
		oldfit = fit;
		old_or = *op;

		/* Solve equations */
		lambda /= cp->lfact;
		if (lambda < cp->lmin)
			lambda = cp->lmin;
		damp(p, lambda, cp->zdamp);
		solve(p, f, op, cp, sp, print);
		if (q)
			FITCLOSE(q);
		q = p;
	}

	if (p)
		FITCLOSE(p);
	print = (cp->iprint >= FINAL);
	mkeqns((FITPROB *)NULL, f, mp, op, rdgs, bwfunc,
		assoc, bwrslt, nrdg, cp, sp, print, &fit);
	return SUCCEED;
}
/**/
/* Process readings, generate equations */
static int
mkeqns(p, f, mp, op, rdgs, bwfunc, assoc, bwrslt, nrdg, cp, sp, print, pfit)
	FITPROB		*p;
	FILE		*f;		/* Printed-output file	(input)	*/
	MODEL		*mp;		/* Earth model		(input) */
	ORIGIN		*op;		/* Event origin		(input)	*/
	READING		*rdgs[];	/* Reading-ptr array	(input)	*/
	tbool		(*bwfunc)();	/* Travel-time function	(input)	*/
	ASSOC		*assoc[];	/* Assoc-ptr array	(output)*/
	BWRSLT		*bwrslt[];	/* Bwrslt-ptr array	(output)*/
	int		nrdg;		/* Number of readings	(input)	*/
	RLCTRL		*cp;		/* Control parameters	(input)	*/
	STATIS		*sp;		/* Statistics of result	(output)*/
	tbool		print;		/* Print-control flag	(input)	*/
	float		*pfit;		/* Goodness-of-fit estimate	*/
{
	float		avres;		/* Average absolute residual	*/
	extern int	errno;		/* Error flag			*/
	double		gdc[3];		/* Geocen. direction cosines	*/
	int		i;		/* Dummy loop index		*/
	int		nfree();	/* No. of free variables	*/
	int		spread();	/* Estimates spread of data	*/
	READING		*rp;
	ASSOC		*ap;
	BWRSLT		*bwrp;

	if (print)
		prorig(f, op);
	sp->chisq = sp->sumwt = 0.0;
	sp->ntasoc = sp->ntused = 0;
#ifdef SLO
	sp->nsasoc = sp->nsused = 0;
#endif SLO
	if (print)
		prhead(f);

	/* Loop over data, calculating distances, azimuths, times, etc. */
	cvscd(1.0, HALFPI-op->o_lat, op->o_lon, &gdc[X], &gdc[Y], &gdc[Z]);
	dcrefpt(gdc);
	for (i=0; i<nrdg; i++)
		if (rdgs[i]->flags & ASSOCIATED)
			resid(mp, op, rdgs[i], assoc[i], bwfunc, bwrslt[i], sp);

	/* Estimate spread of data */
	spread(rdgs, assoc, nrdg, &avres, pfit);

	/* Loop over data, generating condition equations */
	for (i=0; i<nrdg; i++) {
		rp = rdgs[i]; ap = assoc[i]; bwrp = bwrslt[i];
		if ((rp->flags & ASSOCIATED) == 0)
			continue;
		if (ap->brnum >= 0) {
			if (rp->flags & T_EXISTS) {
				if (rp->flags & USE_T)
					addtim(p, ap, bwrp,
						sp, cp->mu, avres, *pfit);
				if (print)
					prtime(f, i, rp, ap);
			}
#ifdef SLO
			if (rp->flags & S_EXISTS) {
				if (rp->flags & USE_S)
					addslo(p, ap, bwrp, sp, cp->mu, *pfit);
				if (print)
					prslow(f, i, rp, ap);
			}
#endif SLO
		}
		else if (print)
			prpher(f, i, rp, ap);
	}

	/* Do we have enough data to proceed? */
	sp->ndf = sp->ntused - nfree(sp);
#ifdef SLO
	sp->ndf += 2*sp->nsused;
#endif SLO
	if (sp->ndf < 0 ||
	   (sp->ntused < 1 && ! cp->hold[ITIME]) ||
	   sp->sumwt == 0) {
		errno = EFEWDATA;
		longjmp(r_env, -1);
	}
	if (print)
		prfit(f, *pfit);
}
/**/
/* Calculate distance, azimuth, residuals for one reading */
static
resid(mp, op, rdg, assoc, bwfunc, bwrslt, sp)
	MODEL		*mp;		/* Earth model		(input) */
	ORIGIN		*op;		/* Origin		(input)	*/
	READING		*rdg;		/* Reading		(input)	*/
	ASSOC		*assoc;		/* Association		(in/out)*/
	tbool		(*bwfunc)();	/* Travel-time function	(input)	*/
	BWRSLT		*bwrslt;	/* Body-wave result	(output)*/
	STATIS		*sp;		/* Statistics		(output)*/
{
#ifdef SLO
	double		pox, poy;	/* Observed slowness components*/
	double		pcx, pcy;	/* Calculated slowness compcnents*/
#endif SLO
	double		cd, sd, cz, sz;
	double		gdc[3];

	gdc[0] = rdg->sta.gdc[0];
	gdc[1] = rdg->sta.gdc[1];
	gdc[2] = rdg->sta.gdc[2];
	dcdaz(gdc, &cd, &sd, &cz, &sz);
	assoc->delta = atan2(sd, cd);
	assoc->aze = atan2(sz, cz);
	if (sz < 0.0)
		assoc->aze += TWOPI;
	dcbaz(gdc, &cz, &sz);
	assoc->azs = atan2(sz, cz);
	if (sz < 0.0)
		assoc->azs += TWOPI;

	/* Compute theoretical time */
	if (! (*bwfunc)(mp, op, rdg, assoc, bwrslt)) {
		assoc->ercode = errno;
		assoc->brnum = -1;
		return;
	}
	assoc->brnum = bwrslt->bw_brnum;
	if (rdg->flags & T_EXISTS) {
		(sp->ntasoc)++;
		assoc->atresid = rdg->atime - op->o_time - bwrslt->bw_t;
		assoc->atsigma = hypot(bwrslt->bw_set, rdg->setime);
	}
#ifdef SLO
	/* Slowness residual (vector components) */
	if (rdg->flags & S_EXISTS) {
		(sp->nsasoc)++;
		cvptor(&rdg->slowness, &rdg->azimuth, &pox, &poy);
		cvptor(&bwrslt->bw_td, &assoc->azs, &pcx, &pcy);		/* Calculated */
		assoc->szresid[0] = pox - pcx; assoc.szresid[1] = poy - pcy;	/* Residual */
		assoc->szsigma = hypot(bwrslt->bw_setd, rdg->seslow);
	}
#endif SLO
}
/**/
/* Estimate average & spread of arrival-time residuals */
static int
spread	(rdgs, assoc, nrdg, t, s) 
	READING		*rdgs[];	/* Reading-ptr array	(input)	*/
	ASSOC		*assoc[];	/* Assoc-ptr array	(input)	*/
	int		nrdg;		/* Number of readings	(input)	*/
	float		*t;		/* Estimate of average 	(output)*/
	float		*s;		/* Estimate of spread 	(output)*/
{
	int		nx;		/* No. of active data		*/
	float		*x;		/* Scratch array		*/
	double		q1, q3;		/* 1st, 3rd quartiles of residuals */

	/* Allocate scratch space */
	x = (float *)emalloc((unsigned)nrdg*sizeof(float));

	/* Construct array of residuals */
	nx = 0;
	for ( ; nrdg--; rdgs++, assoc++)
		if ((~(*rdgs)->flags & (ASSOCIATED | T_EXISTS | USE_T)) == 0 &&
		    (*assoc)->brnum >= 0)
			x[nx++] = (*assoc)->atresid;

	/* Estimate average, spread */
	q1 = pctil(x, nx, 0.25);
	q3 = pctil(x, nx, 0.75);
	*t = 0.5*(q1 + q3);
	*s = (q3 - q1)/SQRT3;
	free((char *)x);
}
/**/
/* Add arrival time observation to equations */
static
addtim(p, assoc, bwrslt, sp, mu, avres, fit)
	FITPROB		*p;
	ASSOC		*assoc;	/* Association struct		(input)	*/
	BWRSLT		*bwrslt;/* Body-wave result struct	(input)	*/
	STATIS		*sp;	/* Statistics struct		(input)	*/
	double		mu;	/* Frequency of bad data	(input)	*/
	double		avres;	/* Average travel-time residual	(input)	*/
	double		fit;	/* Spread of residuals		(input)	*/
{
	float		a[4];		/* Condition eqn. coefs.	*/
	double		weight();	/* Weight from uniform reduction*/
	double		x, ws;

	assoc->atwt = weight(assoc->atresid-avres, mu, MAX(fit, assoc->atsigma));
	ws = assoc->atwt/assoc->atsigma;
	x = ws*assoc->atresid;
	sp->chisq += x*x;
	sp->sumwt += assoc->atwt;
	(sp->ntused)++;
	if (p) {
		/* Generate condition equation */
		cetime(bwrslt->bw_td, bwrslt->bw_th, assoc->aze, a);
		a[ITIME] = 1.0;
		fscale(a, 4, ws);		/* Multiply by weight */
		FITEQN(p, a, ws*assoc->atresid);/* Incorporate into system */
	}
}

#ifdef SLO
/* Add one slowness observation to equations */
static
addslo(p, assoc, bwrslt, sp, mu, fit)
	FITPROB		*p;
	ASSOC		*assoc;	/* Association struct		(input)	*/
	BWRSLT		*bwrslt;/* Body-wave result struct	(input)	*/
	STATIS		*sp;	/* Statistics struct		(input)	*/
	double		mu;	/* Frequency of bad data	(input)	*/
	double		fit;	/* Spread of resids/est. spread	(input)	*/
{
	float		a[2][4], b[2];
	double		szresmag;
	double		weight();
	double		x, ws;

	/* r.h.s. of condition equation */
	b[EAST]  = assoc->szresid[EAST];
	b[NORTH] = assoc->szresid[NORTH];
	szresmag = hypot(b[EAST], b[NORTH]);
	assoc->szwt = weight(szresmag/assoc->szsigma, mu, fit);
	ws = assoc->szwt/assoc->szsigma;
	x = ws*szresmag;
	sp->chisq += x*x;
	sp->sumwt += assoc->szwt;
	(sp->nsused)++;
	if (p) {
		ceslow(assoc->delta, assoc->aze, assoc->azs,
			bwrslt->bw_td, bwrslt->bw_tdd, bwrslt->bw_tdh, a);
		a[EAST][ITIME] = a[NORTH][ITIME] = 0.0;
		fscale(a, 8, ws);
		FITEQN(p, &a[EAST][0], ws*b[0]);
		FITEQN(p, &a[NORTH][0], ws*b[1]);
	}
}
#endif SLO
/**/
/* Condition equation coefficients for arrival-time observation */
static
cetime(td, th, aze, a)
	double		td;	/* dt/ddelta		(s/Rad)	(input)	*/
	double		th;	/* dt/dh		(s/?)	(input)	*/
	double		aze;	/* Azimuth		(Rad)	(input)	*/
	float		a[3];	/* Cond. eqn. coeffs.		(output)*/
{
	/* Assumes right-handed coordinate system with z axis downward	*/
	/* Azimuth measured from x axis toward y axis			*/
	a[X] = -td*cos(aze);		/* dt/dx */
	a[Y] = -td*sin(aze);		/* dt/dy */
	a[Z] =  th;			/* dt/dz */
}

#ifdef SLO
/* Condition equations coefficients for slowness-azimuth observation */
static
ceslow(delta, aze, azs, td, tdd, tdh, a)
	double	delta;	/* Epicentral distance			(input)	*/
	double	aze;	/* Epicenter -> station azimuth		(input)	*/
	double	azs;	/* Station -> epicenter azimuth		(input)	*/
	double	td;	/* dt/ddelta = p			(input)	*/
	double	tdd;	/* dp/ddelta				(input)	*/
	double	tdh;	/* dp/ddepth				(input)	*/
	float	a[2][4]	/* Cond. eqn. coeffs.			(output)*/
{
	double	sze=sin(aze), cze=cos(aze);
	double	szs=sin(azs), czs=cos(azs);
	double	sd=sin(delta);

	a[EAST ][X]    = -tdd*cze*szs + td*sze*czs/sd; /* d(py)/dx	*/
	a[NORTH][X]    = -tdd*cze*czs - td*sze*szs/sd; /* d(px)/dx	*/
	a[EAST ][Y]    = -tdd*sze*szs - td*cze*czs/sd; /* d(py)/dy	*/
	a[NORTH][Y]    = -tdd*sze*czs + td*cze*szs/sd; /* d(px)/dy	*/
	a[EAST ][Z]    =  tdh*szs;		       /* d(py)/dz	*/
	a[NORTH][Z]    =  tdh*czs;		       /* d(px)/dz	*/
}
#endif SLO
/**/
/* Solve relocation equations and refine origin */
static int
solve(p, f, op, cp, sp, print)
	FITPROB		*p;
	FILE		*f;		/* Print-out unit	(input) */
	ORIGIN		*op;		/* Origin struct	(in/out)*/
	RLCTRL		*cp;		/* Control struct	(input)	*/
	STATIS		*sp;		/* Statistics struct	(output)*/
	tbool		print;		/* Print-control flag	(input) */
{
	float		cor[8];		/* Origin change		*/
					/* And Lagrange multipliers	*/
	double		dist, az;	/* Epicenter shift		*/
	double		gdc[3];		/* Geocentric direction cosines	*/
	double		junk, lat, lon;
	float		g[4];		/* Coefs. of restraint eqn.	*/
	double		zforce;
	double		znew;		/* Perturbed depth value	*/

	/* Solve equations */
	if (FITSLV(p, cor) == FAIL)
		error("Routine FITSLV");

	/* Change focal depth, test for legality */
	znew = op->o_depth + cor[IDEPTH]*REARTH;
	if (znew < cp->min_depth)
		zforce = cp->min_depth;
	else if (znew > cp->max_depth)
		zforce = cp->max_depth;
	else
		goto OK;

	/* Relocate, restraining depth */
	if (print)
		prmsg(f, "Depth (%g) restrained to %g", znew, zforce);
	sp->held[IDEPTH] = TRUE;
	fset(g, 4, 0.0); g[IDEPTH] = 1.0;
 	FITCON(p, g, (zforce - op->o_depth)/REARTH);
	if (FITSLV(p, cor) == FAIL)
		error("Routine FITSLV");
	fituncon(p);
	znew = op->o_depth + cor[IDEPTH]*REARTH;
	assert((zforce - znew) <  0.001);
	assert((zforce - znew) > -0.001);
OK: ;
	assert(znew >= cp->min_depth - 0.001);
	assert(znew <= cp->max_depth + 0.001);
	op->o_depth = znew;

	op->o_time += cor[ITIME];		/* Change origin time */

	/* Change epicenter (move along great circle) */
	cvscd(1.0, HALFPI-op->o_lat, op->o_lon, &gdc[X], &gdc[Y], &gdc[Z]);
	dcrefpt(gdc);
	cvrpd(cor[ILAT], cor[ILON], &dist, &az);
	dcback(cos(dist), sin(dist), cos(az), sin(az), gdc);
	cvcsd(gdc[X], gdc[Y], gdc[Z], &junk, &lat, &lon);
	op->o_lon = lon;
	op->o_lat = HALFPI - lat;

	/* Test for convergence */
	sp->converged = (dist <= cp->epcvthresh) && 
		(ABS(cor[IDEPTH]) <= cp->fdcvthresh) && 
		(ABS(cor[ITIME]) <= cp->otcvthresh);
}
/**/
/* Force flagged coordinates to not change */
static
hold(p, flags)
	FITPROB		*p;
	tbool		flags[];
{
	float		e[4];		/* Unit vector			*/
	int		i;

	for (i=0; i<4; i++)
		if (flags[i]) {
			fset(e, 4, 0.0); e[i] = 1.0;	/* ith unit vector */
			FITCON(p, e, 0.0);
		}
}

/* Impose step-size damping */
static
damp(p, lambda, zfact)
	FITPROB		*p;
	double		lambda;	/* Eigenvalue threshold */
	double		zfact;	/* Multiplies depth threshold */
{
	float		e[4];
	int		i;

	for (i=0; i<4; i++)
		if (i != ITIME) {
			/* lambda times unit vector in nth direction */
			fset(e, 4, 0.0); e[i] = lambda;
			if (i == IDEPTH)
				e[i] *= zfact;
			FITEQN(p, e, 0.0);
		}
}

/* No. of unrestrained origin variables */
static int
nfree(sp)
	STATIS		*sp;		/* Statistics struct	(input) */
{
	int		i, n;

	n = 0;
	for (i=0; i<4; i++)
		if (! sp->held[i])
			n++;
	return n;
}
