#ifndef lint
static char rcsid[] = "$Header: rlprnt.c,v 1.3 90/06/15 11:20:11 julian Exp $";
#endif lint
/*
 * Output routines for subroutine reloc
 * Bruce R. Julian, USGS Menlo Park, Calif.
 */

#include <varargs.h>
#include <stdio.h>
#include <math.h>
#include <local/quake.h>
#include <local/mathconst.h>

char	*ermsg();
double	gctogg();
char	*sprintf();

/* Convert time to character string */
static char*
time_str(t)
	TIME	t;
{
	int		yr, mo, da, hr, mn;
	double		sec;
	static char	buf[30];
	static char	*month[] = {
		"???",
		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
	};

	datime(t, &yr, &mo, &da, &hr, &mn, &sec, GREGORIAN);
	return sprintf(buf, "%04d %s %02d %02d%02d:%06.3f",
		yr, month[(mo>0 && mo < 13 ? mo : 0)], da, hr, mn, sec);
}

/* Print Table Headings */
prhead(f)
	FILE		*f;
{
	(void)fprintf(f,  "\n                      AZIMUTH\n");
	(void)fprintf(f,    "     STA    DELTA    EP    STA");
	(void)fprintf(f,    "    TIME                 RESID   SIGMA   WT\n");
	(void)fprintf(f,    "            (deg)  (deg)  (deg)");
	(void)fprintf(f,    "                         (s)     (s)\n\n");
}

/* Print Arrival-Time Datum */
prtime(f, i, rp, ap)
	FILE		*f;
	int		i;
	READING		*rp;
	ASSOC		*ap;
{
	(void)fprintf(f, "%3d %.4s%8.3f%7.1f%7.1f %s %8s %8.3g%8.2g %6.2f\n",
		i,
		rp->sta.s_code,
		DEG*(ap->delta),
		DEG*(ap->aze),
		DEG*(ap->azs),
		time_str(rp->atime) + 12,	/* Skip date (12 chars) */
		rp->phcode,
		ap->atresid,
		ap->atsigma,
		ap->atwt);
}

#ifdef SLO
/* Print Slowness Datum */
prslow(f, i, rp, ap)
	FILE		*f;
	int		i;
	READING		*rp;
	ASSOC		*ap;
{

	(void)fprintf(f, "%3.0f  %.4s%8.3f%7.1f%7.1f %8s                       %8.2f%8.2f%8.1f%6.2f%6.2f\n",
		i,
		rp->sta.s_code,
		DEG*ap->delta,
		DEG*ap->aze,
		DEG*ap->azs,
		rp->phase_code,
		rp->slowness,
		rp->azimuth,
		hypot(ap->szresid[0], ap->szresid[1]),
		DEG*atan2(ap->szresid[0], ap->szresid[1]),
		ap->szsigma,
		ap->szwt);
}
#endif SLO

/* Print Message For Phase Error */
prpher(f, i, rp, ap)
	FILE		*f;
	int		i;
	READING		*rp;
	ASSOC		*ap;
{
	(void)fprintf(f, "%3d %.4s%8.3f%7.1f%7.1f %s %8s %s\n",
		i,
		rp->sta.s_code,
		DEG*ap->delta,
		DEG*ap->aze,
		DEG*ap->azs,
		time_str(rp->atime) + 12,	/* Skip date (12 chars) */
		rp->phcode,
		ermsg(ap->ercode));
}

/* Print Goodness-Of-Fit Information */
prfit(f, fit)
	FILE		*f;
	double		fit;
{
	(void)fprintf(f, "Fit = %.3g\n\n", fit);
}
#if 0
prfit(f, o_stat)
	FILE		*f;
	STATIS		*o_stat;
{
	(void)fprintf(f, "Chi squared = %.2g on %d degrees of freedom.\n\n",
		o_stat->chisq, o_stat->ndf);
}
#endif

/* Print Origin Values */
prorig(f, op)
	FILE		*f;
	ORIGIN		*op;
{
	(void)fprintf(f, "%s UTC\nLAT: %9.3f     LON: %10.3f     DEPTH: %8.1f KM\n",
		time_str(op->o_time),
		DEG*gctogg(op->o_lat),
		DEG*op->o_lon,
		op->o_depth);
}

/* Print message */
prmsg(va_alist)
	va_dcl
{
	va_list	args;
	FILE	*f;
	char	*fmt;

	va_start(args);
	f = va_arg(args, FILE*);
	fmt = va_arg(args, char*);
	(void)vfprintf(f, fmt, args);
	(void)fprintf(f, "\n");
	va_end(args);
}
