#ifndef lint
static char rcsid[]="$Header: qpack.c,v 1.3 90/06/15 11:01:25 julian Exp $";
#endif lint
/*
 * Filter to convert USGS earthquake cards to binary data structures
 * Bruce R. Julian, USGS Menlo Park, Calif, 10 Sept 1983
 */

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <signal.h>
#include <local/cmd_opt.h>
#include <local/quake.h>
#include <local/station.h>

#define  strneq    !strncmp

int	errno;		/* Error flag			*/
int	evno;		/* Event number			*/
int	l;		/* Input line count		*/
ORIGIN	org;		/* Origin structure		*/
READING	rdg[2];
ASSOC	assoc[2];
char	*stafile;	/* Name of station file		*/
tbool	delays;		/* Include station corrections?	*/
READING	**rpa;		/* Dynamic pointer array	*/
ASSOC	**apa;		/* Dynamic pointer array	*/
int	nalloc;		/* Number of ptrs. allocated	*/
int	nrdgs;		/* No. of readings		*/

void	bytset();
FILE	*efopen();
char	*emalloc();
char	*realloc();
char	*ermsg();
int	strncmp();
char	*verify();
void	error();
void	bytcpy();

/*----  C O M M A N D - L I N E   A R G U M E N T   P R O C E S S I N G  ----*/

qhelp	(){ if (eargc>2) prt_help(); else prt_doc(); }
qdelays (){ delays = TRUE; }
qstafile(){ stafile = aarg(); }

struct command cmd[] = {
	qhelp    , "-help"    , "[opt...]", "Documentation help",
	qdelays  , "-delays"  , ""        , "Use station delays",
	qstafile , "-station" , "filename", "Station file",
};

#define MAXLINE 300
#define CHUNKSIZE 16
#define DELIM	1
#define EVLIN	2
#define MAGLIN	3
#define RDGLIN	4
#define INSTR	5
#define STAFILE	"/usr/picker/params/nocast0.b"

main(argc,argv)
	int	argc;
	char	*argv[];
{
	extern int	errno;
	void	init();
	bool	getevent();
	void	putevent();
	void	cleanup();
	STABLE	*st;

	init();
	PROCESS_OPTS;

	/* Open station file */
	if ((st = stopen(stafile)) == NULL)
		error("Can't open station file %s", stafile);

	/* Allocate space for READING, ASSOC pointers */
	nalloc = CHUNKSIZE;
	rpa = (READING **)emalloc((unsigned)nalloc*sizeof(READING *));
	apa = (ASSOC **)emalloc((unsigned)nalloc*sizeof(ASSOC *));

	/* Event loop */
	l = 0;
	evno = 0;
	while (getevent(st)) {
		++evno;
		putevent();
	}
	cleanup();
}

void
init()
{
static	void	abort();

	/* Establish interupt handler */
	(void)signal(SIGINT, abort);

	/* Set defaults */
	stafile = STAFILE;			/* Station file		*/
	delays = FALSE;				/* Don't use delays	*/
}

bool
getevent(st)
	STABLE		*st;
{
	int		i;		/* Dummy loop index		*/
	char		line[MAXLINE];	/* Input buffer			*/
	int		n;		/* No. of readings/phase card	*/
	bool		any;		/* Any lines read?		*/

	/* Read lines, convert to binary structs */
	nrdgs = 0;
	any = FALSE;
	while (fgets(line, sizeof(line), stdin) != NULL) {
		any = TRUE;
		++l;
		switch (lintype(line)) {

		case EVLIN:			/* Event summary card */
			/* Decode line into origin structure */
			if (evdec(line, &org) != 0) {
				(void)fprintf(stderr, "%s: Error on summary card at line %d (event %d)\n",
					progname, l, evno+1);
				errno = 0;
			}
			break;

		case MAGLIN:			/* Magnitude card */
			break;

		case RDGLIN:			/* Reading ("phase") card */
			/* Decode line into READING, ASSOC structures */
			if ((n = rdgdec(st, line, delays, rdg, assoc)) <= 0) {
				(void)fprintf(stderr,
					"%s: Error on reading at line %d (reading %d of event %d): %s",
					progname, l, nrdgs+1, evno+1, ermsg(errno));
				if (errno == ENOSTATION)
					(void)fprintf(stderr, " \"%.4s\"", rdg->sta.s_code);
				(void)fprintf(stderr, "\n");
				errno = 0;
			}
			else for (i=0; i<n; ++i) {
				if (nrdgs >= nalloc) {
					nalloc += CHUNKSIZE;
					rpa = (READING **)realloc((char *)rpa, (unsigned)nalloc*sizeof(READING *));
					apa = (ASSOC **)realloc((char *)apa, (unsigned)nalloc*sizeof(ASSOC *));
					if (rpa == NULL || apa == NULL)
						error("Can't realloc");
				}
				rpa[nrdgs] = (READING *)emalloc((unsigned)sizeof(READING));
				apa[nrdgs] = (ASSOC *)emalloc((unsigned)sizeof(ASSOC));
				bytcpy((char *)&rdg[i], (char *)rpa[nrdgs], sizeof(READING));
				bytcpy((char *)&assoc[i], (char *)apa[nrdgs], sizeof(ASSOC));
				++nrdgs;
			}
			break;

		case INSTR:	/* Instructions */
			(void)fgets(line, sizeof(line), stdin);	/* Discard line */
			++l;
			break;

		case DELIM:	/* Delimiter */
			return any;

		default:
			(void)fprintf(stderr, "%s:  Line %d unrecognizable\n",
				progname, l);
			break;
		}
	}
	return any;
}

void
putevent()
{
	int		i;

	/* Write out origin structure */
	if (fwrite((char *)&org, sizeof(org), 1, stdout) != 1)
		error("Can't write origin structure");

	/* Write READINGs, ASSOCs to output */
	for (i=0; i<nrdgs; ++i) {
		if (fwrite((char *)rpa[i], sizeof(READING), 1, stdout) != 1 ||
		    fwrite((char *)apa[i], sizeof(ASSOC),   1, stdout) != 1)
			error("Can't write to output");
		free((char *)rpa[i]);
		free((char *)apa[i]);
	}

	/* Write out blank structures as delimiter */
	bytset((char *)rdg, sizeof(READING), 0);
	bytset((char *)assoc, sizeof(ASSOC), 0);
	if (fwrite((char *)rdg,   sizeof(READING), 1, stdout) != 1 ||
	    fwrite((char *)assoc, sizeof(ASSOC),   1, stdout) != 1)
		error("Can't write delimiter to output");
}

int
lintype(line)		/* Examine line, decide what type it is */
	char	*line;
{
	if (verify(line, "0 \t\n") == NULL ||	/* Empty line		*/
	      strneq(line, "    ", 4))		/* Blank station code	*/
		return DELIM;
	if (strneq(line, "SUMCARD1", 8))
		return EVLIN;
	if (strneq(line, "SUMCARD2", 8))
		return MAGLIN;
	if (strneq(line, " %%%", 4))
		return INSTR;
	return RDGLIN;
}

void
cleanup()
{
}

static void
abort()
{
	cleanup();
	exit(1);
}
