#ifndef lint
static char rcsid[] = "$Header: rdgdec.c,v 1.4 90/06/15 11:01:43 julian Exp $";
#endif lint
/*
 * Decode phase card into reading, association data structures
 * Bruce R. Julian, USGS Menlo Park, Calif.
 */

#include <stdio.h>
#include <math.h>
#include <local/decode.h>
#include <local/mathconst.h>
#include <local/earthconst.h>
#include <local/quake.h>
#include <local/station.h>

#define strneq !strncmp
#define NULL 0

/* Std. errors to use for specified qualities	*/
float	se_tab[5] = {0.05, 0.15, 0.3, 0.75, 1.0e38};		/* Last entry is machine-dependent */

STINFO	si;

char	*index(), *strncpy();
double	ggtogc();
double	atof();
int	atoi();
void	error();
void	cvscd();

int rdgdec(st, line, delays, rp, ap)
	STABLE		*st;	/* Station table		(input) */
	char		*line;	/* Phase card			(input)	*/
	tbool		delays;	/* Use station delays?		(input)	*/
	READING		*rp;	/* Reading structure		(output)*/
	ASSOC		*ap;	/* Association structure  	(output)*/
{
	char		c;
	float		dtp;		/* P-wave time anomaly for station*/
	int		n;		/* No of readings (1 or 2)	*/
	char		trmk[3];
	int		qual, kfmp;
	int		year, month, day, hour, min;
	double		p, s, ccor;

	n = 0;

	/* Station information */
	(void)strncpy(rp->sta.s_code, line, MXSTCOD);
	if (stafetch(st, &rp->sta, &dtp) == FAIL)
		return -1;

	/* Distance, azimuth (convert to radians) */
	ap[n].delta = DECODE(line+52, 4, atof)/REARTH;
	if (! HASDECPT)
		ap[n].delta *= 0.1;
	ap[n].aze = RAD*DECODE(line+56, 3, atof);

	/* Arrival time */
	year	= DECODE(line+9 , 2, atoi);
	month	= DECODE(line+11, 2, atoi);
	day	= DECODE(line+13, 2, atoi);
	hour	= DECODE(line+15, 2, atoi);
	min	= DECODE(line+17, 2, atoi);

	/* P reading */
	if (! strneq(line+19, "99.99", 5)) {
		rp[n].flags = ASSOCIATED | T_EXISTS | USE_T;

		/* Phase code */
		(void)strncpy(rp[n].phcode, line+4, 2);
		rp[n].phcode[2] = '\0';

		/* "Quality" (convert to std. error) */
		qual = DECODE(line+7, 1, atoi);
		if (qual > 4)
			qual = 4;
		ap[n].atsigma = rp[n].setime = se_tab[qual];

		/* Arrival time */
		p	= DECODE(line+19, 5, atof);
		if (! HASDECPT)
			p *= 0.01;
		rp[n].atime = timvar(1900+year, month, day,
				hour, min, p, GREGORIAN);
		if(delays)
			rp[n].atime -= dtp;

		/* Arrival-time residual and weight */
		ap[n].atresid = DECODE(line+24, 5, atof);
		if (! HASDECPT)
			ap[n].atresid *= 0.01;
		ap[n].atwt = DECODE(line+29, 3, atof);

		(void)strncpy(trmk, line+62, 3);
		ccor = DECODE(line+65, 5, atof);
		if (! HASDECPT)
			ccor *= 0.01;
		kfmp = DECODE(line+70, 5, atoi);

		++n;
	}

	/* S reading */
	s = 0.0;
	if (((c=line[37]) == 's' || c == 'S') &&
		! strneq(line+34, "99.99", 5) && ! strneq(line+34, "     ", 5)) {
		/* Copy structures (will be mostly overwritten) */
		rp[n] = rp[0];
		ap[n] = ap[0];

		/* Phase code */
		(void)strncpy(rp[n].phcode, line+36, 2);
		rp[n].phcode[2] = '\0';

		/* "Quality" (convert to std. error) */
		qual = DECODE(line+39, 1, atoi);
		if (qual > 4)
			qual = 4;
		rp[n].setime = se_tab[qual];

		/* Arrival time */
		s = DECODE(line+31, 5, atof);
		rp[n].atime = timvar(1900+year, month, day,
				hour, min, s, GREGORIAN);

		++n;
	}
	return n;
}

int stafetch(st, stp, dtp)
	STABLE		*st;
	STATION		*stp;		/* Station structure	*/
	float		*dtp;		/* P-wave delay		*/
{
	int		i;
	extern int	errno;
	double		x, y, z;

	if ((i = stindex(st, stp->s_code)) == FAIL) {
		errno = ENOSTATION;
		return FAIL;
	}
	if (stget(st, i, &si) != SUCCEED)
		error("Fatal error while geting station");

	cvscd(1.0, HALFPI-ggtogc(si.lat), si.lon, &x, &y, &z);
	stp->gdc[X] = x;
	stp->gdc[Y] = y;
	stp->gdc[Z] = z;
	stp->s_elev = si.ielev;
	*dtp = si.dly;
	return SUCCEED;
}
