#ifndef lint
static char rcsid[] = "$Header: qparam.c,v 1.4 89/03/14 15:24:36 julian Exp $";
#endif lint
/*
 * Extract magnitudes, time intervals, etc.
 * from stream of earthquake summary lines, write to stdout
 */
#include <stdio.h>
#include <local/local.h>
#include <local/cmd_opt.h>
#include <local/decode.h>
#include <local/date_time.h>

#define MAXLINE	256		/* Length of input line			*/
#define INTERV	1		/* Compute inter-event intervals	*/
#define MAG	2		/* Compute magnitudes			*/

int	errno;			/* Error flag				*/
int	use;			/* Which parameter			*/

/*   C O M M A N D - L I N E   A R G U M E N T   P R O C E S S I N G    */

qhelp	(){ if(eargc>2) prt_help(); else prt_doc(); }
qinterv (){ use = INTERV; }
qmag    (){ use = MAG; }

struct command cmd[] = {
   qhelp    , "-help"     , "[opt...]" , "Print documentation",
   qinterv  , "-interval" , ""         , "Inter-event intervals (default)",
   qmag     , "-magnitude", ""         , "Magnitudes",
};

	double	atof();		/* UNIX C library			*/
	int	atoi();		/* UNIX C library			*/
	FILE	*efopen();	/* -lq (efopen.c)			*/
	void	error();	/* UNIX C library			*/
	void	exit();		/* UNIX C library			*/

main(argc, argv)
	int	argc;
	char	*argv[];
{
	errno = 0;

	/* Interpret arguments */
	use = INTERV;
	PROCESS_OPTS;

	/* Process named files (standard input default) */
	if (iarg >= argc)
		do_file(stdin);
	else
		for (; iarg<argc; iarg++) {
			FILE *f = efopen(argv[iarg], "r");
			do_file(f);
			(void)fclose(f);
		}
	exit(0);
}

do_file(f)
	FILE	*f;			/* Input unit			*/
{
	bool	first;			/* First event?			*/
	char	line[MAXLINE];		/* Input buffer			*/
	int	yr, mo, da;		/* Date				*/
	int	hr, mn;			/* Hour, minute			*/
	TIME	ot;			/* Origin time			*/
	TIME	otlast;			/* Preceding origin time	*/
	double	sec;			/* Seconds			*/

	/* Read summary cards, write desired values */
	first = TRUE;
	while (fgets(line, sizeof(line), f) != NULL) {
		switch (use) {
		case MAG:
			(void)printf("%g\n", DECODE(line+46, 4, atof));
			break;
		case INTERV:
			yr = DECODE(line  , 2, atoi) + 1900;
			mo = DECODE(line+2, 2, atoi);
			da = DECODE(line+4, 2, atoi);
			hr = DECODE(line+7, 2, atoi);
			mn = DECODE(line+9, 2, atoi);
			sec = atof(line+12);
			ot = timvar(yr, mo, da, hr, mn, sec, GREGORIAN);
			if (! first)
				(void)printf("%g\n", ot-otlast);
			otlast = ot;
			first = FALSE;
			break;
		}
	}

	exit(0);
}
