#ifndef lint
static char rcsid[]="$Header: qplace.c,v 1.6 88/11/22 12:44:01 julian Exp $";
#endif lint
/*
 * Determine points closest to each event "summary card"
 * Bruce R. Julian, USGS Menlo Park, CA
 */

#include <ctype.h>
#include <stdio.h>
#include <math.h>
#include <local/local.h>
#include <local/decode.h>
#include <local/mathconst.h>
#include <local/earthconst.h>

#define CHUNK	100	/* No. of table entries per alloc	*/
#define MAXLINE	256	/* Length of input line			*/
#define PFILE	"/usr/picker/params/places"
#define SWAP(a, b) { temp = a; a = b; b = temp; }
#define USAGE "Usage:  %s [-n number] [-f place_file] [summary_file...]"

int	errno;			/* Error flag				*/
char	*progname;		/* For error messages			*/

struct place {
	float	lat, lon;
	char	*name;
} *placetable;
int	nplaces;

typedef struct {
	float	dist;
	float	baz;
	char	*name;
} PLACE;
PLACE		*near;		/* Array[0..n] of nearest points	*/

double	atof();			/* UNIX C library			*/
void	delaz();		/* libq.a				*/
void	doit();			/* Defined below			*/
FILE	*efopen();		/* libq.a				*/
char	*emalloc();		/* libq.a				*/
void	error();		/* libq.a				*/
char	*index();		/* Unix C library			*/
char	*realloc();		/* Unix C library			*/
void	refpt();		/* libq.a				*/
char	*strcpy();		/* UNIX C library			*/
char	*strncpy();		/* UNIX C library			*/

main(argc,argv)
	int	argc;
	char	*argv[];
{
	int		c;
	FILE		*f;	/* Input unit				*/
	FILE		*fg;	/* Geography-file unit			*/
	char		*placefile;
	unsigned	n;	/* No. of nearby points wanted		*/
extern	int		optind;
extern	char		*optarg;

	/* Interpret command-line arguments */
	n = 5;
	progname = NULL;
	placefile = PFILE;
	while ((c = getopt(argc, argv, "f:hn:")) != EOF)
		switch (c) {
		case 'f':
			placefile = optarg;
			break;
		case 'n':
			n = atoi(optarg);
			break;
		case 'h':
		default:
			error(USAGE, argv[0]);
			break;
		}
	progname = argv[0];

	/* Read in place table from file */
	fg = efopen(placefile, "r");
	if ((nplaces = getplaces(fg)) <= 0)
		error("Error reading place file");
	(void)fclose(fg);

	/* Allocate table for nearest points */
	near = (PLACE *)emalloc((unsigned)(n+1)*sizeof(PLACE));

	/* Process named files (standard input default) */
	if (optind >= argc)
		doit(stdin, n);
	else
		for ( ; optind<argc; optind++) {
			f = efopen(argv[optind], "r");
			doit (f, n);
			(void)fclose(f);
		}

	/* Free allocated storage */
	free((char *)near);
	while (--nplaces > 0)
		free((char *)placetable[nplaces].name);
	free ((char *)placetable);
	exit(0);
}

/* Convert string to angle:  "degrees[:minutes[:seconds]]" */
double
angle(p)
	char	*p;
{
	double	sign;
	double	d, m, s;

	sign = 1.0;
	if (*p == '+')   p++;
	if (*p == '-') { p++; sign = -1.0; }
	d = atof(p);
	m = s = 0.0;
	if ((p = index(p, ':')) != NULL) {
		m = atof(++p);
		if ((p = index(p, ':')) != NULL)
			s = atof(++p);
	}
	return sign*(d + (m + s/60.0)/60.0);
}

/* Read in file of place names */
int
getplaces(f)
	FILE	*f;
{
	char		c;
	char		buff[MAXLINE];
	unsigned	d;
	unsigned	l;
	int		n;
	char		slat[MAXLINE], slon[MAXLINE];

	/* Allocate initial chunk of space */
	d = CHUNK;
	placetable = (struct place *)emalloc(d*sizeof(struct place));

	/* Read and store places, allocating more chunks as needed */
	n = 0;
	while (fscanf(f, "%s %s", slat, slon) == 2) {
		if (n >= d) {
			/* Allocate another chunk of space */
			d += CHUNK;
			placetable = (struct place *)realloc((char *)placetable, d*sizeof(struct place));
			if (placetable == NULL)
				error("Can't realloc");
		}
		placetable[n].lat = RAD*angle(slat);
		placetable[n].lon = RAD*angle(slon);

		/* Get place name, skipping initial white space and discarding newline */
		while (isspace(c = getc(f)))
			;
		(void)ungetc(c, f);
		l = strlen(fgets(buff, sizeof(buff), f));
		buff[l-1] = '\0';
		placetable[n].name = emalloc(l);
		(void)strcpy(placetable[n].name, buff);

		n++;
	}
	return n;
}

/* Process one summary file */
void
doit(f, n)
	FILE		*f;
	unsigned	n;
{
	char	line[MAXLINE];	/* Input buffer				*/
	double	dist, az, baz;
	int	i, j;		/* Dummy loop indices			*/
	double	lat, lon;	/* Epicentral coordinates		*/
	PLACE	temp;		/* Scratch area for swapping		*/
static	char	*bnames[] = {	"N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE",
				"S", "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW",
				"N"
};
#define BEARING(z) bnames[(int)(z*(8.0/PI) + 0.5)]

	/* Read summary cards and write, followed by list of nearest points */
	while(fgets(line, sizeof(line), f) != NULL) {
		fputs(line, stdout);

		/* Read latitude and longitude of event */
		lat = fabs(DECODE(line+18, 2, atof)) + DECODE(line+21, 5, atof)/60.0;
		if (line[20] == 'S') lat *= -1.0;
		lon = fabs(DECODE(line+27, 3, atof)) + DECODE(line+31, 5, atof)/60.0;
		if (line[30] != 'E') lon *= -1.0;
		if (lat == 0.0 && lon == 0.0)
			continue;

		/* Put sentinel and "infinite" distances into table near[] */
		near[0].dist = -1.0;
		for (i=1; i<=n; i++)
			near[i].dist = TWOPI;

		/* Compute distances, azimuths to tabulated places,	*/
		/* remember n closest in table near[1..n]		*/
		refpt(RAD*lat, RAD*lon);
		for (i=0; i<nplaces; i++) {
			delaz(placetable[i].lat, placetable[i].lon,
				&dist, &az, &baz);
			if (dist < near[n].dist) {
				near[n].dist = dist;
				near[n].baz = baz;
				near[n].name = placetable[i].name;
				for (j=n; dist<near[j-1].dist; --j)
					SWAP(near[j], near[j-1]);
			}
		}

		/* Print remembered (closest) places */
		for (i=1; i<=n; i++)
			fprintf(stdout, "%3.0f km (%3.0f miles) %-3s (%3.0f degrees) of %s\n",
				REARTH*near[i].dist, MILE*REARTH*near[i].dist,
				BEARING(near[i].baz), DEG*near[i].baz,
				near[i].name);
	}
}
