#ifndef lint
static char rcsid[]="$Header: qprint.c,v 1.8 92/06/10 03:51:32 julian Exp $";
#endif lint
/*
 * Filter to convert data structures to printable form
 * Bruce R. Julian, USGS Menlo Park, Calif, 12 Dec 1983
 */

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <signal.h>
#include <local/cmd_opt.h>
#include <local/mathconst.h>
#include <local/earthconst.h>
#include <local/quake.h>

#define MAXLINE		133
#define RDGLIN		1
#define EVLIN		2

	FILE	*efopen();
	void	exit();
	double	gctogg();
	char	*index();
	double	modf();
	double	sign();
	char	*sprintf();
	void	def_hdr(), def_o(), def_rdg();
	void	h71_hdr(), h71_o(), h71_rdg();
	void	simul3_hdr(), simul3_o(), simul3_rdg();

int	dordgs;				/* Controls printing of readinsg*/
int	header;				/* Controls printing of header	*/
double	dfactor;			/* Multiplies delta		*/
ORIGIN	org;				/* Origin structure		*/
void	(*pr_o)(),			/* Origin-printing function	*/
	(*pr_hdr)(),			/* Header-printing function	*/
	(*pr_rdg)();			/* Reading-printing function	*/
READING	rdg;				/* Reading structure		*/
ASSOC	assoc;				/* Association structure	*/


/*--------------- A R G U M E N T   P R O C E S S I N G ----------------*/

fhelp	(){ if(eargc>2) prt_help(); else prt_doc(""); }
fh71    (){ pr_o = h71_o; pr_hdr = h71_hdr; pr_rdg = h71_rdg; }
fsimul3 (){ pr_o = simul3_o; pr_hdr = simul3_hdr; pr_rdg = simul3_rdg; }
fnohead (){ header = FALSE; }
fnordgs (){ dordgs = FALSE; fnohead(); }
fkm     (){ dfactor = REARTH; }

struct command cmd[] = {
	fhelp    , "-help"    , "[opt...]", "documentation help",
	fh71     , "-h71"     , ""        , "hypo71 format",
	fsimul3  , "-simul3"  , ""        , "simul3 event-data format",
	fnohead  , "-noheader", ""        , "supress header on output",
	fnordgs  , "-noreadings", ""      , "supress readings on output",
	fkm      , "-km"      , ""        , "print delta in kilometers",
};


main(argc,argv)
	int	argc;
	char	*argv[];
{
	double	angmin();
	void	cleanup();
	FILE	*fin, *fout;
	void	init();
	char	*time_str();

	/* Interpret command-line arguments */
	init();
	PROCESS_OPTS
	fin = stdin;
	fout = stdout;

	while (fread((char *)&org, sizeof(org), 1, fin) == 1) {
		/* Origin */
		(*pr_o)(fout, &org);

		/* Readings */
		if (header)
			(*pr_hdr)(fout);
		while(fread((char *)&rdg, sizeof(rdg), 1, fin) == 1 &&
		      fread((char *)&assoc,  sizeof(assoc),  1, fin) == 1) {
			if (! *rdg.sta.s_code) {	/* Empty string? */
				(void)fprintf(fout, "\n\n");
				break;
			}
			if (dordgs)
				(*pr_rdg)(fout, &org, &rdg, &assoc);
		}
	}
	exit(0);
}

/*---------------------------  M I S C  -----------------------------*/

void
init()
{
	/* Set defaults */
	header	= TRUE;
	dordgs	= TRUE;
	dfactor	= DEG;
	pr_o	= def_o;
	pr_hdr	= def_hdr;
	pr_rdg	= def_rdg;
}

double
angmin(a)
	float	a;
{
	double	d, m;

	d = (int)a;
	m = 60.0*(a - d);
	if (m < 0.0)
		m = (-m);
	return m;
}

char*
time_str(t)
	TIME	t;
{
	double		sec;
	int		yr, mo, da, hr, mn;
	static char	buf[33];
	static char	*month[] = { "???", "Jan", "Feb", "Mar", "Apr", "May", "Jun",
				            "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

	datime(t, &yr, &mo, &da, &hr, &mn, &sec, GREGORIAN);
	return sprintf(buf, "%d %s %d   %02d:%02d:%06.3f UTC",
		yr, month[(mo>0 && mo < 13 ? mo : 0)], da, hr, mn, sec);
}
/**/
/*   O U T P U T   R O U T I N E S   ( D E F A U L T   F O R M A T )   */

/* Origin */
void
def_o(f, o)
	FILE	*f;
	ORIGIN	*o;
{
	double	dlt, dln;

	(void)fprintf(f, "%s\n", time_str(o->o_time));
	dlt = DEG*gctogg(o->o_lat);
	dln = DEG*o->o_lon;
	(void)fprintf(f, "Lat:  %9.5f      Lon:  %10.5f", dlt, dln);
	(void)fprintf(f, "      Depth:  %.3f km\n", o->o_depth);
	(void)fprintf(f, "      %3d:%06.3f %c         %4d:%06.3f %c\n",
		(int)ABS(dlt), angmin(dlt), (dlt>=0.0 ? 'N' : 'S'),
		(int)ABS(dln), angmin(dln), (dln>=0.0 ? 'E' : 'W'));
}

/* Reading header */
void
def_hdr(f)
	FILE	*f;
{
	(void)fprintf(f, "\nSTA   DELTA  AZ BAZ");
	(void)fprintf(f, "   PHASE    TIME     RES   SIGMA  WT  IMP");
	(void)fprintf(f, "   AMPLITUDE    FREQ\n\n");
}

/* Reading */
void
def_rdg(f, o, r, a)
	FILE	*f;
	ORIGIN	*o;
	READING	*r;
	ASSOC	*a;
{
	/* Staion code, distance, azimuth */
	(void)fprintf(f,
		"%-4.4s %6.2f %3.0f %3.0f",
		r->sta.s_code,
		dfactor*a->delta,
		DEG*a->aze,
		DEG*a->azs
	);

	/* Arrival time and associated info. */
	(void)fprintf(f,
		" %-8.8s %7.3f %7.3f %6.3f %3.2f %3.2f",
		r->phcode,
		r->atime - o->o_time,
		a->atresid,
		a->atsigma,
		a->atwt,
		a->atimp
	);

	/* Amplitude and associated info. */
	(void)fprintf(f,
		" %10.3e %10.3e",
		sign(r->pol)*r->amp,
		r->freq
	);
	(void)fprintf(f, "\n");
}

/* Return 1.0, -1.0, or 0.0, depending on polarity */
double
sign(c)
	char	c;
{
static	char	plus[] = "uUcC+";	
static	char	minus[] = "dDrR-";	
	if (index(plus , c) != NULL)	return  1.0;
	if (index(minus, c) != NULL)	return -1.0;
					return  0.0;
}
/**/
/*   O U T P U T   R O U T I N E S   ( H Y P O 7 1   F O R M A T )   */

void
h71_o(f, o)
	FILE	*f;
	ORIGIN	*o;
{
	double	sec;
	int	yr, mo, da, hr, mn;
	double	d;

	datime(o->o_time, &yr, &mo, &da, &hr, &mn, &sec, GREGORIAN);
	(void)fprintf(f, "%02d%02d%02d %02d%02d %05.2f",
		yr%100, mo, da, hr, mn, sec);
	d = DEG*gctogg(o->o_lat);
	(void)fprintf(f, " %2d-%05.2f", (int)ABS(d), angmin(d));
	d = DEG*o->o_lon;
	(void)fprintf(f, " %3d-%05.2f", (int)ABS(d), angmin(d));
	(void)fprintf(f, " %6.3f", o->o_depth);
	(void)fprintf(f, "\n");
}

void
h71_hdr(f)
	FILE	*f;
{
}

void
h71_rdg(f, o, r, a)
	FILE	*f;
	ORIGIN	*o;
	READING	*r;
	ASSOC	*a;
{
}
/**/
/*   O U T P U T   R O U T I N E S   ( S I M U L 3   F O R M A T )   */

void
simul3_o(f, o)
	FILE	*f;
	ORIGIN	*o;
{
	double	sec;
	int	yr, mo, da, hr, mn;
	double	d;

	datime(o->o_time, &yr, &mo, &da, &hr, &mn, &sec, GREGORIAN);
	(void)fprintf(f, "%02d%02d%02d %02d%02d %05.2f",
		yr%100, mo, da, hr, mn, sec);
	d = DEG*gctogg(o->o_lat);
	(void)fprintf(f, " %2d %05.2f", (int)ABS(d), angmin(d));
	d = DEG*o->o_lon;
	(void)fprintf(f, "%3d %05.2f", (int)ABS(d), angmin(d));
	(void)fprintf(f, " %7.2f", o->o_depth);
}

void
simul3_hdr(f)
	FILE	*f;
{
}

void
simul3_rdg(f, o, r, a)
	FILE	*f;
	ORIGIN	*o;
	READING	*r;
	ASSOC	*a;
{
static	int	n = 0;		/* No. of rdgs. so far on output line	*/

	if ((n++)%6 == 0)
		(void)fprintf(f, "\n");
	(void)fprintf(f, "%4.4s%1c%1c%6.3f",
		r->sta.s_code, r->phcode[0], r->qual, r->atime-o->o_time);
}
