#ifndef lint
static char rcsid[] = "$Header: mkregtab.c,v 1.10 87/10/08 10:46:48 ppicker Exp $";
#endif lint
/*
 * Convert Fred Kline's qplreg file to C declaration statement
 * Derive array of region numbers for latitude strips of width ww
 *
 * $Log:	mkregtab.c,v $
 * Revision 1.10  87/10/08  10:46:48  ppicker
 * Accepts new qplreg format (with explicit region numbers).
 * Backward compatible:  old qplreg format still works.
 * 
 * Revision 1.9  87/07/16  11:48:44  julian
 * Put in dummy table entries for obsolete regions,
 * so as to keep regions numbers compatible with Fred Klein.
 * 
 * Revision 1.8  86/09/18  11:12:54  julian
 * Minor cleanup
 * 
 * Revision 1.7  86/08/28  12:16:47  julian
 * Ignores "super regions" (marked with % character)
 * 
 * Revision 1.6  86/08/24  16:49:22  julian
 * Added fflush(stdout) to help in pinpointing errors.
 * Set errno=0 before checking latitude limits.
 * 
 * Revision 1.5  86/06/30  11:30:02  julian
 * Use rank table instead of index table for search order.
 * 
 * Revision 1.3  86/06/27  15:32:16  julian
 * Option to sort strips into optimal search order.
 * 
 */
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <local/local.h>
#include "region.h"

#define MAXLINE	256
#define MAXBUF 20

/* ASCII-to-number conversion		*/
/* char *s is beginning of field	*/
/* int	j  is field length		*/
/*     	f is conversion function	*/
#define DECODE(s, j, f) (buff[j]='\0', f(strncpy(buff, s, j)))

/* Buffer used by DECODE macro */
char	buff[MAXBUF];

char	*progname;	/* For error messages			*/
double	lat[MAXVERT],	/* Vertices of region			*/
	lon[MAXVERT];
int	nreg;		/* No. of regions			*/
int	srank[102+1];	/* Rank table for optimal search order	*/

double		atof();
FILE		*efopen();
void		error();
static int	fcomp();	/* Comparison function for sort	*/
char		*index();
static char	*match();
char		*strncpy();

/* Functions to read one line, with pushback */
char	pushbuf[MAXLINE];
int	pushed = FALSE;

char *
getline(s, n, f)
	char	*s;
	int	n;
	FILE	*f;
{
	if (pushed) {
		pushed = FALSE;
		return pushbuf;
	}
	return fgets(s, n, f);
}

/* N.B.: Only one pushback buffer. */
/* Can read only one unit at a time */
void
ungetline(s)
	char	*s;
{
	strcpy(pushbuf, s);
	pushed = TRUE;
}
/**/
main(argc, argv)
	int	argc;
	char	*argv[];
{
	extern	errno;
	double	d, m;		/* Degrees, minutes		*/
	FILE	*f;		/* Input unit			*/
	double	latmax,		/* Extremeties of a region	*/
		latmin;
	int	lmax,		/* Extreme strips for a region	*/
		lmin;
	int	array[MAXSTRIPS][40];	/* Regions in latitude strips	*/
	int	count[MAXSTRIPS];/* No. of regions in each strip	*/
	int	kmax;		/* North non-empty strip	*/
	int	maxcount;	/* Biggest count for strip	*/
	int	ireg;
	int	i;		/* Dummy loop index		*/
	int	j;		/* Dummy loop index		*/
	int	k;		/* Dummy loop index		*/
	int	nvert;		/* Vertex number		*/
	char	*p;		/* Scratch pointer		*/
	char	line[MAXLINE];	/* I/O buffer			*/

	errno = 0;
	if (argc < 2 || argc > 3)
		error("Usage:  mkregtab region_file [order_file]");
	progname = argv[0];
	f = efopen(argv[1], "r");

	/* Initialize count[] */
	for (k=0; k<MAXSTRIPS; ++k)
		count[k] = 0;

	/* Write beginning of declaration statement for region boundaries */
	printf("static struct reg_poly reg_tab[] = {\n");
	printf("\t(char *)0,\n\t0,\n\t{0},\n\t{0},\n");

	/* Loop over regions */
	nreg = 0;
	while (getline(line, sizeof(line), f) != NULL) {
		++nreg;
		if (index(line, '%') != (char *)NULL) {
			/* Obsolete region: write dummy entry */
			while (getline(line, sizeof(line), f) != NULL)
				if (match(line  , "------") != NULL) {
					ungetline(line);
					break;
				}
			printf("\t(char *)0,\n\t0,\n\t{0},\n\t{0},\n");
			continue;
		}

		/* Check region number */
		if (sscanf(line, "%d", &ireg) == 1 && ireg != nreg)
			error("Region number %d out of order.", ireg);

		/* Write out region name */
		for (p=line+strlen(line)-1; *p==' ' || *p=='-'; --p)
			*p = '\0';
		for (p=line; isdigit(*p) || isspace(*p) || *p=='-'; p++)
			;
		printf("\t\"%.22s\",\n", p);
		fflush(stdout);

		/* Read in vertices of region */
		nvert = 0;
		latmin = 100.;
		latmax = -100.;
		while (getline(line, sizeof(line), f) != NULL) {
			if (match(line  , "------") != NULL) {
				ungetline(line);
				break;
			}
			d = DECODE(line+5, 2, atof);
			m = DECODE(line+9, 4, atof);
			lat[nvert] = d + m/60.;
			if (lat[nvert] > latmax) latmax = lat[nvert];
			if (lat[nvert] < latmin) latmin = lat[nvert];
			d = DECODE(line+14, 3, atof);
			m = DECODE(line+19, 4, atof);
			lon[nvert] = -(d + m/60.);
			if (++nvert >= MAXVERT)
				error("Too many vertices");
		}

		/* Find latitude strips that pass through region */
		errno = 0;
		if ((lmin = floor((latmin - LATSOUTH)/DLAT)) < 0)
			error("Region too far south");
		if ((lmax = floor((latmax - LATSOUTH)/DLAT)) > MAXSTRIPS)
			error("Region too far north");
		for (k=lmin; k<=lmax; ++k) {
			array[k][count[k]] = nreg;
			if (++(count[k]) >= DIM(array[0]))
				error("Too many regions in strip %d. (Max %d)", k, DIM(array[0]));
		}

		/* Write out vertices */
		printf("\t%d,\n", nvert);
		printf("\t{ ");
		for (j=0; j<nvert; ++j)
			printf("%g, ", lat[j]);
		printf("},\n");
		printf("\t{ ");
		for (j=0; j<nvert; ++j)
			printf("%g, ", lon[j]);
		printf("},\n");
	}

	/* Write end of declaration statement for region boundaries */
	printf("};\n\n");
	fclose(f);

	/* Read optimal search order, if available */
	errno = 0;
	i = 0;
	if (argc > 2) {
		f = efopen(argv[2], "r");
		while (fscanf(f, "%d", &j) == 1) {
			if (j < 1 || j >= DIM(srank))
				error("Region no. %d out of range (file \"%s\")", j, argv[2]);
			srank[j] = ++i;
		}
		fclose(f);
		if (i != nreg)
			error("Wrong no. of values in file %s", argv[2]);
	}

	/* Write out array */
	maxcount = 0;
	kmax = 0;
	for (k=0; k<DIM(count); ++k) {
		if (count[k] > maxcount)
			maxcount = count[k];
		if (count[k] > 0)
			kmax = k;
	}
	printf("static short array[%d][%d] = {\n", kmax+1, maxcount+1);
	for (k=0; k<=kmax; ++k) {
		if (i > 0)
			qsort((char *)array[k], count[k], sizeof(array[0][0]), fcomp);
		printf("\t{");
		for (j=0; j<(count[k]); ++j)
			printf(" %d,", array[k][j]);
		printf(" 0 },\n");
	}
	printf("};\n");

	exit(0);
}

/*
 * Comparison function for sorting regions into optimal order
 */
static int
fcomp(p1, p2)
	register
	int	*p1, *p2;	/* Region numbers	*/
{
	if (*p1 < 1 || *p2 < 1 || *p1 > nreg || *p2 > nreg)
		error("Region no. out out of range in qsort");
	return srank[*p1] - srank[*p2];
}

/*
 * Return address of substring t in s, NULL if none
 */
static char
*match(s, t)
	char	*s, *t;
{
	register char	*p, *q;

	for ( ; *s; s++) {
		for (p=s, q=t; *q && *p==*q; p++, q++)
			;
		if (*q == '\0')
			return s;
	}
	return (char *)NULL;
}
