#ifndef lint
static char rcsid[]="$Header: qregion.c,v 1.4 87/07/16 11:49:44 julian Exp $";
#endif lint
/*
 * Determine regions corresponding to event "summary cards"
 * Bruce R. Julian and Mary O'Neill, USGS Menlo Park, CA
 * 
 *$Log:	qregion.c,v $
 * Revision 1.4  87/07/16  11:49:44  julian
 * Three-digit region numbers.
 * 
 * Revision 1.3  86/09/18  11:17:16  julian
 * Prints region number even if zero.
 * 
 * Revision 1.2  86/06/27  16:17:39  julian
 * Added optional input file.
 * Added usage message.
 * 
 * Revision 1.1  86/06/27  15:33:07  julian
 * Initial revision
 * 
 */

#include <stdio.h>
#include <math.h>
#include <local/local.h>
#include "region.h"

/* ASCII-to-number conversion (for reading summary cards, etc.) */
/* char *s is beginning of field	*/
/* int	j  is field length		*/
/*     	f is conversion function	*/
#define DECODE(s, j, f) (buff[j]='\0', f(strncpy(buff, s, j)))

/* Limits */
#define MAXLINE 	150		/* Length of input line		*/
#define MAXBUF		20		/* Field length for decoding	*/

/* Error handling */
int	errno;			/* Error flag			*/
char	*progname;		/* For error messages		*/

/* Buffer used by DECODE macro */
static char	buff[MAXBUF];

/* Functions */
double	atof();
FILE	*efopen();
void	error();
char	*index();
char	*strncpy();

main(argc,argv)
	int	argc;
	char	*argv[];
{
	double	d, m;		/* Degrees, minutes		*/
	FILE	*f;		/* Input unit			*/
	int	i;		/* Region number		*/
	double	lat, lon;	/* Epicentral coordinates	*/
	char	line[MAXLINE];	/* Input buffer			*/

	/* Interpret arguments */
	if ((argc > 2) || (argc == 2 && argv[1][0] == '-'))
		error("Usage:  qregion [summary_file]");
	if (argc == 2)
		f = efopen(argv[1], "r");
	else
		f = stdin;

	/* Read summary cards and write, followed by region number and name */
	while(fgets(line, sizeof(line), f) != NULL) {
		*index(line, '\n') = '\0';	/* Delete newline character */
		fputs(line, stdout);
		d = DECODE(line+18, 2, atof);
		m = DECODE(line+21, 5, atof);
		lat = ABS(d) + m/60.;
		if (line[20] == 'S')
			lat *= -1.;
		d = DECODE(line+27, 3, atof);
		m = DECODE(line+31, 5, atof);
		lon = ABS(d) + m/60.;
		if (line[30] != 'E')
			lon *= -1.;
		i = regno(lat, lon);
		printf(" (%03d)", i);
		if (i != 0)
			printf(" %s", regname(i));
		printf("\n");
	}
	if (f != stdin)
		fclose(f);

	exit(0);
}
