#ifndef lint
static char rcsid[]="$Header: qselect.c,v 1.13 92/08/13 19:04:44 julian Exp $";
#endif lint
/*
 * Select event "summary cards" matching specified criteria
 * Bruce R. Julian, USGS Menlo Park, CA
 */

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <local/local.h>
#include <local/date_time.h>
#include <local/cmd_opt.h>
#include <local/mathconst.h>
#include <local/earthconst.h>
#include <local/decode.h>

/* Limits */
#define MAXLINE 	150		/* Length of input line		*/
#define MAXVERT		50		/* No. of vertices of polygon	*/
#define	MAXCATALOGFILES	1023		/* Max # of catalog files to search */

/* Variables that control testing */
static bool	do_lat,			/* Which tests to do	*/
		do_lon,
		do_depth,
		do_otime,
		do_mag,
		do_close,
		do_nsta,
		do_rms,
		do_gap,
		do_qual,
		do_delta,
		do_poly;
static bool	search_catalog;		/* Search catalog.	*/
static bool	verbose;		/* Verbose information.	*/
static bool	negate;			/* Reverse sense of test*/
static char	*quals;			/* Quality codes	*/
static double	clomin, clomax;		/* Closest station	*/
static double	gapmin, gapmax;		/* Gap			*/
static double	latmin, latmax;		/* Latitude 		*/
static double	lonmin, lonmax;		/* Longitude 		*/
static double	magmin, magmax;		/* Magnitude		*/
static int	nstmin, nstmax;		/* No. of stations	*/
static double	rmsmin, rmsmax;		/* RMS residual		*/
static TIME	otmin,  otmax;		/* Origin time		*/
static double	delmin, delmax;		/* Epicentral distance	*/
static double	plat[MAXVERT+1],	/* Vertices of polygon	*/
		plon[MAXVERT+1];
static int	nvert;			/* No. of vertices	*/
static double	zmin, zmax;		/* Focal depth		*/
static double	ct0, st0, phi0;		/* Ref. pt. for delta	*/
static double	cdmin, cdmax;		/* cos(delmin), cos(delmax)	*/

int	errno;			/* Error flag			*/

	double	atof();		/* UNIX C library			*/
	int	atoi();		/* UNIX C library			*/
	double	dmax();		/* -lq (dmax.c)				*/
	double	dmin();		/* -lq (dmin.c)				*/
	FILE	*efopen();	/* -lq (efopen.c)			*/
	void	error();	/* -lq (error.c)			*/
	void	exit();		/* UNIX C library			*/
	int	strncmp();	/* UNIX C library			*/
	char	*strncpy();	/* UNIX C library			*/
	char	*index();	/* UNIX C library			*/
	double	gctogg();	/* -lq (cvlat.c)			*/
	double	ggtogc();	/* -lq (cvlat.c)			*/
	FILE	*ecfopen();
/**/
/*----  C O M M A N D - L I N E   A R G U M E N T   P R O C E S S I N G  ----*/
check(mn, mx, s)	/* Be sure mn <= mx */
	double mn, mx;
	char *s;
{
	if (mn > mx) error("minimum %s value greater than maximum", s);
}

/* Convert string to angle */
/* "degrees[:minutes[:seconds]]" */
double
angle(p)
	char	*p;
{
	double	sign;
	double	d, m, s;

	sign = 1.0;
	if (*p == '+')
		p++;
	if (*p == '-') {
		sign = -1.0;
		p++;
	}
	d = atof(p);
	m = s = 0.;
	if ((p = index(p, ':')) != NULL) {
		m = atof(++p);
		if ((p = index(p, ':')) != NULL)
			s = atof(++p);
	}
	return sign*(d + (m + s/60.)/60.);
}

/* Check whether string looks like an angle */
bool
isangle(p)
	char	*p;
{
	if (p == (char *)NULL)
		return FALSE;
	while (*p)
		if (index("+-0123456789.:", *p++) == (char *)NULL)
			return FALSE;
	return TRUE;
}

/* Convert string to TIME */
/* "[cc]yymmddhhmmss.ss...[+-offset]" */
TIME epoch(s)
	char	*s;
{
	double	se;
	int	yr, mo, da, hr, mn;
	char	*p;
	double	offset();

	/* Get year (twentieth century default) */
	for (p=s; isdigit(*p); p++)
		;
	switch (p - s) {
	case 14:
		yr = DECODE(s, 4, atoi);
		s += 2;
		break;
	case 12:
		yr = 1900 + DECODE(s, 2, atoi);
		break;
	default:
		error ("Bad date format: \"%s\"", s);
	}

	/* Get month, day, hour, minute, second */
	mo = DECODE(s+ 2, 2, atoi);
	da = DECODE(s+ 4, 2, atoi);
	hr = DECODE(s+ 6, 2, atoi);
	mn = DECODE(s+ 8, 2, atoi);
	se = atof(s+10);

	/* Skip to end of seconds field */
	s += 12;
	if (*s == '.')
		s++;
	while(isdigit(*s))
		s++;

	return timvar(yr, mo, da, hr, mn, se, GREGORIAN) + offset(s);
}

static double offset(s)
	char	*s;
{
	double	sign, x, offset;

	switch (*s) {
	case '\0':	return 0.;
	case '+':	sign = 1.;	break;
	case '-':	sign = -1.;	break;
	default:
		error("Illegal sign character \"%c\"", *s);
	}
	s++;
	offset = 0.;
	while (isdigit(*s) || *s == '.') {
		x = atof(s);
		while (isdigit(*s))	s++;
		if (*s == '.')		s++;
		while (isdigit(*s))	s++;
		switch (*s) {
		case 'd': case 'D': x *= DAY	; break;
		case 'h': case 'H': x *= HOUR	; break;
		case 'm': case 'M': x *= MINUTE	; break;
		case 's': case 'S': x *= SECOND	; break;
		default:
			error("Bad time offset unit \"%c\"", *s);
		}
		s++;
		offset += x;
	}
	return sign*offset;
}

/* Smaller of two double values */
double fmin2(a, b)
	double	a, b;
{
	return (a<b ? a : b);
}

/* Larger of two double values */
double fmax2(a, b)
	double	a, b;
{
	return (a>b ? a : b);
}

qhelp	(){ if(eargc>2) prt_help(); else prt_doc("[file...]"); }
qneg    (){ negate = TRUE; }
qlat    (){ latmin = angle(aarg());   latmax = angle(aarg());   check(latmin, latmax, "latitude");  do_lat = TRUE; }
qlon    (){ lonmin = angle(aarg());   lonmax = angle(aarg());   check(lonmin, lonmax, "longitude"); do_lon = TRUE; }
qdepth  (){ zmin   = narg();   zmax   = narg();   check(zmin, zmax, "depth");         do_depth = TRUE; }
qtime   (){ otmin  = epoch(aarg());   otmax  = epoch(aarg());   check(otmin, otmax, "time");        do_otime = TRUE; }
qmag    (){ magmin = narg();   magmax = narg();   check(magmin, magmax, "magnitude"); do_mag = TRUE; }
qclose  (){ clomin = narg();   clomax = narg();   check(clomin, clomax, "close");     do_close = TRUE; }
qnsta   (){ nstmin = narg();   nstmax = narg();   check((double)nstmin, (double)nstmax, "nstations"); do_nsta = TRUE; }
qrms    (){ rmsmin = narg();   rmsmax = narg();   check(rmsmin, rmsmax, "rms");       do_rms = TRUE; }
qgap    (){ gapmin = narg();   gapmax = narg();   check(gapmin, gapmax, "gap");       do_gap = TRUE; }
qqual   (){ char *p; quals=aarg(); for(p=quals;*p;p++) *p = (isupper(*p) ? tolower(*p) : *p); do_qual = TRUE; }
qcat    (){ search_catalog = TRUE; }
qverbose(){ verbose = TRUE; }
qdelta  ()
{
	double	rplat;
	double	dlon, sdlon;

	rplat  = RAD*angle(aarg());
	phi0   = RAD*angle(aarg());
	delmin = narg()/REARTH;
	delmax = narg()/REARTH;
	check(delmin, delmax, "delta");

	/* Do all possible preliminary arithmetic */
	rplat = ggtogc(rplat);
	st0 = cos(rplat);
	ct0 = sin(rplat);
	cdmin = cos(delmin);
	cdmax = cos(delmax);

	/* Shrink lat-lon limits to barely enclose delta annulus */
	if (! do_lat) { latmin =  -90.; latmax =  90.; }
	if (! do_lon) { lonmin = -180.; lonmax = 180.; }
	latmin = fmax2(latmin, DEG*gctogg(rplat-delmax));
	latmax = fmin2(latmax, DEG*gctogg(rplat+delmax));
	if (st0 > 0.) {
		sdlon = sin(delmax)/st0;
		if (sdlon <= 1.) {
			dlon = asin(sdlon);
			lonmin = fmax2(lonmin, DEG*(phi0-dlon));
			lonmax = fmin2(lonmax, DEG*(phi0+dlon));
		}
	}
	do_delta = do_lat = do_lon = TRUE;
}

qpoly   ()
{
	/* Read in vertices of polygon */
	for (nvert=0; isangle(peekarg()) && nvert<MAXVERT; nvert++) {
		plat[nvert] = angle(aarg());
		plon[nvert] = angle(aarg());
	}
	if (nvert < 3)
		error("Polygon needs at least three vertices");
	plat[nvert] = plat[0];
	plon[nvert] = plon[0];
	nvert++;

	/* Shrink latitude-longitude limits to just include polygon */
	if (! do_lat) { latmin =  -90.; latmax =  90.; }
	if (! do_lon) { lonmin = -180.; lonmax = 180.; }
	latmin = fmax2(latmin, dmin(plat, nvert));
	latmax = fmin2(latmax, dmax(plat, nvert));
	lonmin = fmax2(lonmin, dmin(plon, nvert));
	lonmax = fmin2(lonmax, dmax(plon, nvert));
	do_poly = do_lat = do_lon = TRUE;
}

struct command cmd[] = {
   qhelp    , "-help"     , "[opt...]" , "Print documentation",
   qneg     , "-negate"   , ""         , "Print events that FAIL test",
   qlat     , "-latitude" , "min max"  , "Latitude range (d[:m[:s]])",
   qlon     , "-longitude", "min max"  , "Longitude range (d[:m[:s]]) N.B. West is negative!",
   qdepth   , "-depth"    , "min max"  , "Depth range (km)",
   qtime    , "-time"     , "min max"  , "Origin-time interval ([cc]yymmddhhmmss[+-offset])",
   qmag     , "-magnitude", "min max"  , "Magnitude range",
   qclose   , "-closest"  , "min max"  , "Range of closest station (km)",
   qnsta    , "-nstations", "min max"  , "Range of no. of stations",
   qrms     , "-rms"      , "min max"  , "Range of r.m.s. residual (s)",
   qgap     , "-gap"      , "min max"  , "Range of azimuthal gap (deg)",
   qqual    , "-quality"  , "letters"  , "Quality codes (e.g. abc)",
   qdelta   , "-delta"    , "lat lon min max"  , "Epicentral distance (km)",
   qpoly    , "-polygon"  , "lat lon...","Polygon (give 3 or more vertices)",
   qcat     , "-catalog"  , ""         , "Search catalog - uses CUSPCATDIR variable",
   qverbose , "-verbose"  , ""         , "Verbose output during search",
};

/*--------------------------  M A I N  ------------------------------*/

main(argc,argv)
	int	argc;
	char	*argv[];
{
	/* Set defaults */
	negate = FALSE;
	search_catalog = verbose = FALSE;
	do_lat = do_lon = do_depth = do_otime = do_mag = 
	do_close = do_nsta = do_rms = do_gap = do_qual =
	do_delta = do_poly = FALSE;		/* Do nothing		*/

	/* Interpret command-line options */
	PROCESS_OPTS;

	/* Process files (standard input default) */
	if ((iarg >= argc) && ! search_catalog) {
		if (verbose) fprintf (stderr, "processing stdin ...\n");
		do_file(stdin);
	}
	else {
		if (search_catalog) {
			TIME catmin = otmin;
			TIME catmax = otmax;
			char **cp, **find_catalog();

			if ((cp = find_catalog(catmin,catmax))==NULL) {
				fprintf(stderr,"unable to find catalog\n");
				exit(1);
			}
			while (*cp != NULL) {
				FILE *f = ecfopen(*cp, "r");
				if (verbose) fprintf(stderr,"processing %s ... ", *cp);
				do_file(f);
				(void)fclose(f);
				cp++;
			}
		}
		for (; iarg<argc; iarg++) {
			FILE *f = ecfopen(argv[iarg], "r");
			if (verbose) fprintf(stderr,"processing %s ... ", argv[iarg]);
			do_file(f);
			(void)fclose(f);
		}
	}
	exit(0);
}

do_file(f)
	FILE	*f;
{
	char		c;
	double		close;		/* Distance to closest station	*/
	double		ct1, st1;
	double		cdelt;
	double		cdlon;
	double		d, m;		/* Degrees, minutes		*/
	double		gap;		/* Largest azimuthal gap	*/
	double		lat, lon, z;	/* Hypocentral coordinates	*/
	double		mag;		/* Magnitude			*/
	int		nsta;		/* No. of stations		*/
	TIME		ot;		/* Origin time			*/
	double		rms;		/* Root-mean-square residual	*/
	char		line[MAXLINE];
	double		se;
	int		yr, mo, da, hr, mn;
	int		lines = 0;
	int		selected = 0;

	/* Read lines, test, and write those that pass */
	while(fgets(line, sizeof(line), f) != NULL) {
		++lines;
		if (do_otime) {	/* ORIGIN TIME */
			yr = DECODE(line  , 2, atoi) + 1900;
			mo = DECODE(line+2, 2, atoi);
			da = DECODE(line+4, 2, atoi);
			hr = DECODE(line+6, 2, atoi);
			mn = DECODE(line+8, 2, atoi);
			se = DECODE(line+10,4, atof)/100.;
			ot = timvar(yr, mo, da, hr, mn, se, GREGORIAN);
			if (ot < otmin || ot > otmax)
				if (negate) goto OUTPUT;
				else        continue;
 		}
		if (do_lat) {	/* LATITUDE */
			d = DECODE(line+14, 2, atof);
			m = DECODE(line+17, 4, atof)/100.;
			lat = fabs(d) + m/60.;
			if (line[16] == 'S')
				lat *= -1.;
			if (lat < latmin || latmax < lat)
				if (negate) goto OUTPUT;
				else        continue;
		}
		if (do_lon) {	/* LONGITUDE */
			d = DECODE(line+21, 3, atof);
			m = DECODE(line+25, 4, atof)/100.;
			lon = fabs(d) + m/60.;
			if (line[24] != 'E')
				lon *= -1.;
			if (lon < lonmin || lonmax < lon)
				if (negate) goto OUTPUT;
				else        continue;
		}
		if (do_depth) {	/* FOCAL DEPTH */
			z = DECODE(line+29, 5, atof)/100.;
			if (z < zmin || zmax < z)
				if (negate) goto OUTPUT;
				else        continue;
		}
		if (do_mag) {	/* MAGNITUDE */
			mag = DECODE(line+67, 2, atof)/10.;
			if (mag < magmin || magmax < mag)
				if (negate) goto OUTPUT;
				else        continue;
		}
		if (do_close) {	/* DISTANCE TO CLOSEST STATION */
			close = DECODE(line+42, 3, atof);
			if (close < clomin || clomax < close)
				if (negate) goto OUTPUT;
				else        continue;
		}
		if (do_nsta) {	/* NUMBER OF STATIONS */
			nsta = DECODE(line+88, 3, atoi);
			if (nsta < nstmin || nstmax < nsta)
				if (negate) goto OUTPUT;
				else        continue;
		}
		if (do_rms) {	/* RMS RESIDUAL */
			rms = DECODE(line+45, 4, atof)/100;
			if (rms < rmsmin || rmsmax < rms)
				if (negate) goto OUTPUT;
				else        continue;
		}
		if (do_gap) {	/* AZIMUTHAL GAP */
			gap = DECODE(line+39, 3, atof);
			if (gap < gapmin || gapmax < gap)
				if (negate) goto OUTPUT;
				else        continue;
		}
/*??*/
		if (do_qual) {	/* "QUALITY" CODE */
			c = line[78];
			if (!index(quals, (isupper(c) ? tolower(c) : c)))
				if (negate) goto OUTPUT;
				else        continue;
		}
		if (do_poly) {	/* POLYGONAL REGION */
			/* Note: do_poly implies do_lat and do_lon */
			if (inpoly(lat, lon, plat, plon, nvert) == 0)
				if (negate) goto OUTPUT;
				else        continue;
		}
		if (do_delta) {	/* EPICENTRAL DISTANCE */
			/* Note: do_delta implies do_lat and do_lon */
			lat = ggtogc(RAD*lat);
			ct1 = sin(lat);
			st1 = cos(lat);
			cdlon = cos(RAD*lon - phi0);
			cdelt = st0*st1*cdlon + ct0*ct1;
			if (cdelt > cdmin || cdelt < cdmax)
				if (negate) goto OUTPUT;
				else        continue;
			/* sdlon = sin(lon - phi0);					*/
			/* cvrpd(st0*ct1-st1*ct0*cdlon, st1*sdlon, &sdelt, az0);	*/
			/* *delta = atan2(sdelt, cdelt);				*/
			/* cvrpd(st1*ct0-st0*ct1*cdlon, -sdlon*st0, &sdelt, az1);	*/
		}
		if (negate)
			continue;
OUTPUT:
		(void)fputs(line, stdout);
		++selected;
	}
	if (verbose) fprintf (stderr,"%d lines, %d selected\n", lines, selected);
}

#include    <sys/types.h>
#include    <sys/stat.h>
#include    <dirent.h>
#include    <string.h>

static char **p_catalogfiles = NULL;
static int  n_catalogfiles = 0;
static TIME catmin;
static TIME catmax;
static int  catmin_year = 0;
static int  catmin_month = 0;
static int  catmax_year = 0;
static int  catmax_month = 0;

#define	    MAXPATHLEN	1024
#define	DATE_GE(y1,m1,y2,m2)	(y1>y2 | (y1==y2 && m1>=m2))
#define	DATE_LE(y1,m1,y2,m2)	(y1<y2 | (y1==y2 && m1<=m2))

/*  Simple sort of filenames.  Since year and month are first,		*/
/*  we can use standard string sort.					*/
cmp_catalog_filename(pp1, pp2) char **pp1, **pp2; { return (strcmp(*pp1,*pp2)); }

/*  Determine whether we want to search this catalog file.		*/
/*  NOTE:   Decision is based on syntax of the filename.		*/
/*	    This routine will have to change if the filename structure	*/
/*	    is ever changed.						*/
catalog_file_in_range (filename)
    char *filename;
{
    int year, month;
    char *p;
    char *file = ((p = strrchr(filename,'/'))!=NULL) ? ++p : filename;

    if ((sscanf (file, "%d.%d.catalog", &year, &month)) != 2) return (0);
    return ((catmin_year == 0 || DATE_GE(year,month,catmin_year,catmin_month)) &&
	    (catmax_year == 0 || DATE_LE(year,month,catmax_year,catmax_month)));
}

/*  Recursively search the directory for catalog files.			*/
find_catalog_files (dir)
    char *dir;
{
    char filename[MAXPATHLEN];
    DIR *dirp;
    struct dirent *direntp;
    struct stat st;
    int dir_len = strlen(dir)+1;
    strcpy (filename, dir);
    strcat (filename, "/");
    
    if ((dirp=opendir (dir)) == NULL) return (-1);
    while ((direntp=readdir(dirp))!=NULL) {
	/*  Ignore all entries beginning with ".".			*/
	if (direntp->d_name[0]=='.') continue;
	if (dir_len + direntp->d_namlen + 1 >= MAXPATHLEN) continue;
	filename[dir_len] = 0;
	strcat (filename,direntp->d_name, direntp->d_namlen);
	filename[dir_len+direntp->d_namlen] = 0;
	if (stat(filename, &st)!= 0) continue;
	if (S_ISDIR(st.st_mode)) 
	    find_catalog_files(filename);
	else if (S_ISREG(st.st_mode) && catalog_file_in_range (filename)) {
 	    if ((p_catalogfiles[n_catalogfiles]=(char *)malloc(strlen(filename))) == NULL) {
		fprintf (stderr, "Unable to malloc space for catalog filename\n");
		exit(1);
	    }
	    strcpy (p_catalogfiles[n_catalogfiles++],filename);
	}
    }
}

/*  Find the root for the catalog directory, and search for catalog	*/
/*  files that are within our search time interval.			*/
char **
find_catalog (minyear, maxyear)
    TIME minyear, maxyear;
{
    char cuspcatdir[256];
    char filename[1024];
    int	day, hour, min;
    double sec;
    
    catmin = minyear;
    catmax = maxyear;
    getlocation ("CUSPCATDIR",cuspcatdir, 1);
    n_catalogfiles = 0;
    if (p_catalogfiles == NULL) 
	p_catalogfiles = (char **)malloc(sizeof(char *)*(MAXCATALOGFILES+1));
    if (p_catalogfiles == NULL) {
	fprintf (stderr, "unable to malloc table for catalog files\n");
	return(NULL);
    }

    /*	If we have a start and end date, only process through them.	*/
    /*	Otherwise we have to scan the entire catalog.  Either way	*/
    /*	we need to return the list in date-sorted order.  We can assume	*/
    /*	that the file sort will do that.				*/
    
    if (catmin != 0.0) datime (catmin, &catmin_year, &catmin_month, &day, 
			      &hour, &min, &sec, GREGORIAN);
    if (catmax != 0.0) datime (catmax, &catmax_year, &catmax_month, &day,
			      &hour, &min, &sec, GREGORIAN);
    /*  Perform recursive search of directories looking for files.	*/
    find_catalog_files (cuspcatdir);
    p_catalogfiles[n_catalogfiles]=NULL;
    qsort (p_catalogfiles, n_catalogfiles, sizeof(char *), cmp_catalog_filename);
    return (p_catalogfiles);
}

FILE *
ecfopen(file, mode)
    char *file, *mode;
{
    char cmd[MAXPATHLEN+100];
    FILE *fp;
    int n;
    int compressed = ((n=strlen(file))>2) ? (strcmp(file+n-2,".Z")==0) : 0;
    if (compressed) {
	sprintf (cmd, "uncompress -c %s", file);
	fp = popen (cmd, mode);
    }
    else fp = fopen (file, mode);
    if (fp ==NULL) {
	fprintf (stderr, "Unable to open file %s\n", file);
	exit(1);
    }
    return (fp);
}
