.TH QSELECT 1Q "19 March 1986"
.SH NAME
.B qselect
\- filter to select earthquakes satisfying user-defined criteria
.SH SYNTAX
.B qselect
.RI [ option ...]
.RI [ file ...]
.SH DESCRIPTION
.B qselect
reads earthquake catalog cards (``summary cards'') from an on-line
catalog or the specified 
.IR files
(standard input default),
and outputs those whose locations, magnitudes, or other characteristics
agree with criteria specified in the command-line \fIoption\fRs.
.SH OPTIONS
.PP
The following command-line \fIoption\fRs can be used to control the program.
Options are scanned from left to right;
if conflicting \fIoption\fRs are given, the rightmost one takes precedence.
Only enough characters need be given to uniquely identify an \fIoption\fR.
.TP
\fB-help\fR [\fIoption\fR ...]
Print brief explanations of the meanings of the named \fIoption\fRs.
If no \fIoption\fRs are named, descriptions are printed for all \fIoptions\fR.
The explanations are written to the standard error output.
.TP
\fB-latitude\fI min max\fR
Selects earthquakes with latitudes between
.I min
and
.IR max .
These limits are measured in degrees,
and may optionally be followed by one or two fields
specifying minutes and seconds.
If multiple fields are used,
they must be separated from each other by colons \fB:\fR\ .
For example,
.B 37.5
and
.B 37:30
both specify the same latitude -- the first specifies it in
decimal degrees, and the second specifies it in degrees and minutes.
.TP
\fB-longitude\fI min max\fR
Selects earthquakes with longitudes between
.I min
and
.IR max .
These are in the same format as the latitude limits,
described above.
(Note:  West longitudes are negative!)
.TP
\fB-depth\fI min max\fR
Selects earthquakes with focal depths between
.I min
and
.I max
(km).
.TP
\fB-time\fI start end\fR
Selects earthquakes with origin times between the epochs
.I start
and
.I end,
which are given in the form
[cc]yymmddhhmmss .
If the century is not given, the twentieth century is assumed.
The seconds field may contain a decimal point and a fractional part,
but in any case the integral part must be two digits,
even if they are zeros.
An epoch may optionally be followed by an offset consisting of a
.B +
or
.B \-
sign and one or more occurrences of a number followed by a
.BR d ,
.BR h ,
.BR m ,
or
.B s
character
(for days, hours, minutes, or seconds).
For example,
.B +1.5d
and
.B +1d12h
mean the same thing.
This option is also used with the \fB-catalog\fR entry to limit the number of
catalog files to search.
.TP
\fB-magnitude\fI min max\fR
Selects earthquakes with magnitudes between
.I min
and
.I max.
.TP
\fB-closest\fI min max\fR
Selects earthquakes with distances to the closest station between
.I min
and
.I max
(km).
.TP
\fB-nstations\fI min max\fR
Selects earthquakes with the number of stations between
.I min
and
.I max.
.TP
\fB-rms\fI min max\fR
Selects earthquakes with root-mean-squared residuals between
.I min
and
.I max
(s).
.TP
\fB-gap\fI min max\fR
Selects earthquakes with azimuthal gaps between
.I min
and
.I max
(deg).
.TP
\fB-quality\fI letters\fR
Selects earthquakes with specified quality codes (e.g.
.BR acd ).
The difference between upper and lower case letters is ignored.
.TP
\fB-polygon\fI lat lon...\fR
Selects earthquakes within a polygon.
The polygon is specified as three or more pairs of values,
each pair giving the latitude and longitude of a vertex of the polygon.
The format of these latitudes and longitudes is the same as that
for latitude limits, described above.
(Note:  West longitudes are negative!)
.TP
\fB-delta\fI lat lon min max\fR
Selects earthquakes whose distance from the point
.RI ( lat ,
.IR lon )
is between
.I min
and
.IR max .
The format for
.I lat
and
.I lon
is the same as that for latitude limits, described above.
(Note:  West longitudes are negative!)
.I Min
and
.I max
are measured in kilometers.
.TP
\fB-catalog\fR 
Specifies that the on-line catalog should be searched in addition to any
specified input files.  The variable CUSPCATDIR, which can be specified in
either the system-wide LOCATIONS file or as an environment variable,
specifies the directory that contains the monthly catalog files.  Note that
input will NOT be read from stdin when this option is used.
.SH EXAMPLES
.PP
To select earthquakes larger than magnitude 3
from the entire on-line catalog:
.IP
.B
qselect -catalog -mag 3 10
.PP
To select earthquakes within a two-day interval from the catalog
.IP
.B
set t=840301123456.7
.br
.B
qselect -catalog -time $t-2d $t
.PP
To select earthquakes from the catalog using a number of options,
which are stored in the file
.B option_file
(Note the use of grave accents, not to be confused with apostrophes):
.IP
qselect -catalog \`cat option_file\`
.B
.PP
To re-select events that were previously extracted into multiple files ending in
.BR .subset
use one of the following:
.IP
cat \fR*\fB.subset | qselect \fR ...
.br
qselect \fR ... \fB*.subset
.B
.SH NOTES
It is STRONLY suggested that you limit your searches whenever possible with
the \fB-time\fR option whenever you use the \fB-catalog\fR option.
Otherwise, all catalog files on the system will be searched, and this can
take a considerable amount of time.
.SH AUTHOR
Bruce R. Julian, USGS Menlo Park, Calif.
.br
Modified by Doug Neuhauser, UCB Seismographic Station, Calif.
